/* Intel Sandy Bridge -EN/-EP/-EX Memory Controller kernel module
 *
 * This driver supports the memory controllers found on the Intel
 * processor family Sandy Bridge.
 *
 * This file may be distributed under the terms of the
 * GNU General Public License version 2 only.
 *
 * Copyright (c) 2011 by:
 *	 Mauro Carvalho Chehab
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/pci_ids.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/edac.h>
#include <linux/mmzone.h>
#include <linux/smp.h>
#include <linux/cpu.h>
#include <linux/bitmap.h>
#include <linux/math64.h>
#include <asm/processor.h>
#include <asm/mce.h>

#include "edac_core.h"

/* Static vars */
static LIST_HEAD(sbridge_edac_list);
static DEFINE_MUTEX(sbridge_edac_lock);
static int probed;

/*
 * Alter this version for the module when modifications are made
 */
#define SBRIDGE_REVISION    " Ver: 1.1.1 "
#define EDAC_MOD_STR      "sbridge_edac"

/*
 * Debug macros
 */
#define sbridge_printk(level, fmt, arg...)			\
	edac_printk(level, "sbridge", fmt, ##arg)

#define sbridge_mc_printk(mci, level, fmt, arg...)		\
	edac_mc_chipset_printk(mci, level, "sbridge", fmt, ##arg)

/*
 * Get a bit field at register value <v>, from bit <lo> to bit <hi>
 */
#define GET_BITFIELD(v, lo, hi)	\
	(((v) & GENMASK_ULL(hi, lo)) >> (lo))

/* Devices 12 Function 6, Offsets 0x80 to 0xcc */
static const u32 sbridge_dram_rule[] = {
	0x80, 0x88, 0x90, 0x98, 0xa0,
	0xa8, 0xb0, 0xb8, 0xc0, 0xc8,
};

static const u32 ibridge_dram_rule[] = {
	0x60, 0x68, 0x70, 0x78, 0x80,
	0x88, 0x90, 0x98, 0xa0,	0xa8,
	0xb0, 0xb8, 0xc0, 0xc8, 0xd0,
	0xd8, 0xe0, 0xe8, 0xf0, 0xf8,
};

#define SAD_LIMIT(reg)		((GET_BITFIELD(reg, 6, 25) << 26) | 0x3ffffff)
#define DRAM_ATTR(reg)		GET_BITFIELD(reg, 2,  3)
#define INTERLEAVE_MODE(reg)	GET_BITFIELD(reg, 1,  1)
#define DRAM_RULE_ENABLE(reg)	GET_BITFIELD(reg, 0,  0)
#define A7MODE(reg)		GET_BITFIELD(reg, 26, 26)

static char *get_dram_attr(u32 reg)
{
	switch(DRAM_ATTR(reg)) {
		case 0:
			return "DRAM";
		case 1:
			return "MMCFG";
		case 2:
			return "NXM";
		default:
			return "unknown";
	}
}

static const u32 sbridge_interleave_list[] = {
	0x84, 0x8c, 0x94, 0x9c, 0xa4,
	0xac, 0xb4, 0xbc, 0xc4, 0xcc,
};

static const u32 ibridge_interleave_list[] = {
	0x64, 0x6c, 0x74, 0x7c, 0x84,
	0x8c, 0x94, 0x9c, 0xa4, 0xac,
	0xb4, 0xbc, 0xc4, 0xcc, 0xd4,
	0xdc, 0xe4, 0xec, 0xf4, 0xfc,
};

struct interleave_pkg {
	unsigned char start;
	unsigned char end;
};

static const struct interleave_pkg sbridge_interleave_pkg[] = {
	{ 0, 2 },
	{ 3, 5 },
	{ 8, 10 },
	{ 11, 13 },
	{ 16, 18 },
	{ 19, 21 },
	{ 24, 26 },
	{ 27, 29 },
};

static const struct interleave_pkg ibridge_interleave_pkg[] = {
	{ 0, 3 },
	{ 4, 7 },
	{ 8, 11 },
	{ 12, 15 },
	{ 16, 19 },
	{ 20, 23 },
	{ 24, 27 },
	{ 28, 31 },
};

static const u32 ibridge_interleave_hash_list[] = {
    12, 14, 16, 18, 20, 22, 24, 26,
};

static inline int sad_pkg(const struct interleave_pkg *table, u32 reg,
			  int interleave)
{
	return GET_BITFIELD(reg, table[interleave].start,
			    table[interleave].end);
}

/* Devices 12 Function 7 */

#define TOLM		0x80
#define	TOHM		0x84
#define HASWELL_TOLM	0xd0
#define HASWELL_TOHM_0	0xd4
#define HASWELL_TOHM_1	0xd8

#define GET_TOLM(reg)		((GET_BITFIELD(reg, 0,  3) << 28) | 0x3ffffff)
#define GET_TOHM(reg)		((GET_BITFIELD(reg, 0, 20) << 25) | 0x3ffffff)

/* Device 13 Function 6 */

#define SAD_TARGET	0xf0

#define SOURCE_ID(reg)		GET_BITFIELD(reg, 9, 11)

#define SAD_CONTROL	0xf4

/* Device 14 function 0 */

static const u32 tad_dram_rule[] = {
	0x40, 0x44, 0x48, 0x4c,
	0x50, 0x54, 0x58, 0x5c,
	0x60, 0x64, 0x68, 0x6c,
};
#define MAX_TAD	ARRAY_SIZE(tad_dram_rule)

#define TAD_LIMIT(reg)		((GET_BITFIELD(reg, 12, 31) << 26) | 0x3ffffff)
#define TAD_SOCK(reg)		GET_BITFIELD(reg, 10, 11)
#define TAD_CH(reg)		GET_BITFIELD(reg,  8,  9)
#define TAD_TGT3(reg)		GET_BITFIELD(reg,  6,  7)
#define TAD_TGT2(reg)		GET_BITFIELD(reg,  4,  5)
#define TAD_TGT1(reg)		GET_BITFIELD(reg,  2,  3)
#define TAD_TGT0(reg)		GET_BITFIELD(reg,  0,  1)

/* Device 15, function 0 */

#define MCMTR			0x7c

#define IS_ECC_ENABLED(mcmtr)		GET_BITFIELD(mcmtr, 2, 2)
#define IS_LOCKSTEP_ENABLED(mcmtr)	GET_BITFIELD(mcmtr, 1, 1)
#define IS_CLOSE_PG(mcmtr)		GET_BITFIELD(mcmtr, 0, 0)

/* Device 15, function 1 */

#define RASENABLES		0xac
#define IS_MIRROR_ENABLED(reg)		GET_BITFIELD(reg, 0, 0)

/* Device 15, functions 2-5 */

static const int mtr_regs[] = {
	0x80, 0x84, 0x88,
};

#define RANK_DISABLE(mtr)		GET_BITFIELD(mtr, 16, 19)
#define IS_DIMM_PRESENT(mtr)		GET_BITFIELD(mtr, 14, 14)
#define RANK_CNT_BITS(mtr)		GET_BITFIELD(mtr, 12, 13)
#define RANK_WIDTH_BITS(mtr)		GET_BITFIELD(mtr, 2, 4)
#define COL_WIDTH_BITS(mtr)		GET_BITFIELD(mtr, 0, 1)

static const u32 tad_ch_nilv_offset[] = {
	0x90, 0x94, 0x98, 0x9c,
	0xa0, 0xa4, 0xa8, 0xac,
	0xb0, 0xb4, 0xb8, 0xbc,
};
#define CHN_IDX_OFFSET(reg)		GET_BITFIELD(reg, 28, 29)
#define TAD_OFFSET(reg)			(GET_BITFIELD(reg,  6, 25) << 26)

static const u32 rir_way_limit[] = {
	0x108, 0x10c, 0x110, 0x114, 0x118,
};
#define MAX_RIR_RANGES ARRAY_SIZE(rir_way_limit)

#define IS_RIR_VALID(reg)	GET_BITFIELD(reg, 31, 31)
#define RIR_WAY(reg)		GET_BITFIELD(reg, 28, 29)

#define MAX_RIR_WAY	8

static const u32 rir_offset[MAX_RIR_RANGES][MAX_RIR_WAY] = {
	{ 0x120, 0x124, 0x128, 0x12c, 0x130, 0x134, 0x138, 0x13c },
	{ 0x140, 0x144, 0x148, 0x14c, 0x150, 0x154, 0x158, 0x15c },
	{ 0x160, 0x164, 0x168, 0x16c, 0x170, 0x174, 0x178, 0x17c },
	{ 0x180, 0x184, 0x188, 0x18c, 0x190, 0x194, 0x198, 0x19c },
	{ 0x1a0, 0x1a4, 0x1a8, 0x1ac, 0x1b0, 0x1b4, 0x1b8, 0x1bc },
};

/* Device 16, functions 2-7 */

/*
 * FIXME: Implement the error count reads directly
 */

static const u32 correrrcnt[] = {
	0x104, 0x108, 0x10c, 0x110,
};

#define RANK_ODD_OV(reg)		GET_BITFIELD(reg, 31, 31)
#define RANK_ODD_ERR_CNT(reg)		GET_BITFIELD(reg, 16, 30)
#define RANK_EVEN_OV(reg)		GET_BITFIELD(reg, 15, 15)
#define RANK_EVEN_ERR_CNT(reg)		GET_BITFIELD(reg,  0, 14)

static const u32 correrrthrsld[] = {
	0x11c, 0x120, 0x124, 0x128,
};

#define RANK_ODD_ERR_THRSLD(reg)	GET_BITFIELD(reg, 16, 30)
#define RANK_EVEN_ERR_THRSLD(reg)	GET_BITFIELD(reg,  0, 14)


/* Device 17, function 0 */

#define SB_RANK_CFG_A		0x0328

#define IB_RANK_CFG_A		0x0320

/*
 * sbridge structs
 */

#define NUM_CHANNELS		8	/* 2MC per socket, four chan per MC */
#define MAX_DIMMS		3	/* Max DIMMS per channel */
#define CHANNEL_UNSPECIFIED	0xf	/* Intel IA32 SDM 15-14 */

enum type {
	SANDY_BRIDGE,
	IVY_BRIDGE,
	HASWELL,
	BROADWELL,
};

struct sbridge_pvt;
struct sbridge_info {
	enum type	type;
	u32		mcmtr;
	u32		rankcfgr;
	u64		(*get_tolm)(struct sbridge_pvt *pvt);
	u64		(*get_tohm)(struct sbridge_pvt *pvt);
	u64		(*rir_limit)(u32 reg);
	u64     (*rir_rank_offset)(u32 reg);
	u32     (*rir_rank_tgt)(u32 reg);
	u64     (*rir_offset)(u32 reg);
	const u32	*dram_rule;
	const u32	*interleave_list;
	const struct interleave_pkg *interleave_pkg;
	const u32   *interleave_hash_list;
	u8		max_sad;
	u8		max_interleave;
    u8      max_hash;
	u8		(*get_node_id)(struct sbridge_pvt *pvt);
	enum mem_type	(*get_memory_type)(struct sbridge_pvt *pvt);
	struct pci_dev	*pci_vtd;
};

struct sbridge_channel {
	u32		ranks;
	u32		dimms;
};

struct pci_id_descr {
	int			dev_id;
	int			optional;
};

struct pci_id_table {
	const struct pci_id_descr	*descr;
	int				n_devs;
};

struct sbridge_dev {
	struct list_head	list;
	u8			bus, mc;
	u8			node_id, source_id;
	struct pci_dev		**pdev;
	int			n_devs;
	struct mem_ctl_info	*mci;
};

struct sbridge_pvt {
	struct pci_dev		*pci_ta, *pci_ddrio, *pci_ras;
	struct pci_dev		*pci_sad0, *pci_sad1;
	struct pci_dev		*pci_ha0, *pci_ha1;
	struct pci_dev      *pci_ha0_perfmon, *pci_ha1_perfmon;
	struct pci_dev		*pci_br0, *pci_br1;
	struct pci_dev		*pci_ha1_ta;
	struct pci_dev		*pci_tad[NUM_CHANNELS];
	struct pci_dev      *pci_err[NUM_CHANNELS];

	struct sbridge_dev	*sbridge_dev;

	struct sbridge_info	info;
	struct sbridge_channel	channel[NUM_CHANNELS];

	/* Memory type detection */
	bool			is_mirrored, is_lockstep, is_close_pg;

	/* Fifo double buffers */
	struct mce		mce_entry[MCE_LOG_LEN];
	struct mce		mce_outentry[MCE_LOG_LEN];

	/* Fifo in/out counters */
	unsigned		mce_in, mce_out;

	/* Count indicator to show errors not got */
	unsigned		mce_overrun;

	/* Memory description */
	u64			tolm, tohm;
};

static DEFINE_MUTEX(sbridge_range_list_lock);
static LIST_HEAD(sbridge_range_list);

struct sbridge_range {
    u64         start;
    u64         end;
    int         socket[8];
    int         channel[8][8];
    int         rank[8][8][8];
    struct list_head    list;
};

#define PCI_DESCR(device_id, opt)	\
	.dev_id = (device_id),		\
	.optional = opt

static const struct pci_id_descr pci_dev_descr_sbridge[] = {
		/* Processor Home Agent */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_HA0, 0)	},

		/* Memory controller */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TA, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_RAS, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD1, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD2, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD3, 0)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_DDRIO, 1)	},

		/* System Address Decoder */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_SAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_SAD1, 0)	},

		/* Broadcast Registers */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_SBRIDGE_BR, 0)		},
};

#define PCI_ID_TABLE_ENTRY(A) { .descr=A, .n_devs = ARRAY_SIZE(A) }
static const struct pci_id_table pci_dev_descr_sbridge_table[] = {
	PCI_ID_TABLE_ENTRY(pci_dev_descr_sbridge),
	{0,}			/* 0 terminated list. */
};

/* This changes depending if 1HA or 2HA:
 * 1HA:
 *	0x0eb8 (17.0) is DDRIO0
 * 2HA:
 *	0x0ebc (17.4) is DDRIO0
 */
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_1HA_DDRIO0	0x0eb8
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_2HA_DDRIO0	0x0ebc

/* pci ids */
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0		0x0ea0
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA		0x0ea8
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_RAS		0x0e71
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD0	0x0eaa
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD1	0x0eab
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD2	0x0eac
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD3	0x0ead
#define PCI_DEVICE_ID_INTEL_IBRIDGE_SAD			0x0ec8
#define PCI_DEVICE_ID_INTEL_IBRIDGE_BR0			0x0ec9
#define PCI_DEVICE_ID_INTEL_IBRIDGE_BR1			0x0eca
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1		0x0e60
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TA		0x0e68
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_RAS		0x0e79
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD0	0x0e6a
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD1	0x0e6b
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD2	0x0e6c
#define PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD3	0x0e6d

static const struct pci_id_descr pci_dev_descr_ibridge[] = {
		/* Processor Home Agent */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0, 0)		},

		/* Memory controller */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA, 0)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_RAS, 0)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD1, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD2, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD3, 0)	},

		/* System Address Decoder */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_SAD, 0)			},

		/* Broadcast Registers */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_BR0, 1)			},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_BR1, 0)			},

		/* Optional, mode 2HA */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1, 1)		},
#if 0
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TA, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_RAS, 1)	},
#endif
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD0, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD1, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD2, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD3, 1)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_1HA_DDRIO0, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_2HA_DDRIO0, 1)	},
};

static const struct pci_id_table pci_dev_descr_ibridge_table[] = {
	PCI_ID_TABLE_ENTRY(pci_dev_descr_ibridge),
	{0,}			/* 0 terminated list. */
};

/* Haswell support */
/* EN processor:
 *	- 1 IMC
 *	- 3 DDR3 channels, 2 DPC per channel
 * EP processor:
 *	- 1 or 2 IMC
 *	- 4 DDR4 channels, 3 DPC per channel
 * EP 4S processor:
 *	- 2 IMC
 *	- 4 DDR4 channels, 3 DPC per channel
 * EX processor:
 *	- 2 IMC
 *	- each IMC interfaces with a SMI 2 channel
 *	- each SMI channel interfaces with a scalable memory buffer
 *	- each scalable memory buffer supports 4 DDR3/DDR4 channels, 3 DPC
 */
#define HASWELL_DDRCRCLKCONTROLS 0xa10 /* Ditto on Broadwell */
#define HASWELL_HASYSDEFEATURE2 0x84
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_VTD_MISC 0x2f28
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0	0x2fa0
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1	0x2f60
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TA	0x2fa8
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_THERMAL 0x2f71
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TA	0x2f68
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_THERMAL 0x2f79
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD0 0x2ffc
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD1 0x2ffd
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD0 0x2faa
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD1 0x2fab
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD2 0x2fac
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD3 0x2fad
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD0 0x2f6a
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD1 0x2f6b
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD2 0x2f6c
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD3 0x2f6d
#define PCI_DEVICE_ID_INTEL_HASWELL_IMC_DDRIO0 0x2fbd
static const struct pci_id_descr pci_dev_descr_haswell[] = {
	/* first item must be the HA */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0, 0)		},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD1, 0)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1, 1)		},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TA, 0)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_THERMAL, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD1, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD2, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD3, 1)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_DDRIO0, 1)		},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TA, 1)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_THERMAL, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD0, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD1, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD2, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD3, 1)	},
};

static const struct pci_id_table pci_dev_descr_haswell_table[] = {
	PCI_ID_TABLE_ENTRY(pci_dev_descr_haswell),
	{0,}			/* 0 terminated list. */
};

/*
 * Broadwell support
 *
 * DE processor:
 *	- 1 IMC
 *	- 2 DDR3 channels, 2 DPC per channel
 * EP processor:
 *	- 1 or 2 IMC
 *	- 4 DDR4 channels, 3 DPC per channel
 * EP 4S processor:
 *	- 2 IMC
 *	- 4 DDR4 channels, 3 DPC per channel
 * EX processor:
 *	- 2 IMC
 *	- each IMC interfaces with a SMI 2 channel
 *	- each SMI channel interfaces with a scalable memory buffer
 *	- each scalable memory buffer supports 4 DDR3/DDR4 channels, 3 DPC
 */
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_VTD_MISC 0x6f28
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0	0x6fa0
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_PERFMON 0x6f30
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1	0x6f60
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_PERFMON 0x6f38
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TA	0x6fa8
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_THERMAL 0x6f71
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TA	0x6f68
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_THERMAL 0x6f79
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD0 0x6ffc
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD1 0x6ffd
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD0 0x6faa
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD1 0x6fab
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD2 0x6fac
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD3 0x6fad
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR0 0x6fb2
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR1 0x6fb3
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR2 0x6fb6
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR3 0x6fb7
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD0 0x6f6a
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD1 0x6f6b
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD2 0x6f6c
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD3 0x6f6d
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_DDRIO0 0x6faf
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR0 0x6fd2
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR1 0x6fd3
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR2 0x6fd6
#define PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR3 0x6fd7

static const struct pci_id_descr pci_dev_descr_broadwell[] = {
	/* first item must be the HA */
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0, 0)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_PERFMON, 0)   },

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD1, 0)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1, 1)		},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_PERFMON, 1)   },

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TA, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_THERMAL, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD0, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD1, 0)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD2, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD3, 1)	},
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR0, 0)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR1, 0)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR2, 1)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR3, 1)  },

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_DDRIO0, 1)	},

	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TA, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_THERMAL, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD0, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD1, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD2, 1)	},
	{ PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD3, 1)	},
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD3, 1)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR0, 1)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR1, 1)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR2, 1)  },
    { PCI_DESCR(PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR3, 1)  },
};

static const struct pci_id_table pci_dev_descr_broadwell_table[] = {
	PCI_ID_TABLE_ENTRY(pci_dev_descr_broadwell),
	{0,}			/* 0 terminated list. */
};

/*
 *	pci_device_id	table for which devices we are looking for
 */
static const struct pci_device_id sbridge_pci_tbl[] = {
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_HA0)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0)},
	{0,}			/* 0 terminated list. */
};

/*
 * Error injection support
 */
static DEFINE_SPINLOCK(g_error_inject_lock);

#define REG_HASYSDEFEATURE           0x80
#define BIT_DEMAND_SCRUB             9

#define REG_MC5_STATUS_SHADOW_LOW    0x68
#define REG_MC5_STATUS_SHADOW_HIGH   0x6C

#define REG_RSP_FUNC_ADDR_MATCH_LO   0x1C0
#define REG_RSP_FUNC_ADDR_MATCH_HI   0x1C4
#define BIT_RSP_FUNC_ADDR_MATCH_EN   11
#define REG_RSP_FUNC_ADDR_MASK_LO    0x1C8
#define REG_RSP_FUNC_ADDR_MASK_HI    0x1CC

#define REG_RSP_FUNC_CRC_ERR_INJ_DEV0_XOR_MSK    0x200
#define REG_RSP_FUNC_CRC_ERR_INJ_DEV1_XOR_MSK    0x204
#define REG_RSP_FUNC_CRC_ERR_INJ_EXTRA           0x208

int g_inject_status = 0;

enum inject_fail_type {
    ECC_INJECT_LOCK = 1,
    ADDR_TRANSLATION_FAIL,
    IOREMAP_FAIL,
    ECC_INJECT_FAIL,
    COMPARE_DATA_FAIL,
    ENABLE_ECC_INJECT_FAIL,
    INCORRECT_ECC_COUNT,
    NO_ECC_COUNT,
};

static DEFINE_SPINLOCK(g_rank_addr_detect_lock);
struct rank_addr_detect {
    u64 address;
    u64 rank_offset;
};

static DEFINE_SPINLOCK(g_system_addr_detect_lock);
struct system_addr_detect {
    int target_socket;
    int target_ha;
    int target_channel;
    int target_rank;
    u64 rank_offset;
    u64 system_addr;
};

struct rank_addr_detect g_rank_addr_detect;
struct system_addr_detect g_sys_addr_detect;

/****************************************************************************
			Ancillary status routines
 ****************************************************************************/

static inline int numrank(enum type type, u32 mtr)
{
	int ranks = (1 << RANK_CNT_BITS(mtr));
	int max = 4;

	if (type == HASWELL || type == BROADWELL)
		max = 8;

	if (ranks > max) {
		edac_dbg(0, "Invalid number of ranks: %d (max = %i) raw value = %x (%04x)\n",
			 ranks, max, (unsigned int)RANK_CNT_BITS(mtr), mtr);
		return -EINVAL;
	}

	return ranks;
}

static inline int numrow(u32 mtr)
{
	int rows = (RANK_WIDTH_BITS(mtr) + 12);

	if (rows < 13 || rows > 18) {
		edac_dbg(0, "Invalid number of rows: %d (should be between 14 and 17) raw value = %x (%04x)\n",
			 rows, (unsigned int)RANK_WIDTH_BITS(mtr), mtr);
		return -EINVAL;
	}

	return 1 << rows;
}

static inline int numcol(u32 mtr)
{
	int cols = (COL_WIDTH_BITS(mtr) + 10);

	if (cols > 12) {
		edac_dbg(0, "Invalid number of cols: %d (max = 4) raw value = %x (%04x)\n",
			 cols, (unsigned int)COL_WIDTH_BITS(mtr), mtr);
		return -EINVAL;
	}

	return 1 << cols;
}

static struct sbridge_dev *get_sbridge_dev(u8 bus)
{
	struct sbridge_dev *sbridge_dev;

	list_for_each_entry(sbridge_dev, &sbridge_edac_list, list) {
		if (sbridge_dev->bus == bus)
			return sbridge_dev;
	}

	return NULL;
}

static struct sbridge_dev *alloc_sbridge_dev(u8 bus,
					   const struct pci_id_table *table)
{
	struct sbridge_dev *sbridge_dev;

	sbridge_dev = kzalloc(sizeof(*sbridge_dev), GFP_KERNEL);
	if (!sbridge_dev)
		return NULL;

	sbridge_dev->pdev = kzalloc(sizeof(*sbridge_dev->pdev) * table->n_devs,
				   GFP_KERNEL);
	if (!sbridge_dev->pdev) {
		kfree(sbridge_dev);
		return NULL;
	}

	sbridge_dev->bus = bus;
	sbridge_dev->n_devs = table->n_devs;
	list_add_tail(&sbridge_dev->list, &sbridge_edac_list);

	return sbridge_dev;
}

static void free_sbridge_dev(struct sbridge_dev *sbridge_dev)
{
	list_del(&sbridge_dev->list);
	kfree(sbridge_dev->pdev);
	kfree(sbridge_dev);
}

static u64 sbridge_get_tolm(struct sbridge_pvt *pvt)
{
	u32 reg;

	/* Address range is 32:28 */
	pci_read_config_dword(pvt->pci_sad1, TOLM, &reg);
	return GET_TOLM(reg);
}

static u64 sbridge_get_tohm(struct sbridge_pvt *pvt)
{
	u32 reg;

	pci_read_config_dword(pvt->pci_sad1, TOHM, &reg);
	return GET_TOHM(reg);
}

static u64 ibridge_get_tolm(struct sbridge_pvt *pvt)
{
	u32 reg;

	pci_read_config_dword(pvt->pci_br1, TOLM, &reg);

	return GET_TOLM(reg);
}

static u64 ibridge_get_tohm(struct sbridge_pvt *pvt)
{
	u32 reg;

	pci_read_config_dword(pvt->pci_br1, TOHM, &reg);

	return GET_TOHM(reg);
}

static u64 rir_limit(u32 reg)
{
	return ((u64)GET_BITFIELD(reg,  1, 10) << 29) | 0x1fffffff;
}

static u32 rir_rank_tgt(u32 reg)
{
    return GET_BITFIELD(reg, 16, 19);
}

static u64 rir_rank_offset(u32 reg)
{
    return ((u64)GET_BITFIELD(reg,  2, 14) << 26);
}

static enum mem_type get_memory_type(struct sbridge_pvt *pvt)
{
	u32 reg;
	enum mem_type mtype;

	if (pvt->pci_ddrio) {
		pci_read_config_dword(pvt->pci_ddrio, pvt->info.rankcfgr,
				      &reg);
		if (GET_BITFIELD(reg, 11, 11))
			/* FIXME: Can also be LRDIMM */
			mtype = MEM_RDDR3;
		else
			mtype = MEM_DDR3;
	} else
		mtype = MEM_UNKNOWN;

	return mtype;
}

static enum mem_type haswell_get_memory_type(struct sbridge_pvt *pvt)
{
	u32 reg;
	bool registered = false;
	enum mem_type mtype = MEM_UNKNOWN;

	if (!pvt->pci_ddrio)
		goto out;

	pci_read_config_dword(pvt->pci_ddrio,
			      HASWELL_DDRCRCLKCONTROLS, &reg);
	/* Is_Rdimm */
	if (GET_BITFIELD(reg, 16, 16))
		registered = true;

	pci_read_config_dword(pvt->pci_ta, MCMTR, &reg);
	if (GET_BITFIELD(reg, 14, 14)) {
		if (registered)
			mtype = MEM_RDDR4;
		else
			mtype = MEM_DDR4;
	} else {
		if (registered)
			mtype = MEM_RDDR3;
		else
			mtype = MEM_DDR3;
	}

out:
	return mtype;
}

static u8 get_node_id(struct sbridge_pvt *pvt)
{
	u32 reg;
	pci_read_config_dword(pvt->pci_br0, SAD_CONTROL, &reg);
	return GET_BITFIELD(reg, 0, 2);
}

static u8 haswell_get_node_id(struct sbridge_pvt *pvt)
{
	u32 reg;

	pci_read_config_dword(pvt->pci_sad1, SAD_CONTROL, &reg);
	return GET_BITFIELD(reg, 0, 3);
}

static u64 haswell_get_tolm(struct sbridge_pvt *pvt)
{
	u32 reg;

	pci_read_config_dword(pvt->info.pci_vtd, HASWELL_TOLM, &reg);
	return (GET_BITFIELD(reg, 26, 31) << 26) | 0x3ffffff;
}

static u64 haswell_get_tohm(struct sbridge_pvt *pvt)
{
	u64 rc;
	u32 reg;

	pci_read_config_dword(pvt->info.pci_vtd, HASWELL_TOHM_0, &reg);
	rc = GET_BITFIELD(reg, 26, 31);
	pci_read_config_dword(pvt->info.pci_vtd, HASWELL_TOHM_1, &reg);
	rc = ((reg << 6) | rc) << 26;

	return rc | 0x1ffffff;
}

static u64 haswell_rir_limit(u32 reg)
{
	return (((u64)GET_BITFIELD(reg,  1, 11) + 1) << 29) - 1;
}

static u64 haswell_rir_rank_offset(u32 reg)
{
    return ((u64)GET_BITFIELD(reg,  2, 15) << 26);
}

static u32 broadwell_rir_rank_tgt(u32 reg)
{
    return GET_BITFIELD(reg, 20, 23);
}

static inline u8 sad_pkg_socket(u8 pkg)
{
	/* on Ivy Bridge, nodeID is SASS, where A is HA and S is node id */
	return ((pkg >> 3) << 2) | (pkg & 0x3);
}

static inline u8 sad_pkg_ha(u8 pkg)
{
	return (pkg >> 2) & 0x1;
}

static inline u8 sad_socket_ha_to_pkg(u8 socket, u8 ha)
{
    return ((ha << 2) | (((socket & 0x4) >> 2) << 3) | (socket & 0x3));
}

/****************************************************************************
			Memory check routines
 ****************************************************************************/
static struct pci_dev *get_pdev_same_bus(u8 bus, u32 id)
{
	struct pci_dev *pdev = NULL;

	do {
		pdev = pci_get_device(PCI_VENDOR_ID_INTEL, id, pdev);
		if (pdev && pdev->bus->number == bus)
			break;
	} while (pdev);

	return pdev;
}

/**
 * check_if_ecc_is_active() - Checks if ECC is active
 * @bus:	Device bus
 * @type:	Memory controller type
 * returns: 0 in case ECC is active, -ENODEV if it can't be determined or
 *	    disabled
 */
static int check_if_ecc_is_active(const u8 bus, enum type type)
{
	struct pci_dev *pdev = NULL;
	u32 mcmtr, id;

	switch (type) {
	case IVY_BRIDGE:
		id = PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA;
		break;
	case HASWELL:
		id = PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TA;
		break;
	case SANDY_BRIDGE:
		id = PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TA;
		break;
	case BROADWELL:
		id = PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TA;
		break;
	default:
		return -ENODEV;
	}

	pdev = get_pdev_same_bus(bus, id);
	if (!pdev) {
		sbridge_printk(KERN_ERR, "Couldn't find PCI device "
					"%04x:%04x! on bus %02d\n",
					PCI_VENDOR_ID_INTEL, id, bus);
		return -ENODEV;
	}

	pci_read_config_dword(pdev, MCMTR, &mcmtr);
	if (!IS_ECC_ENABLED(mcmtr)) {
		sbridge_printk(KERN_ERR, "ECC is disabled. Aborting\n");
		return -ENODEV;
	}
	return 0;
}

static int get_dimm_config(struct mem_ctl_info *mci)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct dimm_info *dimm;
	unsigned i, j, banks, ranks, rows, cols, npages;
	u64 size;
	u32 reg;
	enum edac_type mode;
	enum mem_type mtype;

	if (pvt->info.type == HASWELL || pvt->info.type == BROADWELL)
		pci_read_config_dword(pvt->pci_sad1, SAD_TARGET, &reg);
	else
		pci_read_config_dword(pvt->pci_br0, SAD_TARGET, &reg);

	pvt->sbridge_dev->source_id = SOURCE_ID(reg);

	pvt->sbridge_dev->node_id = pvt->info.get_node_id(pvt);
	edac_dbg(0, "mc#%d: Node ID: %d, source ID: %d\n",
		 pvt->sbridge_dev->mc,
		 pvt->sbridge_dev->node_id,
		 pvt->sbridge_dev->source_id);

	pci_read_config_dword(pvt->pci_ras, RASENABLES, &reg);
	if (IS_MIRROR_ENABLED(reg)) {
		edac_dbg(0, "Memory mirror is enabled\n");
		pvt->is_mirrored = true;
	} else {
		edac_dbg(0, "Memory mirror is disabled\n");
		pvt->is_mirrored = false;
	}

	pci_read_config_dword(pvt->pci_ta, MCMTR, &pvt->info.mcmtr);
	if (IS_LOCKSTEP_ENABLED(pvt->info.mcmtr)) {
		edac_dbg(0, "Lockstep is enabled\n");
		mode = EDAC_S8ECD8ED;
		pvt->is_lockstep = true;
	} else {
		edac_dbg(0, "Lockstep is disabled\n");
		mode = EDAC_S4ECD4ED;
		pvt->is_lockstep = false;
	}
	if (IS_CLOSE_PG(pvt->info.mcmtr)) {
		edac_dbg(0, "address map is on closed page mode\n");
		pvt->is_close_pg = true;
	} else {
		edac_dbg(0, "address map is on open page mode\n");
		pvt->is_close_pg = false;
	}

	mtype = pvt->info.get_memory_type(pvt);
	if (mtype == MEM_RDDR3 || mtype == MEM_RDDR4)
		edac_dbg(0, "Memory is registered\n");
	else if (mtype == MEM_UNKNOWN)
		edac_dbg(0, "Cannot determine memory type\n");
	else
		edac_dbg(0, "Memory is unregistered\n");

	if (mtype == MEM_DDR4 || mtype == MEM_RDDR4)
		banks = 16;
	else
		banks = 8;

	for (i = 0; i < NUM_CHANNELS; i++) {
		u32 mtr;

		if (!pvt->pci_tad[i])
			continue;
		for (j = 0; j < ARRAY_SIZE(mtr_regs); j++) {
			dimm = EDAC_DIMM_PTR(mci->layers, mci->dimms, mci->n_layers,
				       i, j, 0);
			pci_read_config_dword(pvt->pci_tad[i],
					      mtr_regs[j], &mtr);
			edac_dbg(4, "Channel #%d  MTR%d = %x\n", i, j, mtr);
			if (IS_DIMM_PRESENT(mtr)) {
				pvt->channel[i].dimms++;

				ranks = numrank(pvt->info.type, mtr);
				rows = numrow(mtr);
				cols = numcol(mtr);

				size = ((u64)rows * cols * banks * ranks) >> (20 - 3);
				npages = MiB_TO_PAGES(size);

				edac_dbg(0, "mc#%d: ha %d channel %d, dimm %d, %lld Mb (%d pages) bank: %d, rank: %d, row: %#x, col: %#x\n",
					 pvt->sbridge_dev->mc, i/4, i%4, j,
					 size, npages,
					 banks, ranks, rows, cols);

				dimm->nr_pages = npages;
				dimm->grain = 32;
				switch (banks) {
				case 16:
					dimm->dtype = DEV_X16;
					break;
				case 8:
					dimm->dtype = DEV_X8;
					break;
				case 4:
					dimm->dtype = DEV_X4;
					break;
				}
				dimm->mtype = mtype;
				dimm->edac_mode = mode;
				snprintf(dimm->label, sizeof(dimm->label),
					 "CPU_SrcID#%u_Ha#%u_Chan#%u_DIMM#%u",
					 pvt->sbridge_dev->source_id, i/4, i%4, j);
			}
		}
	}

	return 0;
}

static void get_memory_layout(const struct mem_ctl_info *mci)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	int i, j, k, n_sads, n_tads, sad_interl;
	u32 reg;
	u64 limit, prv = 0;
	u64 tmp_mb;
	u32 gb, mb;
	u32 rir_way;

	/*
	 * Step 1) Get TOLM/TOHM ranges
	 */

	pvt->tolm = pvt->info.get_tolm(pvt);
	tmp_mb = (1 + pvt->tolm) >> 20;

	gb = div_u64_rem(tmp_mb, 1024, &mb);
	edac_dbg(0, "TOLM: %u.%03u GB (0x%016Lx)\n",
		gb, (mb*1000)/1024, (u64)pvt->tolm);

	/* Address range is already 45:25 */
	pvt->tohm = pvt->info.get_tohm(pvt);
	tmp_mb = (1 + pvt->tohm) >> 20;

	gb = div_u64_rem(tmp_mb, 1024, &mb);
	edac_dbg(0, "TOHM: %u.%03u GB (0x%016Lx)\n",
		gb, (mb*1000)/1024, (u64)pvt->tohm);

	/*
	 * Step 2) Get SAD range and SAD Interleave list
	 * TAD registers contain the interleave wayness. However, it
	 * seems simpler to just discover it indirectly, with the
	 * algorithm bellow.
	 */
	prv = 0;
	for (n_sads = 0; n_sads < pvt->info.max_sad; n_sads++) {
		/* SAD_LIMIT Address range is 45:26 */
		pci_read_config_dword(pvt->pci_sad0, pvt->info.dram_rule[n_sads],
				      &reg);
		limit = SAD_LIMIT(reg);

		if (!DRAM_RULE_ENABLE(reg))
			continue;

		if (limit <= prv)
			break;

		tmp_mb = (limit + 1) >> 20;
		gb = div_u64_rem(tmp_mb, 1024, &mb);
		edac_dbg(0, "SAD#%d %s up to %u.%03u GB (0x%016Lx) Interleave: %s reg=0x%08x\n",
			 n_sads,
			 get_dram_attr(reg),
			 gb, (mb*1000)/1024,
			 ((u64)tmp_mb) << 20L,
			 INTERLEAVE_MODE(reg) ? "8:6" : "[8:6]XOR[18:16]",
			 reg);
		prv = limit;

		pci_read_config_dword(pvt->pci_sad0, pvt->info.interleave_list[n_sads],
				      &reg);
		sad_interl = sad_pkg(pvt->info.interleave_pkg, reg, 0);
		for (j = 0; j < 8; j++) {
			u32 pkg = sad_pkg(pvt->info.interleave_pkg, reg, j);
			if (j > 0 && sad_interl == pkg)
				break;

			edac_dbg(0, "SAD#%d, interleave #%d: %d\n",
				 n_sads, j, pkg);
		}
	}

	/*
	 * Step 3) Get TAD range
	 */
	prv = 0;
	for (n_tads = 0; n_tads < MAX_TAD; n_tads++) {
		pci_read_config_dword(pvt->pci_ha0, tad_dram_rule[n_tads],
				      &reg);
		limit = TAD_LIMIT(reg);
		if (limit <= prv)
			break;
		tmp_mb = (limit + 1) >> 20;

		gb = div_u64_rem(tmp_mb, 1024, &mb);
		edac_dbg(0, "TAD#%d: up to %u.%03u GB (0x%016Lx), socket interleave %d, memory interleave %d, TGT: %d, %d, %d, %d, reg=0x%08x\n",
			 n_tads, gb, (mb*1000)/1024,
			 ((u64)tmp_mb) << 20L,
			 (u32)TAD_SOCK(reg),
			 (u32)TAD_CH(reg),
			 (u32)TAD_TGT0(reg),
			 (u32)TAD_TGT1(reg),
			 (u32)TAD_TGT2(reg),
			 (u32)TAD_TGT3(reg),
			 reg);
		prv = limit;
	}

	/*
	 * Step 4) Get TAD offsets, per each channel
	 */
	for (i = 0; i < NUM_CHANNELS; i++) {
		if (!pvt->channel[i].dimms)
			continue;
		for (j = 0; j < n_tads; j++) {
			pci_read_config_dword(pvt->pci_tad[i],
					      tad_ch_nilv_offset[j],
					      &reg);
			tmp_mb = TAD_OFFSET(reg) >> 20;
			gb = div_u64_rem(tmp_mb, 1024, &mb);
			edac_dbg(0, "TAD CH#%d, offset #%d: %u.%03u GB (0x%016Lx), reg=0x%08x\n",
				 i, j,
				 gb, (mb*1000)/1024,
				 ((u64)tmp_mb) << 20L,
				 reg);
		}
	}

	/*
	 * Step 6) Get RIR Wayness/Limit, per each channel
	 */
	for (i = 0; i < NUM_CHANNELS; i++) {
		if (!pvt->channel[i].dimms)
			continue;
		for (j = 0; j < MAX_RIR_RANGES; j++) {
			pci_read_config_dword(pvt->pci_tad[i],
					      rir_way_limit[j],
					      &reg);

			if (!IS_RIR_VALID(reg))
				continue;

			tmp_mb = pvt->info.rir_limit(reg) >> 20;
			rir_way = 1 << RIR_WAY(reg);
			gb = div_u64_rem(tmp_mb, 1024, &mb);
			edac_dbg(0, "CH#%d RIR#%d, limit: %u.%03u GB (0x%016Lx), way: %d, reg=0x%08x\n",
				 i, j,
				 gb, (mb*1000)/1024,
				 ((u64)tmp_mb) << 20L,
				 rir_way,
				 reg);

			for (k = 0; k < rir_way; k++) {
				pci_read_config_dword(pvt->pci_tad[i],
						      rir_offset[j][k],
						      &reg);
				tmp_mb = pvt->info.rir_rank_offset(reg) >> 20;

				gb = div_u64_rem(tmp_mb, 1024, &mb);
				edac_dbg(0, "CH#%d RIR#%d INTL#%d, offset %u.%03u GB (0x%016Lx), tgt: %d, reg=0x%08x\n",
					 i, j, k,
					 gb, (mb*1000)/1024,
					 ((u64)tmp_mb) << 20L,
					 (u32)pvt->info.rir_rank_tgt(reg),
					 reg);
			}
		}
	}
}

static struct mem_ctl_info *get_mci_for_node_id(u8 node_id)
{
	struct sbridge_dev *sbridge_dev;

	list_for_each_entry(sbridge_dev, &sbridge_edac_list, list) {
		if (sbridge_dev->node_id == node_id)
			return sbridge_dev->mci;
	}
	return NULL;
}

static int get_memory_error_data(struct mem_ctl_info *mci,
				 u64 addr,
				 u8 *socket, u8 *ha,
				 long *channel_mask,
				 u8 *rank,
				 char **area_type, char *msg)
{
	struct mem_ctl_info	*new_mci;
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct pci_dev		*pci_ha;
	int			n_rir, n_sads, n_tads, sad_way, sck_xch, n_hash;
	int			sad_interl, idx, base_ch;
	int			interleave_mode, shiftup = 0, hashen = 0;
	unsigned		sad_interleave[pvt->info.max_interleave];
	u32			reg, dram_rule;
	u8			ch_way, sck_way, pkg, sad_ha = 0, ch_add = 0;
	u32			tad_offset;
	u32			rir_way;
	u32			mb, gb;
	u64			ch_addr, offset, limit = 0, prv = 0;


	/*
	 * Step 0) Check if the address is at special memory ranges
	 * The check bellow is probably enough to fill all cases where
	 * the error is not inside a memory, except for the legacy
	 * range (e. g. VGA addresses). It is unlikely, however, that the
	 * memory controller would generate an error on that range.
	 */
	if ((addr > (u64) pvt->tolm) && (addr < (1LL << 32))) {
		sprintf(msg, "Error at TOLM area, on addr 0x%08Lx", addr);
		return -EINVAL;
	}
	if (addr >= (u64)pvt->tohm) {
		sprintf(msg, "Error at MMIOH area, on addr 0x%016Lx", addr);
		return -EINVAL;
	}

	/*
	 * Step 1) Get socket
	 */
	for (n_sads = 0; n_sads < pvt->info.max_sad; n_sads++) {
		pci_read_config_dword(pvt->pci_sad0, pvt->info.dram_rule[n_sads],
				      &reg);

		if (!DRAM_RULE_ENABLE(reg))
			continue;

		limit = SAD_LIMIT(reg);
		if (limit <= prv) {
			sprintf(msg, "Can't discover the memory socket");
			return -EINVAL;
		}
		if  (addr <= limit)
			break;
		prv = limit;
	}
	if (n_sads == pvt->info.max_sad) {
		sprintf(msg, "Can't discover the memory socket");
		return -EINVAL;
	}
	dram_rule = reg;
	*area_type = get_dram_attr(dram_rule);
	interleave_mode = INTERLEAVE_MODE(dram_rule);

	pci_read_config_dword(pvt->pci_sad0, pvt->info.interleave_list[n_sads],
			      &reg);

	if (pvt->info.type == SANDY_BRIDGE) {
		sad_interl = sad_pkg(pvt->info.interleave_pkg, reg, 0);
		for (sad_way = 0; sad_way < 8; sad_way++) {
			u32 pkg = sad_pkg(pvt->info.interleave_pkg, reg, sad_way);
			if (sad_way > 0 && sad_interl == pkg)
				break;
			sad_interleave[sad_way] = pkg;
			edac_dbg(0, "SAD interleave #%d: %d\n",
				 sad_way, sad_interleave[sad_way]);
		}
		edac_dbg(0, "mc#%d: Error detected on SAD#%d: address 0x%016Lx < 0x%016Lx, Interleave [%d:6]%s\n",
			 pvt->sbridge_dev->mc,
			 n_sads,
			 addr,
			 limit,
			 sad_way + 7,
			 !interleave_mode ? "" : "XOR[18:16]");
		if (interleave_mode)
			idx = ((addr >> 6) ^ (addr >> 16)) & 7;
		else
			idx = (addr >> 6) & 7;
		switch (sad_way) {
		case 1:
			idx = 0;
			break;
		case 2:
			idx = idx & 1;
			break;
		case 4:
			idx = idx & 3;
			break;
		case 8:
			break;
		default:
			sprintf(msg, "Can't discover socket interleave");
			return -EINVAL;
		}
		*socket = sad_interleave[idx];
		edac_dbg(0, "SAD interleave index: %d (wayness %d) = CPU socket %d\n",
			 idx, sad_way, *socket);
	} else if (pvt->info.type == HASWELL || pvt->info.type == BROADWELL) {
		int bits, a7mode = A7MODE(dram_rule);

		if (a7mode) {
			/* A7 mode swaps P9 with P6 */
			bits = GET_BITFIELD(addr, 7, 8) << 1;
			bits |= GET_BITFIELD(addr, 9, 9);
		} else
			bits = GET_BITFIELD(addr, 6, 8);

		if (interleave_mode == 0) {
			/* interleave mode will XOR {8,7,6} with {18,17,16} */
			idx = GET_BITFIELD(addr, 16, 18);
			idx ^= bits;
		} else
			idx = bits;

		pkg = sad_pkg(pvt->info.interleave_pkg, reg, idx);
		*socket = sad_pkg_socket(pkg);
		sad_ha = sad_pkg_ha(pkg);
		if (sad_ha)
			ch_add = 4;

		if (a7mode) {
			/* MCChanShiftUpEnable */
			pci_read_config_dword(pvt->pci_ha0,
					      HASWELL_HASYSDEFEATURE2, &reg);
			shiftup = GET_BITFIELD(reg, 22, 22);
		}

		hashen = GET_BITFIELD(reg, 21, 21);

		edac_dbg(0, "SAD interleave package: %d = CPU socket %d, HA %i, shiftup: %i\n",
			 idx, *socket, sad_ha, shiftup);
	} else {
		/* Ivy Bridge's SAD mode doesn't support XOR interleave mode */
		idx = (addr >> 6) & 7;
		pkg = sad_pkg(pvt->info.interleave_pkg, reg, idx);
		*socket = sad_pkg_socket(pkg);
		sad_ha = sad_pkg_ha(pkg);
		if (sad_ha)
			ch_add = 4;
		edac_dbg(0, "SAD interleave package: %d = CPU socket %d, HA %d\n",
			 idx, *socket, sad_ha);
	}

	*ha = sad_ha;

	/*
	 * Move to the proper node structure, in order to access the
	 * right PCI registers
	 */
	new_mci = get_mci_for_node_id(*socket);
	if (!new_mci) {
		sprintf(msg, "Struct for socket #%u wasn't initialized",
			*socket);
		return -EINVAL;
	}
	mci = new_mci;
	pvt = mci->pvt_info;

	/*
	 * Step 2) Get memory channel
	 */
	prv = 0;
	if (pvt->info.type == SANDY_BRIDGE)
		pci_ha = pvt->pci_ha0;
	else {
		if (sad_ha)
			pci_ha = pvt->pci_ha1;
		else
			pci_ha = pvt->pci_ha0;
	}
	for (n_tads = 0; n_tads < MAX_TAD; n_tads++) {
		pci_read_config_dword(pci_ha, tad_dram_rule[n_tads], &reg);
		limit = TAD_LIMIT(reg);
		if (limit <= prv) {
			sprintf(msg, "Can't discover the memory channel");
			return -EINVAL;
		}
		if  (addr <= limit)
			break;
		prv = limit;
	}
	if (n_tads == MAX_TAD) {
		sprintf(msg, "Can't discover the memory channel");
		return -EINVAL;
	}

	ch_way = TAD_CH(reg) + 1;
	sck_way = TAD_SOCK(reg);

	if (ch_way == 3)
		idx = addr >> 6;
	else
		idx = (addr >> (6 + sck_way + shiftup)) & 0x3;

    if (pvt->info.interleave_hash_list && hashen) {
        for (n_hash = 0; n_hash < pvt->info.max_hash; n_hash++)
            idx ^= GET_BITFIELD(addr, pvt->info.interleave_hash_list[n_hash], pvt->info.interleave_hash_list[n_hash] + 1);
    }

	idx = idx % ch_way;

	/*
	 * FIXME: Shouldn't we use CHN_IDX_OFFSET() here, when ch_way == 3 ???
	 */
	switch (idx) {
	case 0:
		base_ch = TAD_TGT0(reg);
		break;
	case 1:
		base_ch = TAD_TGT1(reg);
		break;
	case 2:
		base_ch = TAD_TGT2(reg);
		break;
	case 3:
		base_ch = TAD_TGT3(reg);
		break;
	default:
		sprintf(msg, "Can't discover the TAD target");
		return -EINVAL;
	}
	*channel_mask = 1 << base_ch;

	pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
				tad_ch_nilv_offset[n_tads],
				&tad_offset);

	if (pvt->is_mirrored) {
		*channel_mask |= 1 << ((base_ch + 2) % 4);
		switch(ch_way) {
		case 2:
		case 4:
			sck_xch = 1 << sck_way * (ch_way >> 1);
			break;
		default:
			sprintf(msg, "Invalid mirror set. Can't decode addr");
			return -EINVAL;
		}
	} else
		sck_xch = (1 << sck_way) * ch_way;

	if (pvt->is_lockstep)
		*channel_mask |= 1 << ((base_ch + 1) % 4);

	offset = TAD_OFFSET(tad_offset);

	edac_dbg(0, "TAD#%d: address 0x%016Lx < 0x%016Lx, socket interleave %d, channel interleave %d (offset 0x%08Lx), index %d, base ch: %d, ch mask: 0x%02lx\n",
		 n_tads,
		 addr,
		 limit,
		 (u32)TAD_SOCK(reg),
		 ch_way,
		 offset,
		 idx,
		 base_ch,
		 *channel_mask);

	/* Calculate channel address */
	/* Remove the TAD offset */

	if (offset > addr) {
		sprintf(msg, "Can't calculate ch addr: TAD offset 0x%08Lx is too high for addr 0x%08Lx!",
			offset, addr);
		return -EINVAL;
	}

    ch_addr = addr - offset;
    ch_addr >>= (6 + shiftup);
    ch_addr /= sck_xch;
    ch_addr <<= (6 + shiftup);
    ch_addr |= addr & ((1 << (6 + shiftup)) - 1);

	/*
	 * Step 3) Decode rank
	 */
	for (n_rir = 0; n_rir < MAX_RIR_RANGES; n_rir++) {
		pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
				      rir_way_limit[n_rir],
				      &reg);

		if (!IS_RIR_VALID(reg))
			continue;

		limit = pvt->info.rir_limit(reg);
		gb = div_u64_rem(limit >> 20, 1024, &mb);
		edac_dbg(0, "RIR#%d, limit: %u.%03u GB (0x%016Lx), way: %d\n",
			 n_rir,
			 gb, (mb*1000)/1024,
			 limit,
			 1 << RIR_WAY(reg));
		if  (ch_addr <= limit)
			break;
	}
	if (n_rir == MAX_RIR_RANGES) {
		sprintf(msg, "Can't discover the memory rank for ch addr 0x%08Lx",
			ch_addr);
		return -EINVAL;
	}
	rir_way = RIR_WAY(reg);

	if (pvt->is_close_pg)
		idx = (ch_addr >> 6);
	else
		idx = (ch_addr >> 13);	/* FIXME: Datasheet says to shift by 15 */
	idx %= 1 << rir_way;

	pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
			      rir_offset[n_rir][idx],
			      &reg);
	*rank = pvt->info.rir_rank_tgt(reg);

	edac_dbg(0, "RIR#%d: channel address 0x%08Lx < 0x%08Lx, RIR interleave %d, index %d\n",
		 n_rir,
		 ch_addr,
		 limit,
		 rir_way,
		 idx);

	return 0;
}

static int parse_system_addr(struct mem_ctl_info *mci,
                u64 addr,
                u8 *socket, u8 *ha,
                long *channel_mask,
                u8 *rank,
                u64 *rank_addr)
{
    struct mem_ctl_info *new_mci;
    struct sbridge_pvt  *pvt = mci->pvt_info;
    struct pci_dev      *pci_ha;
    int         n_rir, n_sads, n_tads, sad_way, sck_xch, n_hash;
    int         sad_interl, idx, base_ch;
    int         interleave_mode, shiftup = 0, hashen = 0;
    unsigned    sad_interleave[pvt->info.max_interleave];
    u32         reg, dram_rule;
    u8          ch_way, sck_way, pkg, sad_ha = 0, ch_add = 0;
    u32         tad_offset;
    u32         rir_way;
    u64         ch_addr, offset, limit = 0, prv = 0;

    if ((addr > (u64) pvt->tolm) && (addr < (1LL << 32))) {
        sbridge_printk(KERN_ERR, "parse_system_addr: the given address 0x%llx is in tolm - 0x100000000\n", addr);
        return -EINVAL;
    }
    if (addr >= (u64)pvt->tohm) {
        sbridge_printk(KERN_ERR, "parse_system_addr: the given address 0x%llx exceeds tohm\n", addr);
        return -EINVAL;
    }

    /*
     * Step 1) Get socket
     */
    for (n_sads = 0; n_sads < pvt->info.max_sad; n_sads++) {
        pci_read_config_dword(pvt->pci_sad0, pvt->info.dram_rule[n_sads],
                      &reg);

        if (!DRAM_RULE_ENABLE(reg))
            continue;

        limit = SAD_LIMIT(reg);
        if (limit <= prv) {
            sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the memory socket");
            return -EINVAL;
        }
        if  (addr <= limit)
            break;
        prv = limit;
    }
    if (n_sads == pvt->info.max_sad) {
        sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the memory socket");
        return -EINVAL;
    }
    dram_rule = reg;

    interleave_mode = INTERLEAVE_MODE(dram_rule);

    pci_read_config_dword(pvt->pci_sad0, pvt->info.interleave_list[n_sads],
                  &reg);

    if (pvt->info.type == SANDY_BRIDGE) {
        sad_interl = sad_pkg(pvt->info.interleave_pkg, reg, 0);
        for (sad_way = 0; sad_way < 8; sad_way++) {
            u32 pkg = sad_pkg(pvt->info.interleave_pkg, reg, sad_way);
            if (sad_way > 0 && sad_interl == pkg)
                break;
            sad_interleave[sad_way] = pkg;

        }

        if (interleave_mode)
            idx = ((addr >> 6) ^ (addr >> 16)) & 7;
        else
            idx = (addr >> 6) & 7;
        switch (sad_way) {
        case 1:
            idx = 0;
            break;
        case 2:
            idx = idx & 1;
            break;
        case 4:
            idx = idx & 3;
            break;
        case 8:
            break;
        default:
            sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover socket interleave\n");
            return -EINVAL;
        }
        *socket = sad_interleave[idx];
    } else if (pvt->info.type == HASWELL || pvt->info.type == BROADWELL) {
        int bits, a7mode = A7MODE(dram_rule);

        if (a7mode) {
            /* A7 mode swaps P9 with P6 */
            bits = GET_BITFIELD(addr, 7, 8) << 1;
            bits |= GET_BITFIELD(addr, 9, 9);
        } else
            bits = GET_BITFIELD(addr, 6, 8);

        if (interleave_mode == 0) {
            /* interleave mode will XOR {8,7,6} with {18,17,16} */
            idx = GET_BITFIELD(addr, 16, 18);
            idx ^= bits;
        } else
            idx = bits;

        pkg = sad_pkg(pvt->info.interleave_pkg, reg, idx);
        *socket = sad_pkg_socket(pkg);
        sad_ha = sad_pkg_ha(pkg);
        if (sad_ha)
            ch_add = 4;

        if (a7mode) {
            /* MCChanShiftUpEnable */
            pci_read_config_dword(pvt->pci_ha0,
                          HASWELL_HASYSDEFEATURE2, &reg);
            shiftup = GET_BITFIELD(reg, 22, 22);
        }

        hashen = GET_BITFIELD(reg, 21, 21);
    } else {
        /* Ivy Bridge's SAD mode doesn't support XOR interleave mode */
        idx = (addr >> 6) & 7;
        pkg = sad_pkg(pvt->info.interleave_pkg, reg, idx);
        *socket = sad_pkg_socket(pkg);
        sad_ha = sad_pkg_ha(pkg);
        if (sad_ha)
            ch_add = 4;
    }

    *ha = sad_ha;

    /*
     * Move to the proper node structure, in order to access the
     * right PCI registers
     */
    new_mci = get_mci_for_node_id(*socket);
    if (!new_mci) {
        sbridge_printk(KERN_ERR, "parse_system_addr: Can not find mci for socket %d\n", *socket);
        return -EINVAL;
    }
    mci = new_mci;
    pvt = mci->pvt_info;

    /*
     * Step 2) Get memory channel
     */
    prv = 0;
    if (pvt->info.type == SANDY_BRIDGE)
        pci_ha = pvt->pci_ha0;
    else {
        if (sad_ha)
            pci_ha = pvt->pci_ha1;
        else
            pci_ha = pvt->pci_ha0;
    }
    for (n_tads = 0; n_tads < MAX_TAD; n_tads++) {
        pci_read_config_dword(pci_ha, tad_dram_rule[n_tads], &reg);
        limit = TAD_LIMIT(reg);
        if (limit <= prv) {
            sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the memory channel");
            return -EINVAL;
        }
        if  (addr <= limit)
            break;
        prv = limit;
    }
    if (n_tads == MAX_TAD) {
        sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the memory channel");
        return -EINVAL;
    }

    ch_way = TAD_CH(reg) + 1;
    sck_way = TAD_SOCK(reg);

    if (ch_way == 3)
        idx = addr >> 6;
    else
        idx = (addr >> (6 + sck_way + shiftup)) & 0x3;

    if (pvt->info.interleave_hash_list && hashen) {
        for (n_hash = 0; n_hash < pvt->info.max_hash; n_hash++)
            idx ^= GET_BITFIELD(addr, pvt->info.interleave_hash_list[n_hash], pvt->info.interleave_hash_list[n_hash] + 1);
    }

    idx = idx % ch_way;

    /*
     * FIXME: Shouldn't we use CHN_IDX_OFFSET() here, when ch_way == 3 ???
     */
    switch (idx) {
    case 0:
        base_ch = TAD_TGT0(reg);
        break;
    case 1:
        base_ch = TAD_TGT1(reg);
        break;
    case 2:
        base_ch = TAD_TGT2(reg);
        break;
    case 3:
        base_ch = TAD_TGT3(reg);
        break;
    default:
        sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the TAD target");
        return -EINVAL;
    }
    *channel_mask = 1 << base_ch;

    pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
                tad_ch_nilv_offset[n_tads],
                &tad_offset);

    if (pvt->is_mirrored) {
        *channel_mask |= 1 << ((base_ch + 2) % 4);
        switch(ch_way) {
        case 2:
        case 4:
            sck_xch = 1 << sck_way * (ch_way >> 1);
            break;
        default:
            sbridge_printk(KERN_ERR, "parse_system_addr: Invalid mirror set. Can't decode addr");
            return -EINVAL;
        }
    } else
        sck_xch = (1 << sck_way) * ch_way;

    if (pvt->is_lockstep)
        *channel_mask |= 1 << ((base_ch + 1) % 4);

    offset = TAD_OFFSET(tad_offset);

    /* Calculate channel address */
    /* Remove the TAD offset */

    if (offset > addr) {
        sbridge_printk(KERN_ERR, "parse_system_addr: Can't calculate ch addr: TAD offset 0x%08Lx is too high for addr 0x%08Lx!",
            offset, addr);
        return -EINVAL;
    }

    ch_addr = addr - offset;
    ch_addr >>= (6 + shiftup);
    ch_addr /= sck_xch;
    ch_addr <<= (6 + shiftup);
    ch_addr |= addr & ((1 << (6 + shiftup)) - 1);

    /*
     * Step 3) Decode rank
     */
    for (n_rir = 0; n_rir < MAX_RIR_RANGES; n_rir++) {
        pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
                      rir_way_limit[n_rir],
                      &reg);

        if (!IS_RIR_VALID(reg))
            continue;

        limit = pvt->info.rir_limit(reg);

        if  (ch_addr <= limit)
            break;
    }
    if (n_rir == MAX_RIR_RANGES) {
        sbridge_printk(KERN_ERR, "parse_system_addr: Can't discover the memory rank for ch addr 0x%08Lx",
            ch_addr);
        return -EINVAL;
    }
    rir_way = RIR_WAY(reg);

    if (pvt->is_close_pg)
        idx = (ch_addr >> 6);
    else
        idx = (ch_addr >> 13);
    idx %= 1 << rir_way;

    pci_read_config_dword(pvt->pci_tad[ch_add + base_ch],
                  rir_offset[n_rir][idx],
                  &reg);
    *rank = pvt->info.rir_rank_tgt(reg);

    *rank_addr = (ch_addr >> 13) / (1 << rir_way);
    *rank_addr = (*rank_addr << 13) | GET_BITFIELD(ch_addr, 0, 12);
    *rank_addr -= pvt->info.rir_rank_offset(reg);

    return 0;
}

static int get_system_addr(u8 socket, u8 ha,
                u8 channel, u8 rank,
                u64 rank_offset,
                u64 *addr)
{
    struct mem_ctl_info *mci;
    struct sbridge_pvt  *pvt;
    struct pci_dev      *pci_ha;
    int         n_rir, n_sads, n_tads, sck_xch, n_hash;
    int         idx, target_ch;
    int         interleave_mode, a7mode = 0, shiftup = 0, hashen = 0;
    int         shift_bits, lower_bits, a181716_bits, missing_bits;
    u32         reg;
    u8          rir_way, ch_way, sck_way, pkg, ch_add = 0;
    u32         tad_offset;
    u64         system_addr = 0, temp_addr = 0, ch_addr = 0, offset, limit = 0, prv = 0;
    int         match;
    u32         match_mb, match_lb;

    mci = get_mci_for_node_id(socket);
    if (mci == NULL)
    {
        sbridge_printk(KERN_ERR, "get_system_addr: No mci for socket %d\n", socket);
        return -EINVAL;
    }

    pvt = mci->pvt_info;

    if (ha) {
        pci_ha = pvt->pci_ha1;
        ch_add = 4;
    } else {
        pci_ha = pvt->pci_ha0;
        ch_add = 0;
    }

    prv = 0;
    // Search target RIR
    for (n_rir = 0; n_rir < MAX_RIR_RANGES; n_rir++) {
        pci_read_config_dword(pvt->pci_tad[ch_add + channel],
                      rir_way_limit[n_rir],
                      &reg);

        if (!IS_RIR_VALID(reg))
            continue;

        limit = pvt->info.rir_limit(reg);

        if (limit == prv)
        {
            sbridge_printk(KERN_ERR, "get_system_addr: Can not find the target RIR range for rank %d\n", rank);
            return -EINVAL;
        }

        rir_way = 1 << RIR_WAY(reg);

        // Search target rank
        for (idx = 0; idx < rir_way; idx++) {
            pci_read_config_dword(pvt->pci_tad[ch_add + channel],
                              rir_offset[n_rir][idx],
                              &reg);

            if (rank == pvt->info.rir_rank_tgt(reg)) {
                offset = pvt->info.rir_rank_offset(reg);
                break;
            }
        }

        if (idx == rir_way)
            continue;

        // Get channel address
        temp_addr = offset + rank_offset;

        if (pvt->is_close_pg) {
            ch_addr = GET_BITFIELD(temp_addr, 0, 5);
            temp_addr = (temp_addr >> 6) * rir_way;
            temp_addr |= idx;
            ch_addr |= (temp_addr << 6);
        } else {
            ch_addr = GET_BITFIELD(temp_addr, 0, 12);
            temp_addr = (temp_addr >> 13) * rir_way;
            temp_addr |= idx;
            ch_addr |= (temp_addr << 13);
        }

        if ((prv < ch_addr) && (ch_addr <= limit)) {
            break;
        }

        prv = limit;
    }

    pci_read_config_dword(pvt->pci_ha0,
                          HASWELL_HASYSDEFEATURE2, &reg);
    shiftup = GET_BITFIELD(reg, 22, 22);
    hashen = GET_BITFIELD(reg, 21, 21);

    prv = 0;
    // Search target TAD
    for (n_tads = 0; n_tads < MAX_TAD; n_tads++) {
        pci_read_config_dword(pci_ha, tad_dram_rule[n_tads], &reg);
        limit = TAD_LIMIT(reg);

        if (limit <= prv) {
            sbridge_printk(KERN_ERR, "get_system_addr: Can not find the target TAD for channel %d\n", channel);
            return -EINVAL;
        }

        sck_way = TAD_SOCK(reg);
        ch_way = TAD_CH(reg) + 1;
        sck_xch = (1 << sck_way) * ch_way;

        shift_bits = shiftup + 6;
        // Search for target channel
        for (idx = 0; idx < ch_way; idx++) {
            switch (idx) {
                case 0:
                    target_ch = TAD_TGT0(reg);
                    break;
                case 1:
                    target_ch = TAD_TGT1(reg);
                    break;
                case 2:
                    target_ch = TAD_TGT2(reg);
                    break;
                case 3:
                    target_ch = TAD_TGT3(reg);
                    break;
            }

            if (channel == target_ch)
                break;
        }
        if (idx == ch_way)
        {
            prv = limit;
            continue;
        }

        pci_read_config_dword(pvt->pci_tad[ch_add + channel],
                        tad_ch_nilv_offset[n_tads],
                        &tad_offset);

        offset = TAD_OFFSET(tad_offset);

        // Get system address
        temp_addr = ch_addr >> shift_bits;
        temp_addr *= sck_xch;
        temp_addr <<= shift_bits;
        temp_addr += offset;
        temp_addr |= GET_BITFIELD(ch_addr, 0, shift_bits - 1);

        if (pvt->info.interleave_hash_list && hashen) {
            for (n_hash = 0; n_hash < pvt->info.max_hash; n_hash++)
                idx ^= GET_BITFIELD(temp_addr, pvt->info.interleave_hash_list[n_hash], pvt->info.interleave_hash_list[n_hash] + 1);
        }

        idx %= ch_way;

        temp_addr += idx << (sck_way + shift_bits);

        if ((prv < temp_addr) && (temp_addr <= limit))
        {
            system_addr = temp_addr;
            break;
        }

        prv = limit;
    }

    // Get target package
    pkg = sad_socket_ha_to_pkg(socket, ha);

    prv = 0;
    match = 0;
    // Search target SAD
    for (n_sads = 0; n_sads < pvt->info.max_sad; n_sads++) {
        pci_read_config_dword(pvt->pci_sad0, pvt->info.dram_rule[n_sads],
                      &reg);

        if (!DRAM_RULE_ENABLE(reg))
            continue;

        limit = SAD_LIMIT(reg);
        if (limit <= prv) {
            sbridge_printk(KERN_ERR, "get_system_addr: Can not find the target SAD for the socket %d and ha %d\n", socket, ha);
            return -EINVAL;
        }

        if ((prv < system_addr) && (system_addr <= limit))
        {
            a7mode = A7MODE(reg);
            interleave_mode = INTERLEAVE_MODE(reg);

            if (a7mode) {
                lower_bits = GET_BITFIELD(system_addr, 7, 8) << 1;
                lower_bits |= GET_BITFIELD(system_addr, 9, 9);
            } else
                lower_bits = GET_BITFIELD(system_addr, 6, 8);

            pci_read_config_dword(pvt->pci_sad0, pvt->info.interleave_list[n_sads],
                              &reg);

            // Get system address according to SAD's configurations
            if (interleave_mode) {
                for (idx = 0; idx < 8; idx++) {
                    if ((lower_bits & idx) != lower_bits)
                        continue;

                    if (pkg == sad_pkg(pvt->info.interleave_pkg, reg, idx)) {
                        if (a7mode) {
                            system_addr |= (idx & 0x1) << 9;
                            system_addr |= (idx >> 1) << 7;
                        } else
                            system_addr |= idx << 6;

                        match = 1;
                        break;
                    }
                }
            } else {
                for (idx = 0; idx < 8; idx++) {
                    if (pkg == sad_pkg(pvt->info.interleave_pkg, reg, idx)) {
                        a181716_bits = ((system_addr >> 16) & 0x7);
                        missing_bits = idx ^ a181716_bits;

                        if (a7mode) {
                            match_mb = missing_bits & 0x1;
                            match_mb = (((missing_bits >> 1) >> sck_way) << 1) | match_mb;
                            match_lb = lower_bits & 0x1;
                            match_lb = (((lower_bits >> 1) >> sck_way) << 1) | match_lb;
                        } else {
                            match_mb = missing_bits >> sck_way;
                            match_lb = lower_bits >> sck_way;
                        }

                        if (match_lb == match_mb) {
                            if (a7mode) {
                                system_addr &= ~(0x7ULL << 7);
                                system_addr |= ((missing_bits >> 1) << 7);
                                system_addr |= (missing_bits & 0x1) << 9;
                            } else {
                                system_addr |= (missing_bits << 6);
                            }
                            match = 1;
                            break;
                        }
                    }
                }
            }

            if (match)
                break;
        }
        prv = limit;
    }

    *addr = system_addr;
    return 0;
}

/****************************************************************************
	Device initialization routines: put/get, init/exit
 ****************************************************************************/

/*
 *	sbridge_put_all_devices	'put' all the devices that we have
 *				reserved via 'get'
 */
static void sbridge_put_devices(struct sbridge_dev *sbridge_dev)
{
	int i;

	edac_dbg(0, "\n");
	for (i = 0; i < sbridge_dev->n_devs; i++) {
		struct pci_dev *pdev = sbridge_dev->pdev[i];
		if (!pdev)
			continue;
		edac_dbg(0, "Removing dev %02x:%02x.%d\n",
			 pdev->bus->number,
			 PCI_SLOT(pdev->devfn), PCI_FUNC(pdev->devfn));
		pci_dev_put(pdev);
	}
}

static void sbridge_put_all_devices(void)
{
	struct sbridge_dev *sbridge_dev, *tmp;

	list_for_each_entry_safe(sbridge_dev, tmp, &sbridge_edac_list, list) {
		sbridge_put_devices(sbridge_dev);
		free_sbridge_dev(sbridge_dev);
	}
}

static int sbridge_get_onedevice(struct pci_dev **prev,
				 u8 *num_mc,
				 const struct pci_id_table *table,
				 const unsigned devno)
{
	struct sbridge_dev *sbridge_dev;
	const struct pci_id_descr *dev_descr = &table->descr[devno];
	struct pci_dev *pdev = NULL;
	u8 bus = 0;

	sbridge_printk(KERN_DEBUG,
		"Seeking for: PCI ID %04x:%04x\n",
		PCI_VENDOR_ID_INTEL, dev_descr->dev_id);

	pdev = pci_get_device(PCI_VENDOR_ID_INTEL,
			      dev_descr->dev_id, *prev);

	if (!pdev) {
		if (*prev) {
			*prev = pdev;
			return 0;
		}

		if (dev_descr->optional)
			return 0;

		/* if the HA wasn't found */
		if (devno == 0)
			return -ENODEV;

		sbridge_printk(KERN_INFO,
			"Device not found: %04x:%04x\n",
			PCI_VENDOR_ID_INTEL, dev_descr->dev_id);

		/* End of list, leave */
		return -ENODEV;
	}
	bus = pdev->bus->number;

	sbridge_dev = get_sbridge_dev(bus);
	if (!sbridge_dev) {
		sbridge_dev = alloc_sbridge_dev(bus, table);
		if (!sbridge_dev) {
			pci_dev_put(pdev);
			return -ENOMEM;
		}
		(*num_mc)++;
	}

	if (sbridge_dev->pdev[devno]) {
		sbridge_printk(KERN_ERR,
			"Duplicated device for %04x:%04x\n",
			PCI_VENDOR_ID_INTEL, dev_descr->dev_id);
		pci_dev_put(pdev);
		return -ENODEV;
	}

	sbridge_dev->pdev[devno] = pdev;

	/* Be sure that the device is enabled */
	if (unlikely(pci_enable_device(pdev) < 0)) {
		sbridge_printk(KERN_ERR,
			"Couldn't enable %04x:%04x\n",
			PCI_VENDOR_ID_INTEL, dev_descr->dev_id);
		return -ENODEV;
	}

	edac_dbg(0, "Detected %04x:%04x\n",
		 PCI_VENDOR_ID_INTEL, dev_descr->dev_id);

	/*
	 * As stated on drivers/pci/search.c, the reference count for
	 * @from is always decremented if it is not %NULL. So, as we need
	 * to get all devices up to null, we need to do a get for the device
	 */
	pci_dev_get(pdev);

	*prev = pdev;

	return 0;
}

/*
 * sbridge_get_all_devices - Find and perform 'get' operation on the MCH's
 *			     devices we want to reference for this driver.
 * @num_mc: pointer to the memory controllers count, to be incremented in case
 *	    of success.
 * @table: model specific table
 *
 * returns 0 in case of success or error code
 */
static int sbridge_get_all_devices(u8 *num_mc,
				   const struct pci_id_table *table)
{
	int i, rc;
	struct pci_dev *pdev = NULL;

	while (table && table->descr) {
		for (i = 0; i < table->n_devs; i++) {
			pdev = NULL;
			do {
				rc = sbridge_get_onedevice(&pdev, num_mc,
							   table, i);
				if (rc < 0) {
					if (i == 0) {
						i = table->n_devs;
						break;
					}
					sbridge_put_all_devices();
					return -ENODEV;
				}
			} while (pdev);
		}
		table++;
	}

	return 0;
}

static int sbridge_mci_bind_devs(struct mem_ctl_info *mci,
				 struct sbridge_dev *sbridge_dev)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct pci_dev *pdev;
	int i;

	for (i = 0; i < sbridge_dev->n_devs; i++) {
		pdev = sbridge_dev->pdev[i];
		if (!pdev)
			continue;

		switch (pdev->device) {
		case PCI_DEVICE_ID_INTEL_SBRIDGE_SAD0:
			pvt->pci_sad0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_SAD1:
			pvt->pci_sad1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_BR:
			pvt->pci_br0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_HA0:
			pvt->pci_ha0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TA:
			pvt->pci_ta = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_RAS:
			pvt->pci_ras = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD0:
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD1:
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD2:
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_TAD0;
			pvt->pci_tad[id] = pdev;
		}
			break;
		case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_DDRIO:
			pvt->pci_ddrio = pdev;
			break;
		default:
			goto error;
		}

		edac_dbg(0, "Associated PCI %02x:%02x, bus %d with dev = %p\n",
			 pdev->vendor, pdev->device,
			 sbridge_dev->bus,
			 pdev);
	}

	/* Check if everything were registered */
	if (!pvt->pci_sad0 || !pvt->pci_sad1 || !pvt->pci_ha0 ||
	    !pvt-> pci_tad || !pvt->pci_ras  || !pvt->pci_ta)
		goto enodev;

	for (i = 0; i < NUM_CHANNELS; i++) {
		if (!pvt->pci_tad[i])
			goto enodev;
	}
	return 0;

enodev:
	sbridge_printk(KERN_ERR, "Some needed devices are missing\n");
	return -ENODEV;

error:
	sbridge_printk(KERN_ERR, "Unexpected device %02x:%02x\n",
		       PCI_VENDOR_ID_INTEL, pdev->device);
	return -EINVAL;
}

static int ibridge_mci_bind_devs(struct mem_ctl_info *mci,
				 struct sbridge_dev *sbridge_dev)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct pci_dev *pdev;
	u8 saw_chan_mask = 0;
	int i;

	for (i = 0; i < sbridge_dev->n_devs; i++) {
		pdev = sbridge_dev->pdev[i];
		if (!pdev)
			continue;

		switch (pdev->device) {
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0:
			pvt->pci_ha0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA:
			pvt->pci_ta = pdev;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_RAS:
			pvt->pci_ras = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD0:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD1:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD2:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TAD0;
			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_2HA_DDRIO0:
			pvt->pci_ddrio = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_1HA_DDRIO0:
			pvt->pci_ddrio = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_SAD:
			pvt->pci_sad0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_BR0:
			pvt->pci_br0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_BR1:
			pvt->pci_br1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1:
			pvt->pci_ha1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD0:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD1:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD2:
		case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA1_TAD0 + 4;
			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		default:
			goto error;
		}

		edac_dbg(0, "Associated PCI %02x.%02d.%d with dev = %p\n",
			 sbridge_dev->bus,
			 PCI_SLOT(pdev->devfn), PCI_FUNC(pdev->devfn),
			 pdev);
	}

	/* Check if everything were registered */
	if (!pvt->pci_sad0 || !pvt->pci_ha0 || !pvt->pci_br0 ||
	    !pvt->pci_br1 || !pvt->pci_tad || !pvt->pci_ras  ||
	    !pvt->pci_ta)
		goto enodev;

	if (saw_chan_mask != 0x0f && /* -EN */
	    saw_chan_mask != 0x33 && /* -EP */
	    saw_chan_mask != 0xff)   /* -EX */
		goto enodev;
	return 0;

enodev:
	sbridge_printk(KERN_ERR, "Some needed devices are missing\n");
	return -ENODEV;

error:
	sbridge_printk(KERN_ERR,
		       "Unexpected device %02x:%02x\n", PCI_VENDOR_ID_INTEL,
			pdev->device);
	return -EINVAL;
}

static int haswell_mci_bind_devs(struct mem_ctl_info *mci,
				 struct sbridge_dev *sbridge_dev)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct pci_dev *pdev;
	u8 saw_chan_mask = 0;
	int i;

	/* there's only one device per system; not tied to any bus */
	if (pvt->info.pci_vtd == NULL)
		/* result will be checked later */
		pvt->info.pci_vtd = pci_get_device(PCI_VENDOR_ID_INTEL,
						   PCI_DEVICE_ID_INTEL_HASWELL_IMC_VTD_MISC,
						   NULL);

	for (i = 0; i < sbridge_dev->n_devs; i++) {
		pdev = sbridge_dev->pdev[i];
		if (!pdev)
			continue;

		switch (pdev->device) {
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD0:
			pvt->pci_sad0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_CBO_SAD1:
			pvt->pci_sad1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0:
			pvt->pci_ha0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TA:
			pvt->pci_ta = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_THERMAL:
			pvt->pci_ras = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD0:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD1:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD2:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0_TAD0;

			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD0:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD1:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD2:
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TAD0 + 4;

			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_DDRIO0:
			pvt->pci_ddrio = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1:
			pvt->pci_ha1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA1_TA:
			pvt->pci_ha1_ta = pdev;
			break;
		default:
			break;
		}

		edac_dbg(0, "Associated PCI %02x.%02d.%d with dev = %p\n",
			 sbridge_dev->bus,
			 PCI_SLOT(pdev->devfn), PCI_FUNC(pdev->devfn),
			 pdev);
	}

	/* Check if everything were registered */
	if (!pvt->pci_sad0 || !pvt->pci_ha0 || !pvt->pci_sad1 ||
	    !pvt->pci_ras  || !pvt->pci_ta || !pvt->info.pci_vtd)
		goto enodev;

	if (saw_chan_mask != 0x0f && /* -EN */
	    saw_chan_mask != 0x33 && /* -EP */
	    saw_chan_mask != 0xff)   /* -EX */
		goto enodev;
	return 0;

enodev:
	sbridge_printk(KERN_ERR, "Some needed devices are missing\n");
	return -ENODEV;
}

static int broadwell_mci_bind_devs(struct mem_ctl_info *mci,
				 struct sbridge_dev *sbridge_dev)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	struct pci_dev *pdev;
	u8 saw_chan_mask = 0;
	int i;

	/* there's only one device per system; not tied to any bus */
	if (pvt->info.pci_vtd == NULL)
		/* result will be checked later */
		pvt->info.pci_vtd = pci_get_device(PCI_VENDOR_ID_INTEL,
						   PCI_DEVICE_ID_INTEL_BROADWELL_IMC_VTD_MISC,
						   NULL);

	for (i = 0; i < sbridge_dev->n_devs; i++) {
		pdev = sbridge_dev->pdev[i];
		if (!pdev)
			continue;

		switch (pdev->device) {
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD0:
			pvt->pci_sad0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_CBO_SAD1:
			pvt->pci_sad1 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0:
			pvt->pci_ha0 = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_PERFMON:
		    pvt->pci_ha0_perfmon = pdev;
		    break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TA:
			pvt->pci_ta = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_THERMAL:
			pvt->pci_ras = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD0:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD1:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD2:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_TAD0;
			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD0:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD1:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD2:
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD3:
		{
			int id = pdev->device - PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TAD0 + 4;
			pvt->pci_tad[id] = pdev;
			saw_chan_mask |= 1 << id;
		}
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_DDRIO0:
			pvt->pci_ddrio = pdev;
			break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1:
			pvt->pci_ha1 = pdev;
			break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_PERFMON:
            pvt->pci_ha1_perfmon = pdev;
            break;
		case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_TA:
			pvt->pci_ha1_ta = pdev;
			break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR0:
            pvt->pci_err[0] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR1:
            pvt->pci_err[1] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR2:
            pvt->pci_err[2] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0_ERR3:
            pvt->pci_err[3] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR0:
            pvt->pci_err[4] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR1:
            pvt->pci_err[5] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR2:
            pvt->pci_err[6] = pdev;
            break;
        case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA1_ERR3:
            pvt->pci_err[7] = pdev;
            break;
		default:
			break;
		}

		edac_dbg(0, "Associated PCI %02x.%02d.%d with dev = %p\n",
			 sbridge_dev->bus,
			 PCI_SLOT(pdev->devfn), PCI_FUNC(pdev->devfn),
			 pdev);
	}

	/* Check if everything were registered */
	if (!pvt->pci_sad0 || !pvt->pci_ha0 || !pvt->pci_ha0_perfmon ||
	    !pvt->pci_sad1 || !pvt->pci_ras  || !pvt->pci_ta ||
	    !pvt->info.pci_vtd)
		goto enodev;

	if (pvt->pci_ha1 && (!pvt->pci_ha1_perfmon || !pvt->pci_ha1_ta))
	    goto enodev;

	if (saw_chan_mask != 0x0f && /* -EN */
	    saw_chan_mask != 0x33 && /* -EP */
	    saw_chan_mask != 0xff)   /* -EX */
		goto enodev;
	return 0;

enodev:
	sbridge_printk(KERN_ERR, "Some needed devices are missing\n");
	return -ENODEV;
}

/****************************************************************************
			Error check routines
 ****************************************************************************/

/*
 * While Sandy Bridge has error count registers, SMI BIOS read values from
 * and resets the counters. So, they are not reliable for the OS to read
 * from them. So, we have no option but to just trust on whatever MCE is
 * telling us about the errors.
 */
static void sbridge_mce_output_error(struct mem_ctl_info *mci,
				    const struct mce *m)
{
	struct mem_ctl_info *new_mci;
	struct sbridge_pvt *pvt = mci->pvt_info;
	enum hw_event_mc_err_type tp_event;
	char *type, *optype, msg[256];
	bool ripv = GET_BITFIELD(m->mcgstatus, 0, 0);
	bool overflow = GET_BITFIELD(m->status, 62, 62);
	bool uncorrected_error = GET_BITFIELD(m->status, 61, 61);
	bool recoverable;
	u32 core_err_cnt = GET_BITFIELD(m->status, 38, 52);
	u32 mscod = GET_BITFIELD(m->status, 16, 31);
	u32 errcode = GET_BITFIELD(m->status, 0, 15);
	u32 channel = GET_BITFIELD(m->status, 0, 3);
	u32 optypenum = GET_BITFIELD(m->status, 4, 6);
	long channel_mask, first_channel;
	u8  rank, socket, ha;
	int rc, dimm;
	char *area_type = NULL;

	if (pvt->info.type != SANDY_BRIDGE)
		recoverable = true;
	else
		recoverable = GET_BITFIELD(m->status, 56, 56);

	if (uncorrected_error) {
		if (ripv) {
			type = "FATAL";
			tp_event = HW_EVENT_ERR_FATAL;
		} else {
			type = "NON_FATAL";
			tp_event = HW_EVENT_ERR_UNCORRECTED;
		}
	} else {
		type = "CORRECTED";
		tp_event = HW_EVENT_ERR_CORRECTED;
	}

	/*
	 * According with Table 15-9 of the Intel Architecture spec vol 3A,
	 * memory errors should fit in this mask:
	 *	000f 0000 1mmm cccc (binary)
	 * where:
	 *	f = Correction Report Filtering Bit. If 1, subsequent errors
	 *	    won't be shown
	 *	mmm = error type
	 *	cccc = channel
	 * If the mask doesn't match, report an error to the parsing logic
	 */
	if (! ((errcode & 0xef80) == 0x80)) {
		optype = "Can't parse: it is not a mem";
	} else {
		switch (optypenum) {
		case 0:
			optype = "generic undef request error";
			break;
		case 1:
			optype = "memory read error";
			break;
		case 2:
			optype = "memory write error";
			break;
		case 3:
			optype = "addr/cmd error";
			break;
		case 4:
			optype = "memory scrubbing error";
			break;
		default:
			optype = "reserved";
			break;
		}
	}

	/* Only decode errors with an valid address (ADDRV) */
	if (!GET_BITFIELD(m->status, 58, 58))
		return;

	rc = get_memory_error_data(mci, m->addr, &socket, &ha,
				   &channel_mask, &rank, &area_type, msg);
	if (rc < 0)
		goto err_parsing;
	new_mci = get_mci_for_node_id(socket);
	if (!new_mci) {
		strcpy(msg, "Error: socket got corrupted!");
		goto err_parsing;
	}
	mci = new_mci;
	pvt = mci->pvt_info;

	first_channel = find_first_bit(&channel_mask, NUM_CHANNELS);

	if (rank < 4)
		dimm = 0;
	else if (rank < 8)
		dimm = 1;
	else
		dimm = 2;


	/*
	 * FIXME: On some memory configurations (mirror, lockstep), the
	 * Memory Controller can't point the error to a single DIMM. The
	 * EDAC core should be handling the channel mask, in order to point
	 * to the group of dimm's where the error may be happening.
	 */
	if (!pvt->is_lockstep && !pvt->is_mirrored && !pvt->is_close_pg)
		channel = first_channel;

	snprintf(msg, sizeof(msg),
		 "%s%s area:%s err_code:%04x:%04x socket:%d ha:%d channel_mask:%ld rank:%d",
		 overflow ? " OVERFLOW" : "",
		 (uncorrected_error && recoverable) ? " recoverable" : "",
		 area_type,
		 mscod, errcode,
		 socket, ha,
		 channel_mask,
		 rank);

	edac_dbg(0, "%s\n", msg);

	/* FIXME: need support for channel mask */

	if (channel == CHANNEL_UNSPECIFIED)
		channel = -1;

	/* Call the helper to output message */
	edac_mc_handle_error(tp_event, mci, core_err_cnt,
			     m->addr >> PAGE_SHIFT, m->addr & ~PAGE_MASK, 0,
			     4*ha+channel, dimm, -1,
			     optype, msg);
	return;
err_parsing:
	edac_mc_handle_error(tp_event, mci, core_err_cnt, 0, 0, 0,
			     -1, -1, -1,
			     msg, "");

}

/*
 *	sbridge_check_error	Retrieve and process errors reported by the
 *				hardware. Called by the Core module.
 */
static void sbridge_check_error(struct mem_ctl_info *mci)
{
	struct sbridge_pvt *pvt = mci->pvt_info;
	int i;
	unsigned count = 0;
	struct mce *m;

	/*
	 * MCE first step: Copy all mce errors into a temporary buffer
	 * We use a double buffering here, to reduce the risk of
	 * loosing an error.
	 */
	smp_rmb();
	count = (pvt->mce_out + MCE_LOG_LEN - pvt->mce_in)
		% MCE_LOG_LEN;
	if (!count)
		return;

	m = pvt->mce_outentry;
	if (pvt->mce_in + count > MCE_LOG_LEN) {
		unsigned l = MCE_LOG_LEN - pvt->mce_in;

		memcpy(m, &pvt->mce_entry[pvt->mce_in], sizeof(*m) * l);
		smp_wmb();
		pvt->mce_in = 0;
		count -= l;
		m += l;
	}
	memcpy(m, &pvt->mce_entry[pvt->mce_in], sizeof(*m) * count);
	smp_wmb();
	pvt->mce_in += count;

	smp_rmb();
	if (pvt->mce_overrun) {
		sbridge_printk(KERN_ERR, "Lost %d memory errors\n",
			      pvt->mce_overrun);
		smp_wmb();
		pvt->mce_overrun = 0;
	}

	/*
	 * MCE second step: parse errors and display
	 */
	for (i = 0; i < count; i++)
		sbridge_mce_output_error(mci, &pvt->mce_outentry[i]);
}

/*
 * sbridge_mce_check_error	Replicates mcelog routine to get errors
 *				This routine simply queues mcelog errors, and
 *				return. The error itself should be handled later
 *				by sbridge_check_error.
 * WARNING: As this routine should be called at NMI time, extra care should
 * be taken to avoid deadlocks, and to be as fast as possible.
 */
static int sbridge_mce_check_error(struct notifier_block *nb, unsigned long val,
				   void *data)
{
	struct mce *mce = (struct mce *)data;
	struct mem_ctl_info *mci;
	struct sbridge_pvt *pvt;
	char *type;

	if (get_edac_report_status() == EDAC_REPORTING_DISABLED)
		return NOTIFY_DONE;

	mci = get_mci_for_node_id(mce->socketid);
	if (!mci)
		return NOTIFY_BAD;
	pvt = mci->pvt_info;

	/*
	 * Just let mcelog handle it if the error is
	 * outside the memory controller. A memory error
	 * is indicated by bit 7 = 1 and bits = 8-11,13-15 = 0.
	 * bit 12 has an special meaning.
	 */
	if ((mce->status & 0xefff) >> 7 != 1)
		return NOTIFY_DONE;

	if (mce->mcgstatus & MCG_STATUS_MCIP)
		type = "Exception";
	else
		type = "Event";

	sbridge_mc_printk(mci, KERN_DEBUG, "HANDLING MCE MEMORY ERROR\n");

	sbridge_mc_printk(mci, KERN_DEBUG, "CPU %d: Machine Check %s: %Lx "
			  "Bank %d: %016Lx\n", mce->extcpu, type,
			  mce->mcgstatus, mce->bank, mce->status);
	sbridge_mc_printk(mci, KERN_DEBUG, "TSC %llx ", mce->tsc);
	sbridge_mc_printk(mci, KERN_DEBUG, "ADDR %llx ", mce->addr);
	sbridge_mc_printk(mci, KERN_DEBUG, "MISC %llx ", mce->misc);

	sbridge_mc_printk(mci, KERN_DEBUG, "PROCESSOR %u:%x TIME %llu SOCKET "
			  "%u APIC %x\n", mce->cpuvendor, mce->cpuid,
			  mce->time, mce->socketid, mce->apicid);

	smp_rmb();
	if ((pvt->mce_out + 1) % MCE_LOG_LEN == pvt->mce_in) {
		smp_wmb();
		pvt->mce_overrun++;
		return NOTIFY_DONE;
	}

	/* Copy memory error at the ringbuffer */
	memcpy(&pvt->mce_entry[pvt->mce_out], mce, sizeof(*mce));
	smp_wmb();
	pvt->mce_out = (pvt->mce_out + 1) % MCE_LOG_LEN;

	/* Handle fatal errors immediately */
	if (mce->mcgstatus & 1)
		sbridge_check_error(mci);

	/* Advice mcelog that the error were handled */
	return NOTIFY_STOP;
}

static struct notifier_block sbridge_mce_dec = {
	.notifier_call      = sbridge_mce_check_error,
};

/*
 * inject_error_store  Driver attribute to inject error to the error bit.
 */
static ssize_t inject_error_store(struct device *dev,
                 struct device_attribute *mattr,
                 char *data,
                 size_t count)
{
    struct mem_ctl_info *mci = dev_get_drvdata(dev);
    struct sbridge_pvt  *pvt = NULL;
    u64 addrmatch = 0ULL, pattern = 0ULL;
    int error_bit = 0;
    long channel_mask, first_channel;
    u8  rank, socket, ha;
    char *area_type = NULL;
    char msg[256];
    unsigned long flags = 0;
    struct pci_dev *pci_ha = NULL, *pci_ha_perfmon = NULL;
    u32 save_has_sys_defeature = 0, value = 0;
    u32 orig_error_count = 0, temp_error_count = 0;
    int tad_id = 0, err_id = 0;
    u64 *map_addr, save_value, temp_value;

    if (g_inject_status == ECC_INJECT_LOCK) {
        sscanf(data, "0x%llx 0x%llx %d", &addrmatch, &pattern, &error_bit);
        sbridge_printk(KERN_DEBUG, "Error injection is locked!\n");
        return count;
    }

    spin_lock_irqsave(&g_error_inject_lock, flags);

    sscanf(data, "0x%llx 0x%llx %d", &addrmatch, &pattern, &error_bit);

    sbridge_printk(KERN_DEBUG, "Inject error to 0x%llx with test pattern 0x%llx and error bit %d\n",
            addrmatch,
            pattern,
            error_bit);

    g_inject_status = 0;

    if (0 != get_memory_error_data(mci, addrmatch, &socket, &ha, &channel_mask, &rank, &area_type, msg))
    {
        g_inject_status = ADDR_TRANSLATION_FAIL;
        sbridge_printk(KERN_DEBUG, "Error injection is failed: %s\n", msg);
        spin_unlock_irqrestore(&g_error_inject_lock, flags);
        return count;
    }

    mci = get_mci_for_node_id(socket);
    pvt = mci->pvt_info;
    first_channel = find_first_bit(&channel_mask, NUM_CHANNELS);

    if (ha) {
        pci_ha = pvt->pci_ha1;
        pci_ha_perfmon = pvt->pci_ha1_perfmon;
        tad_id = first_channel + 4;
    } else {
        pci_ha = pvt->pci_ha0;
        pci_ha_perfmon = pvt->pci_ha0_perfmon;
        tad_id = first_channel;
    }
    err_id = tad_id;

    // Disable scrub
    pci_read_config_dword(pci_ha, REG_HASYSDEFEATURE, &save_has_sys_defeature);
    value = save_has_sys_defeature & ~(1 << BIT_DEMAND_SCRUB);
    pci_write_config_dword(pci_ha, REG_HASYSDEFEATURE, value);

    map_addr = (u64 *)ioremap_nocache(addrmatch, sizeof(u64));
    if (!map_addr) {
        g_inject_status = IOREMAP_FAIL;
        sbridge_printk(KERN_DEBUG, "Error injection is failed: fail to ioremap 0x%llx!\n", addrmatch);
        goto fail;
    }

    save_value = *map_addr;

    // Set error bit and injection device
    value = (error_bit / 4);
    value |= 0x3C000000;
    pci_write_config_dword(pvt->pci_err[err_id],
                           REG_RSP_FUNC_CRC_ERR_INJ_EXTRA,
                           value);

    value = 1 << (error_bit % 4);
    pci_write_config_dword(pvt->pci_err[err_id],
                           REG_RSP_FUNC_CRC_ERR_INJ_DEV0_XOR_MSK,
                           value);

    pci_write_config_dword(pvt->pci_err[err_id],
                           REG_RSP_FUNC_CRC_ERR_INJ_DEV1_XOR_MSK,
                           0);

    // Get error count before error injection
    pci_read_config_dword(pci_ha_perfmon,
                          REG_MC5_STATUS_SHADOW_HIGH,
                          &value);

    orig_error_count = (value >> 6) & 0x7FFF;

    pci_write_config_dword(pvt->pci_tad[tad_id],
                           REG_RSP_FUNC_ADDR_MASK_HI,
                           (u32)(0x3FULL >> 35));
    pci_write_config_dword(pvt->pci_tad[tad_id],
                           REG_RSP_FUNC_ADDR_MASK_LO,
                           (u32)(0x3FULL >> 3));

    pci_write_config_dword(pvt->pci_tad[tad_id],
                           REG_RSP_FUNC_ADDR_MATCH_LO,
                           (u32)(addrmatch >> 3));

    pci_write_config_dword(pvt->pci_tad[tad_id],
                           REG_RSP_FUNC_ADDR_MATCH_HI,
                           (u32)(addrmatch >> 35));

    // Enable error inject
    pci_write_config_dword(pvt->pci_tad[tad_id],
                           REG_RSP_FUNC_ADDR_MATCH_HI,
                           ((u32)(addrmatch >> 35) | (0x1 << BIT_RSP_FUNC_ADDR_MATCH_EN)));

    pci_read_config_dword(pvt->pci_tad[tad_id],
                          REG_RSP_FUNC_ADDR_MATCH_HI,
                          &value);

    if (!(value >> BIT_RSP_FUNC_ADDR_MATCH_EN) & 0x1) {
        sbridge_printk(KERN_DEBUG, "Error injection is failed: error inject was not armed!\n");
        g_inject_status = ENABLE_ECC_INJECT_FAIL;
        goto fail;
    }

    // Write memory to create error data
    *(map_addr) = pattern;

    // Clear cache
    asm volatile ("wbinvd");

    pci_read_config_dword(pvt->pci_tad[tad_id],
                          REG_RSP_FUNC_ADDR_MATCH_HI,
                          &value);

    if (!((value >> BIT_RSP_FUNC_ADDR_MATCH_EN) & 0x1)) {
        // Read memory to inject error
        temp_value = (*map_addr);

        if (temp_value != pattern) {
            sbridge_printk(KERN_DEBUG, "Error injection is failed: data is inconsistent!\n");
            g_inject_status = COMPARE_DATA_FAIL;
        }

        if (!g_inject_status) {
            pci_read_config_dword(pci_ha_perfmon,
                                  REG_MC5_STATUS_SHADOW_HIGH,
                                  &value);

            temp_error_count = (value >> 6) & 0x7FFF;
            temp_error_count -= orig_error_count;

            if (temp_error_count != 1) {
                if (temp_error_count == 0) {
                    g_inject_status = NO_ECC_COUNT;
                    sbridge_printk(KERN_DEBUG, "Error injection is failed: no error count!\n");
                } else {
                    g_inject_status = INCORRECT_ECC_COUNT;
                    sbridge_printk(KERN_DEBUG, "Error injection is failed: actual error count: %x expect error count %x!\n", temp_error_count, orig_error_count + 1);
                }   
            }
        }


        // Restore memory content
        *map_addr = save_value;
        asm volatile ("wbinvd");
    } else {
        pci_write_config_dword(pvt->pci_tad[tad_id],
                               REG_RSP_FUNC_ADDR_MATCH_HI,
                               0);

        sbridge_printk(KERN_DEBUG, "Error injection is failed: inject enable bit is not cleared!\n");

        g_inject_status = ECC_INJECT_FAIL;
    }

fail:
    if (map_addr)
        iounmap(map_addr);

    // Restore scrub
    pci_write_config_dword(pci_ha, REG_HASYSDEFEATURE, save_has_sys_defeature);

    spin_unlock_irqrestore(&g_error_inject_lock, flags);

    return count;
}

static DEVICE_ATTR(inject_error, S_IRUGO | S_IWUSR,
            NULL, inject_error_store);

/*
 * inject_error_store  Driver attribute to show the result of error injection.
 */
static ssize_t sbridge_inject_status_show(struct device *dev,
                     struct device_attribute *mattr,
                     char *data)
{
    unsigned long flags = 0;
    int status;

    spin_lock_irqsave(&g_error_inject_lock, flags);
    status = g_inject_status;
    spin_unlock_irqrestore(&g_error_inject_lock, flags);

    return sprintf(data, "%d", status);
}

static DEVICE_ATTR(inject_status, S_IRUGO | S_IWUSR,
            sbridge_inject_status_show, NULL);

static ssize_t sbridge_layout_show(struct device *dev,
                    struct device_attribute *mattr,
                    char *data)
{
    ssize_t count = 0;
    struct mem_ctl_info *mci = dev_get_drvdata(dev);
    struct sbridge_pvt  *pvt = mci->pvt_info;
    struct sbridge_range *range, *temp_range;
    u64     sad_limit, sad_prv, tad_limit, tad_prv, offset, tad_ch_prv[8];
    int     pkg_interleave_table[8], socket_interleave_table[8], channel_interleave_table[4];
    int     i, j, k, temp_channel;
    int     n_ranges = 0, n_sads, n_tads, n_socket, n_ha, n_channel, n_rir_range, n_rir_way;
    int     socket_ways, ha_ways, channel_ways, rir_ways;
    u32     pkg, socket, reg;
    char    channel_message[128], rank_message[256];
    int     ch_msg_count = 0, rank_msg_count = 0;
    u64     temp_rir_limit;
    u64     considered_range_size[8][8], temp_range_size, min_range_size;
    u64     socket_size, channel_size, rank_size;


    if (list_empty(&sbridge_range_list)) {
        mutex_lock(&sbridge_range_list_lock);

        memset(tad_ch_prv, 0, sizeof(tad_ch_prv));

        sad_prv = 0;
        for (n_sads = 0; n_sads < pvt->info.max_sad; n_sads++) {
            pci_read_config_dword(pvt->pci_sad0, pvt->info.dram_rule[n_sads], &reg);
            sad_limit = SAD_LIMIT(reg);

            if (!DRAM_RULE_ENABLE(reg))
                continue;

            if (sad_limit <= sad_prv)
                break;

            socket_ways = 0;

            // Get socket interleave info
            pci_read_config_dword(pvt->pci_sad0, pvt->info.interleave_list[n_sads],
                                  &reg);

            for (i = 0; i < 8; i++) {
                pkg_interleave_table[i] = -1;
                socket_interleave_table[i] = -1;
            }

            for (i = 0; i < 8; i++) {
                pkg = sad_pkg(pvt->info.interleave_pkg, reg, i);
                for (j = 0; j < 8; j++) {
                    if (pkg_interleave_table[j] == -1)
                        break;

                    if (pkg != pkg_interleave_table[j])
                        continue;
                    else
                        break;
                }

                if (j == 8) {
                    count += sprintf(data + count, "fail to get layout");
                    mutex_unlock(&sbridge_range_list_lock);
                    return count;
                }

                if (pkg_interleave_table[j] == -1)
                    pkg_interleave_table[j] = pkg;
                else
                    continue;

                socket = sad_pkg_socket(pkg);
                for (j = 0; j < 8; j++) {
                    if (socket_interleave_table[j] == -1)
                        break;
                    if (socket != socket_interleave_table[j])
                        continue;
                    else
                        break;
                }

                if (j == 8) {
                    count += sprintf(data + count, "fail to get layout");
                    mutex_unlock(&sbridge_range_list_lock);
                    return count;
                }

                if (socket_interleave_table[j] == -1) {
                    socket_interleave_table[j] = socket;
                    socket_ways++;
                }
            }

            // Check each TAD range
            tad_prv = 0;
            for (n_tads = 0; n_tads < MAX_TAD; n_tads++) {
                // Get correct device for socket
                if (socket_interleave_table[0] != pvt->sbridge_dev->node_id) {
                    mci = get_mci_for_node_id(socket_interleave_table[0]);
                    pvt = mci->pvt_info;
                }

                pci_read_config_dword(pvt->pci_ha0, tad_dram_rule[n_tads], &reg);
                tad_limit = TAD_LIMIT(reg);
                if (tad_limit <= tad_prv)
                    break;

                // Skip current TAD which has been considered
                if (tad_limit <= sad_prv)
                    continue;

                // Current TAD exceed current SAD
                if (tad_limit > sad_limit)
                    break;

                range = kzalloc(sizeof(*range), GFP_KERNEL);

                if (range == NULL)
                    break;

                for (i = 0; i < 8; i++) {
                    range->socket[i] = -1;
                    for (j = 0; j < 8; j++) {
                        range->channel[i][j] = -1;
                        for (k = 0; k < 8; k++)
                            range->rank[i][j][k] = -1;
                    }
                }

                INIT_LIST_HEAD(&range->list);

                n_channel = TAD_TGT0(reg);

                pci_read_config_dword(pvt->pci_tad[n_channel],
                             tad_ch_nilv_offset[n_tads],
                             &reg);

                offset = TAD_OFFSET(reg);

                // Get actual offset for system memory address
                if (tad_ch_prv[n_channel])
                    offset += tad_ch_prv[n_channel] + 1;

                range->start = offset;
                range->end = tad_limit;

                tad_ch_prv[n_channel] = tad_limit;
                tad_prv = tad_limit;

                // Get information of sockets and channels for current TAD
                for (i = 0; i < socket_ways; i++) {
                    range->socket[i] = socket_interleave_table[i];

                    mci = get_mci_for_node_id(socket_interleave_table[i]);
                    pvt = mci->pvt_info;

                    pci_read_config_dword(pvt->pci_ha0, tad_dram_rule[n_tads], &reg);

                    ha_ways = 1 << TAD_SOCK(reg);
                    channel_ways = TAD_CH(reg) + 1;

                    channel_interleave_table[0] = TAD_TGT0(reg);
                    channel_interleave_table[1] = TAD_TGT1(reg);
                    channel_interleave_table[2] = TAD_TGT2(reg);
                    channel_interleave_table[3] = TAD_TGT3(reg);

                    // Get channel interleave info
                    for (n_ha = 0; n_ha < (ha_ways / socket_ways); n_ha++)
                    {
                        for (n_channel = 0; n_channel < channel_ways; n_channel++) {
                            temp_channel = n_ha != 0? (channel_interleave_table[n_channel] + (NUM_CHANNELS / (ha_ways / socket_ways)) * n_ha):
                                                      channel_interleave_table[n_channel];

                            range->channel[i][n_ha*channel_ways+n_channel] = temp_channel;
                        }
                    }

                }

                list_add_tail(&range->list, &sbridge_range_list);

                tad_prv = tad_limit;
            }

            sad_prv = sad_limit;
        }

        memset(considered_range_size, 0, sizeof(considered_range_size));

        // Check each TAD range based on each channel
        list_for_each_entry(range, &sbridge_range_list, list) {
            min_range_size = 0xFFFFFFFFFFFFFFFF;
            channel_ways = socket_ways = 0;

            for (i = 0; i < 8; i++) {
                if (range->socket[i] != -1)
                    socket_ways++;
                else
                    break;
            }

            socket_size = (range->end - range->start + 1) / socket_ways;

            for (i = 0; i < socket_ways; i++) {
                n_socket = range->socket[i];
                // Get correct device for socket
                mci = get_mci_for_node_id(n_socket);
                pvt = mci->pvt_info;

                channel_ways = 0;
                for (j = 0; j < 8; j++) {
                    if (range->channel[n_socket][j] != -1)
                        channel_ways++;
                    else
                        break;
                }

                channel_size = socket_size / channel_ways;

                // Search for the minimum rir size of channels for current TAD range
                for (j = 0; j < channel_ways; j++) {
                    n_channel = range->channel[n_socket][j];
                    sbridge_printk(KERN_DEBUG, "channel %d\n", n_channel);

                    for (n_rir_range = 0; n_rir_range < MAX_RIR_RANGES; n_rir_range++) {
                        pci_read_config_dword(pvt->pci_tad[n_channel],
                                      rir_way_limit[n_rir_range],
                                      &reg);

                        if (!IS_RIR_VALID(reg))
                            continue;

                        temp_rir_limit = pvt->info.rir_limit(reg);

                        // Skip current RIR which has been considered
                        if (temp_rir_limit < considered_range_size[n_socket][n_channel])
                            continue;

                        rir_ways = 1 << RIR_WAY(reg);

                        temp_range_size = temp_rir_limit - considered_range_size[n_socket][n_channel] + 1;

                        // Get rank interleave info
                        for (n_rir_way = 0; n_rir_way < rir_ways; n_rir_way++) {
                            pci_read_config_dword(pvt->pci_tad[n_channel],
                                          rir_offset[n_rir_range][n_rir_way],
                                          &reg);

                            range->rank[n_socket][n_channel][n_rir_way] = pvt->info.rir_rank_tgt(reg);
                        }

                        // Current RIR has included entire range for current channel
                        if (temp_range_size >= channel_size)
                            temp_range_size = channel_size;

                        temp_range_size *= channel_ways;

                        if (temp_range_size < min_range_size)
                            min_range_size = temp_range_size;

                        // Check RIRs one by one because different RIR has different
                        // interleave info
                        break;
                    }
                }
            }

            if (min_range_size != 0) {
                // Update considered range size for each channel
                for (i = 0; i < socket_ways; i++) {
                    channel_ways = 0;
                    for (j = 0; j < 8; j++) {
                        if (range->channel[i][j] != -1)
                            channel_ways++;
                        else
                            break;
                    }

                    temp_range_size = min_range_size / channel_ways;
                    for (j = 0; j < channel_ways; j++) {
                        n_channel = range->channel[i][j];
                        considered_range_size[i][n_channel] += temp_range_size;
                    }
                }

                // Need to divide the current TAD range
                if (min_range_size < socket_size) {
                    min_range_size *= socket_ways;

                    temp_range = kzalloc(sizeof(*range), GFP_KERNEL);
                    if (temp_range == NULL)
                        break;

                    memcpy(temp_range, range, sizeof(*range));

                    INIT_LIST_HEAD(&temp_range->list);

                    list_add(&temp_range->list, &range->list);

                    temp_range->start = range->start + min_range_size;
                    range->end = temp_range->start - 1;
                }
            }
        }

        mutex_unlock(&sbridge_range_list_lock);
    }

    mutex_lock(&sbridge_range_list_lock);

    list_for_each_entry(range, &sbridge_range_list, list) {
        ch_msg_count = rank_msg_count = 0;
        count += sprintf(data + count, "range%d:", n_ranges++);
        count += sprintf(data + count, "0x%llx-0x%llx\n", range->start, range->end);
        count += sprintf(data + count, "mc:");
        for (i = 0; i < 8; i++) {
            n_socket = range->socket[i];
            if (n_socket != -1) {

                count += sprintf(data + count, "%d,", n_socket);

                ch_msg_count += sprintf(channel_message + ch_msg_count, "channel:");
                for (j = 0; j < 8; j++) {
                    n_channel = range->channel[n_socket][j];
                    if (n_channel != -1) {
                        ch_msg_count += sprintf(channel_message + ch_msg_count, "%d,", n_channel);

                        rank_msg_count += sprintf(rank_message + rank_msg_count, "rank:");
                        for (k = 0; k < 8; k++) {
                            if (range->rank[n_socket][n_channel][k] != -1)
                                rank_msg_count += sprintf(rank_message + rank_msg_count, "%d,", range->rank[n_socket][n_channel][k]);
                            else
                                break;
                        }
                        rank_msg_count--;
                        rank_msg_count += sprintf(rank_message + rank_msg_count, "\n");
                    } else
                        break;
                }
                ch_msg_count--;
                ch_msg_count += sprintf(channel_message + ch_msg_count, "\n");
            } else
                break;
        }
        count--;
        count += sprintf(data + count, "\n");
        count += sprintf(data + count, "%s", channel_message);
        count += sprintf(data + count, "%s", rank_message);
    }

    mutex_unlock(&sbridge_range_list_lock);

    return count;
}

static DEVICE_ATTR(layout, S_IRUGO | S_IWUSR,
            sbridge_layout_show, NULL);

static ssize_t sbridge_addr_to_rank_store(struct device *dev,
                    struct device_attribute *mattr,
                    char *data,
                    size_t count)
{
    unsigned long flags = 0;
    struct mem_ctl_info *mci = dev_get_drvdata(dev);
    int error;
    long channel_mask;
    u8  rank, socket, ha;


    spin_lock_irqsave(&g_rank_addr_detect_lock, flags);
    sscanf(data, "0x%llx", &g_rank_addr_detect.address);
    error = parse_system_addr(mci, g_rank_addr_detect.address, &socket, &ha, &channel_mask, &rank, &g_rank_addr_detect.rank_offset);
    if (error)
        g_rank_addr_detect.rank_offset = 0xFFFFFFFFFFFFFFFFULL;
    spin_unlock_irqrestore(&g_rank_addr_detect_lock, flags);
    return count;
}

static ssize_t sbridge_addr_to_rank_show(struct device *dev,
                    struct device_attribute *mattr,
                    char *data)
{
    unsigned long flags = 0;
    u64 rank_offset;

    spin_lock_irqsave(&g_rank_addr_detect_lock, flags);
    rank_offset = g_rank_addr_detect.rank_offset;
    spin_unlock_irqrestore(&g_rank_addr_detect_lock, flags);
    return sprintf(data, "0x%llx", rank_offset);
}

static DEVICE_ATTR(addr_to_rank, S_IRUGO | S_IWUSR,
        sbridge_addr_to_rank_show, sbridge_addr_to_rank_store);

static ssize_t sbridge_rank_to_addr_store(struct device *dev,
                    struct device_attribute *mattr,
                    char *data,
                    size_t count)
{
    unsigned long flags = 0;
    int socket = 0, ha = 0, channel = 0, rank = 0;
    spin_lock_irqsave(&g_system_addr_detect_lock, flags);
    sscanf(data, "0x%llx %d %d %d", &g_sys_addr_detect.rank_offset, &socket, &channel, &rank);
    if (channel >= 4)
    {
        ha = 1;
        channel -= 4;
    }
    if (get_system_addr(socket, ha, channel, rank, g_sys_addr_detect.rank_offset, &g_sys_addr_detect.system_addr) != 0)
        g_sys_addr_detect.system_addr = 0xFFFFFFFFFFFFFFFFULL;
    spin_unlock_irqrestore(&g_system_addr_detect_lock, flags);
    return count;
}

static ssize_t sbridge_rank_to_addr_show(struct device *dev,
                    struct device_attribute *mattr,
                    char *data)
{
    unsigned long flags = 0;
    u64 system_addr;

    spin_lock_irqsave(&g_system_addr_detect_lock, flags);
    system_addr = g_sys_addr_detect.system_addr;
    spin_unlock_irqrestore(&g_system_addr_detect_lock, flags);
    return sprintf(data, "0x%llx", system_addr);
}

static DEVICE_ATTR(rank_to_addr, S_IRUGO | S_IWUSR,
        sbridge_rank_to_addr_show, sbridge_rank_to_addr_store);

static struct attribute *sbridge_dev_attrs[] = {
    &dev_attr_inject_error.attr,
    &dev_attr_inject_status.attr,
    &dev_attr_layout.attr,
    &dev_attr_addr_to_rank,
    &dev_attr_rank_to_addr,
    NULL
};

ATTRIBUTE_GROUPS(sbridge_dev);

/****************************************************************************
			EDAC register/unregister logic
 ****************************************************************************/

static void sbridge_unregister_mci(struct sbridge_dev *sbridge_dev)
{
	struct mem_ctl_info *mci = sbridge_dev->mci;
	struct sbridge_pvt *pvt;

	if (unlikely(!mci || !mci->pvt_info)) {
		edac_dbg(0, "MC: dev = %p\n", &sbridge_dev->pdev[0]->dev);

		sbridge_printk(KERN_ERR, "Couldn't find mci handler\n");
		return;
	}

	pvt = mci->pvt_info;

	edac_dbg(0, "MC: mci = %p, dev = %p\n",
		 mci, &sbridge_dev->pdev[0]->dev);

	/* Remove MC sysfs nodes */
	edac_mc_del_mc(mci->pdev);

	edac_dbg(1, "%s: free mci struct\n", mci->ctl_name);
	kfree(mci->ctl_name);
	edac_mc_free(mci);
	sbridge_dev->mci = NULL;
}

static int sbridge_register_mci(struct sbridge_dev *sbridge_dev, enum type type)
{
	struct mem_ctl_info *mci;
	struct edac_mc_layer layers[2];
	struct sbridge_pvt *pvt;
	struct pci_dev *pdev = sbridge_dev->pdev[0];
	int rc;

	/* Check the number of active and not disabled channels */
	rc = check_if_ecc_is_active(sbridge_dev->bus, type);
	if (unlikely(rc < 0))
		return rc;

	/* allocate a new MC control structure */
	layers[0].type = EDAC_MC_LAYER_CHANNEL;
	layers[0].size = NUM_CHANNELS;
	layers[0].is_virt_csrow = false;
	layers[1].type = EDAC_MC_LAYER_SLOT;
	layers[1].size = MAX_DIMMS;
	layers[1].is_virt_csrow = true;
	mci = edac_mc_alloc(sbridge_dev->mc, ARRAY_SIZE(layers), layers,
			    sizeof(*pvt));

	if (unlikely(!mci))
		return -ENOMEM;

	edac_dbg(0, "MC: mci = %p, dev = %p\n",
		 mci, &pdev->dev);

	pvt = mci->pvt_info;
	memset(pvt, 0, sizeof(*pvt));

	/* Associate sbridge_dev and mci for future usage */
	pvt->sbridge_dev = sbridge_dev;
	sbridge_dev->mci = mci;

	mci->mtype_cap = MEM_FLAG_DDR3;
	mci->edac_ctl_cap = EDAC_FLAG_NONE;
	mci->edac_cap = EDAC_FLAG_NONE;
	mci->mod_name = "sbridge_edac.c";
	mci->mod_ver = SBRIDGE_REVISION;
	mci->dev_name = pci_name(pdev);
	mci->ctl_page_to_phys = NULL;

	/* Set the function pointer to an actual operation function */
	mci->edac_check = sbridge_check_error;

	pvt->info.type = type;
	switch (type) {
	case IVY_BRIDGE:
		pvt->info.rankcfgr = IB_RANK_CFG_A;
		pvt->info.get_tolm = ibridge_get_tolm;
		pvt->info.get_tohm = ibridge_get_tohm;
		pvt->info.dram_rule = ibridge_dram_rule;
		pvt->info.get_memory_type = get_memory_type;
		pvt->info.get_node_id = get_node_id;
		pvt->info.rir_limit = rir_limit;
        pvt->info.rir_rank_offset = rir_rank_offset;
        pvt->info.rir_rank_tgt = rir_rank_tgt;
		pvt->info.max_sad = ARRAY_SIZE(ibridge_dram_rule);
		pvt->info.interleave_list = ibridge_interleave_list;
		pvt->info.max_interleave = ARRAY_SIZE(ibridge_interleave_list);
		pvt->info.interleave_pkg = ibridge_interleave_pkg;
        pvt->info.max_hash = ARRAY_SIZE(ibridge_interleave_hash_list);
        pvt->info.interleave_hash_list = ibridge_interleave_hash_list;
		mci->ctl_name = kasprintf(GFP_KERNEL, "Ivy Bridge Socket#%d", mci->mc_idx);

		/* Store pci devices at mci for faster access */
		rc = ibridge_mci_bind_devs(mci, sbridge_dev);
		if (unlikely(rc < 0))
			goto fail0;
		break;
	case SANDY_BRIDGE:
		pvt->info.rankcfgr = SB_RANK_CFG_A;
		pvt->info.get_tolm = sbridge_get_tolm;
		pvt->info.get_tohm = sbridge_get_tohm;
		pvt->info.dram_rule = sbridge_dram_rule;
		pvt->info.get_memory_type = get_memory_type;
		pvt->info.get_node_id = get_node_id;
		pvt->info.rir_limit = rir_limit;
        pvt->info.rir_rank_offset = rir_rank_offset;
        pvt->info.rir_rank_tgt = rir_rank_tgt;
		pvt->info.max_sad = ARRAY_SIZE(sbridge_dram_rule);
		pvt->info.interleave_list = sbridge_interleave_list;
		pvt->info.max_interleave = ARRAY_SIZE(sbridge_interleave_list);
		pvt->info.interleave_pkg = sbridge_interleave_pkg;
        pvt->info.interleave_hash_list = NULL;
		mci->ctl_name = kasprintf(GFP_KERNEL, "Sandy Bridge Socket#%d", mci->mc_idx);

		/* Store pci devices at mci for faster access */
		rc = sbridge_mci_bind_devs(mci, sbridge_dev);
		if (unlikely(rc < 0))
			goto fail0;
		break;
	case HASWELL:
		/* rankcfgr isn't used */
		pvt->info.get_tolm = haswell_get_tolm;
		pvt->info.get_tohm = haswell_get_tohm;
		pvt->info.dram_rule = ibridge_dram_rule;
		pvt->info.get_memory_type = haswell_get_memory_type;
		pvt->info.get_node_id = haswell_get_node_id;
		pvt->info.rir_limit = haswell_rir_limit;
        pvt->info.rir_rank_offset = haswell_rir_rank_offset;
        pvt->info.rir_rank_tgt = rir_rank_tgt;
		pvt->info.max_sad = ARRAY_SIZE(ibridge_dram_rule);
		pvt->info.interleave_list = ibridge_interleave_list;
		pvt->info.max_interleave = ARRAY_SIZE(ibridge_interleave_list);
		pvt->info.interleave_pkg = ibridge_interleave_pkg;
        pvt->info.max_hash = ARRAY_SIZE(ibridge_interleave_hash_list);
        pvt->info.interleave_hash_list = ibridge_interleave_hash_list;
		mci->ctl_name = kasprintf(GFP_KERNEL, "Haswell Socket#%d", mci->mc_idx);

		/* Store pci devices at mci for faster access */
		rc = haswell_mci_bind_devs(mci, sbridge_dev);
		if (unlikely(rc < 0))
			goto fail0;
		break;
	case BROADWELL:
		/* rankcfgr isn't used */
		pvt->info.get_tolm = haswell_get_tolm;
		pvt->info.get_tohm = haswell_get_tohm;
		pvt->info.dram_rule = ibridge_dram_rule;
		pvt->info.get_memory_type = haswell_get_memory_type;
		pvt->info.get_node_id = haswell_get_node_id;
		pvt->info.rir_limit = haswell_rir_limit;
        pvt->info.rir_rank_offset = haswell_rir_rank_offset;
        pvt->info.rir_rank_tgt = broadwell_rir_rank_tgt;
		pvt->info.max_sad = ARRAY_SIZE(ibridge_dram_rule);
		pvt->info.interleave_list = ibridge_interleave_list;
		pvt->info.max_interleave = ARRAY_SIZE(ibridge_interleave_list);
		pvt->info.interleave_pkg = ibridge_interleave_pkg;
        pvt->info.max_hash = ARRAY_SIZE(ibridge_interleave_hash_list);
        pvt->info.interleave_hash_list = ibridge_interleave_hash_list;
		mci->ctl_name = kasprintf(GFP_KERNEL, "Broadwell Socket#%d", mci->mc_idx);

		/* Store pci devices at mci for faster access */
		rc = broadwell_mci_bind_devs(mci, sbridge_dev);
		if (unlikely(rc < 0))
			goto fail0;
		break;
	}

	/* Get dimm basic config and the memory layout */
	get_dimm_config(mci);
	get_memory_layout(mci);

	/* record ptr to the generic device */
	mci->pdev = &pdev->dev;

	/* add this new MC control structure to EDAC's list of MCs */
	switch (type) {
	case BROADWELL:
            if (unlikely(edac_mc_add_mc_with_groups(mci, sbridge_dev_groups))) {
                edac_dbg(0, "MC: failed edac_mc_add_mc()\n");
                rc = -EINVAL;
                goto fail0;
            }
            break;
	default:
	    if (unlikely(edac_mc_add_mc(mci))) {
	        edac_dbg(0, "MC: failed edac_mc_add_mc()\n");
	        rc = -EINVAL;
	        goto fail0;
	    }
	    break;
	}

	return 0;

fail0:
	kfree(mci->ctl_name);
	edac_mc_free(mci);
	sbridge_dev->mci = NULL;
	return rc;
}

/*
 *	sbridge_probe	Probe for ONE instance of device to see if it is
 *			present.
 *	return:
 *		0 for FOUND a device
 *		< 0 for error code
 */

static int sbridge_probe(struct pci_dev *pdev, const struct pci_device_id *id)
{
	int rc = -ENODEV;
	u8 mc, num_mc = 0;
	struct sbridge_dev *sbridge_dev;
	enum type type = SANDY_BRIDGE;
    u64 msr = 0ULL;

	/* get the pci devices we want to reserve for our use */
	mutex_lock(&sbridge_edac_lock);

	/*
	 * All memory controllers are allocated at the first pass.
	 */
	if (unlikely(probed >= 1)) {
		mutex_unlock(&sbridge_edac_lock);
		return -ENODEV;
	}
	probed++;

	switch (pdev->device) {
	case PCI_DEVICE_ID_INTEL_IBRIDGE_IMC_HA0_TA:
		rc = sbridge_get_all_devices(&num_mc, pci_dev_descr_ibridge_table);
		type = IVY_BRIDGE;
		break;
	case PCI_DEVICE_ID_INTEL_SBRIDGE_IMC_HA0:
		rc = sbridge_get_all_devices(&num_mc, pci_dev_descr_sbridge_table);
		type = SANDY_BRIDGE;
		break;
	case PCI_DEVICE_ID_INTEL_HASWELL_IMC_HA0:
		rc = sbridge_get_all_devices(&num_mc, pci_dev_descr_haswell_table);
		type = HASWELL;
		break;
	case PCI_DEVICE_ID_INTEL_BROADWELL_IMC_HA0:
		rc = sbridge_get_all_devices(&num_mc, pci_dev_descr_broadwell_table);
		type = BROADWELL;
		break;
	}
	if (unlikely(rc < 0)) {
		edac_dbg(0, "couldn't get all devices for 0x%x\n", pdev->device);
		goto fail0;
	}

	mc = 0;

	/* Register mci in inverse order because pci devices are scanned from the last bus */
	list_for_each_entry_reverse(sbridge_dev, &sbridge_edac_list, list) {
		edac_dbg(0, "Registering MC#%d (%d of %d)\n",
			 mc, mc + 1, num_mc);

		sbridge_dev->mc = mc++;
		rc = sbridge_register_mci(sbridge_dev, type);
		if (unlikely(rc < 0))
			goto fail1;
	}

	sbridge_printk(KERN_INFO, "%s\n", SBRIDGE_REVISION);

	mutex_unlock(&sbridge_edac_lock);

    rdmsrl_safe(0x790, &msr);
    if (msr)
        g_inject_status = ECC_INJECT_LOCK;
	return 0;

fail1:
	list_for_each_entry(sbridge_dev, &sbridge_edac_list, list)
		sbridge_unregister_mci(sbridge_dev);

	sbridge_put_all_devices();
fail0:
	mutex_unlock(&sbridge_edac_lock);
	return rc;
}

/*
 *	sbridge_remove	destructor for one instance of device
 *
 */
static void sbridge_remove(struct pci_dev *pdev)
{
	struct sbridge_dev *sbridge_dev;
	struct sbridge_range *range, *temp_range;

	edac_dbg(0, "\n");

	/*
	 * we have a trouble here: pdev value for removal will be wrong, since
	 * it will point to the X58 register used to detect that the machine
	 * is a Nehalem or upper design. However, due to the way several PCI
	 * devices are grouped together to provide MC functionality, we need
	 * to use a different method for releasing the devices
	 */

	mutex_lock(&sbridge_edac_lock);

	if (unlikely(!probed)) {
		mutex_unlock(&sbridge_edac_lock);
		return;
	}

	list_for_each_entry(sbridge_dev, &sbridge_edac_list, list)
		sbridge_unregister_mci(sbridge_dev);

	/* Release PCI resources */
	sbridge_put_all_devices();

	probed--;

	mutex_unlock(&sbridge_edac_lock);

	mutex_lock(&sbridge_range_list_lock);

	list_for_each_entry_safe(range, temp_range, &sbridge_range_list, list) {
        list_del(&range->list);
        kfree(range);
    }

    mutex_unlock(&sbridge_range_list_lock);
}

MODULE_DEVICE_TABLE(pci, sbridge_pci_tbl);

/*
 *	sbridge_driver	pci_driver structure for this module
 *
 */
static struct pci_driver sbridge_driver = {
	.name     = "sbridge_edac",
	.probe    = sbridge_probe,
	.remove   = sbridge_remove,
	.id_table = sbridge_pci_tbl,
};

/*
 *	sbridge_init		Module entry function
 *			Try to initialize this module for its devices
 */
static int __init sbridge_init(void)
{
	int pci_rc;

	edac_dbg(2, "\n");

	/* Ensure that the OPSTATE is set correctly for POLL or NMI */
	opstate_init();

	pci_rc = pci_register_driver(&sbridge_driver);
	if (pci_rc >= 0) {
		mce_register_decode_chain(&sbridge_mce_dec);
		if (get_edac_report_status() == EDAC_REPORTING_DISABLED)
			sbridge_printk(KERN_WARNING, "Loading driver, error reporting disabled.\n");
		return 0;
	}

	sbridge_printk(KERN_ERR, "Failed to register device with error %d.\n",
		      pci_rc);

	return pci_rc;
}

/*
 *	sbridge_exit()	Module exit function
 *			Unregister the driver
 */
static void __exit sbridge_exit(void)
{
	edac_dbg(2, "\n");
	pci_unregister_driver(&sbridge_driver);
	mce_unregister_decode_chain(&sbridge_mce_dec);
}

module_init(sbridge_init);
module_exit(sbridge_exit);

module_param(edac_op_state, int, 0444);
MODULE_PARM_DESC(edac_op_state, "EDAC Error Reporting state: 0=Poll,1=NMI");

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Mauro Carvalho Chehab");
MODULE_AUTHOR("Red Hat Inc. (http://www.redhat.com)");
MODULE_DESCRIPTION("MC Driver for Intel Sandy Bridge and Ivy Bridge memory controllers - "
		   SBRIDGE_REVISION);
