/// @file       BaseClassAPI.hpp
/// @brief      BaseClassAPI class
/// @details    Provide base class for API common function
/// @version    1.0.0
/// @date       2014.01.21
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include "BaseClassAPI.hpp"

/// @fn         BaseClassAPI::BaseClassAPI(STRING in_szClassName)
/// @brief      BaseClassAPI constructor
/// @param      in_szClassName is a pointer to a char string for specified class name
/// @exception  Nothing
/// @return     Nothing
BaseClassAPI::BaseClassAPI(STRING in_szClassName)
{
    // Initializations
    m_errorCode         = NO_ERROR;
    m_szTempString[0]   = (CHAR)NULL;
    m_pErrorMsg         = NULL;
        
    // Store the class name and NULL terminate it.
    strncpy(m_szClassName, in_szClassName, CLASS_NAME_LENGTH);
    m_szClassName[CLASS_NAME_LENGTH] = (CHAR)NULL;
}

/// @fn         BaseClassAPI::~BaseClassAPI(void)
/// @brief      BaseClassAPI destructor
/// @exception  Nothing
/// @return     Nothing
BaseClassAPI::~BaseClassAPI(void)
{
    clearErrorMsg();
}

/// @fn         BaseClassAPI::addErrorMsg(ERRORCODE   in_eErrorCode, 
///                                       const CHAR* in_szMessage)
/// @brief      Add ErrorMsg to the tail of the error message list
/// @param      in_eErrorCode is a 32bits error code
/// @param      in_szMessage is a char string for user specified message
/// @exception  std::bad_alloc when memory allocation for new instance is failed
/// @return     Nothing
void
BaseClassAPI::addErrorMsg(ERRORCODE   in_eErrorCode, 
                          const CHAR* in_szMessage)
{
    ErrorMsg* pNewErrorMsg = NULL; 

    if (in_eErrorCode == NO_ERROR)
        return;

    try
    {
        pNewErrorMsg = new ErrorMsg(in_eErrorCode, in_szMessage);

        if (m_pErrorMsg == NULL)
        {
            m_pErrorMsg = pNewErrorMsg;
        }
        else
        {
            // ErrorMsg list will be: original -> new
            m_pErrorMsg->appendToList(pNewErrorMsg);
        }
    }
    catch (std::bad_alloc& e)
    {
        printf("Exception: %s\n", e.what());
    }
}

/// @overload   BaseClassAPI::addErrorMsg(ERRORCODE   in_eErrorCode, 
///                                       const CHAR* in_szMessage,
///                                       ErrorMsg*    in_pNextErrorMsg);
/// @brief      Add ErrorMsg to the tail of the error message list
/// @param      in_eErrorCode is a 32bits error code
/// @param      in_szMessage is a char string for user specified message
/// @param      in_pNextErrorMsg is a pointer to next ErrorMsg for error message list
/// @exception  std::bad_alloc when memory allocation for new instance is failed
/// @return     Nothing
void
BaseClassAPI::addErrorMsg(ERRORCODE   in_eErrorCode, 
                          const CHAR* in_szMessage,
                          ErrorMsg*   in_pNextErrorMsg)

{
    ErrorMsg* pNewErrorMsg = NULL; 

    if (in_eErrorCode == NO_ERROR)
    {
        deleteErrorMsg(in_pNextErrorMsg);
        return;
    }

    try
    {
        pNewErrorMsg = new ErrorMsg(in_eErrorCode, in_szMessage);

        if ((m_pErrorMsg == NULL) || (m_pErrorMsg == in_pNextErrorMsg))
        {
            m_pErrorMsg = pNewErrorMsg;
        }
        else
        {
            // ErrorMsg list will be: original -> new
            m_pErrorMsg->appendToList(pNewErrorMsg);
        }

        // ErrorMsg list will be: new -> next
        pNewErrorMsg->appendToList(in_pNextErrorMsg);
    }
    catch (std::bad_alloc& e)
    {
        printf("Exception: %s\n", e.what());
    }
}

/// @fn         BaseClassAPI::clearErrorMsg(void) 
/// @brief      Clear all of error messages of API
/// @details    This will clear error messages which are not used by other APIs
/// @exception  Nothing
/// @return     Nothing
void
BaseClassAPI::clearErrorMsg(void)
{
    deleteErrorMsg(m_pErrorMsg);
    m_pErrorMsg = NULL;
}

/// @fn         BaseClassAPI::getClassName(void) 
/// @brief      Get the class name of API
/// @exception  Nothing
/// @return     A pointer to char string for class name
STRING 
BaseClassAPI::getClassName(void)
{
    return m_szClassName;
}

/// @fn         BaseClassAPI::getErrorMsg(void)
/// @brief      Get the error message list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassAPI::getErrorMsg(void)
{
    ErrorMsg* pHead = NULL;
    ErrorMsg* pNew = NULL;
    ErrorMsg* pPrevious = NULL;
    ErrorMsg* pTemp = m_pErrorMsg;

    while (pTemp)
    {
        pNew = new ErrorMsg(pTemp->getErrorCode(), pTemp->getDetailMessage());

        if (pTemp->isListHead())
        {
            pHead = pNew;
        }
        // pPrevious must not be a NULL pointer because the first item in the list
        // must be the head
        else
        {
            pPrevious->appendToList(pNew);
        }

        pPrevious = pNew;
        pTemp = pTemp->getNextErrorMsg();
    }

    clearErrorMsg();

    return pHead;
}

/// @fn         BaseClassAPI::returnErrorMsg(ERRORCODE    in_eErrorCode, 
///                                          const CHAR*  in_szMessage, 
///                                          ErrorMsg*    in_pNextErrorMsg);
/// @brief      Add ErrorMsg to the tail of the error message list and return the list
/// @param      in_eErrorCode is a 32bits error code
/// @param      in_szMessage is a char string for user specified message
/// @param      in_pNextErrorMsg is a pointer to next ErrorMsg for error message list
/// @exception  std::bad_alloc when memory allocation for new instance is failed
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassAPI::returnErrorMsg(ERRORCODE    in_eErrorCode, 
                             const CHAR*  in_szMessage, 
                             ErrorMsg*    in_pNextErrorMsg)
{
    ErrorMsg* pNewErrorMsg = NULL; 

    if (in_eErrorCode == NO_ERROR)
    {
        deleteErrorMsg(in_pNextErrorMsg);
        return NULL;
    }

    try
    {
        pNewErrorMsg = new ErrorMsg(in_eErrorCode, in_szMessage);

        // ErrorMsg list will be: new -> next
        pNewErrorMsg->appendToList(in_pNextErrorMsg);
    }
    catch (std::bad_alloc& e)
    {
        printf("Exception: %s\n", e.what());
    }
    
    return pNewErrorMsg;
}

