/// @file       LEDDAPI.hpp
/// @brief      LEDDAPI class
/// @details    Provide diagnostic and utility APIs for LEDs
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#include <unistd.h>
#include <stdlib.h>
#include <algorithm>
#include "LEDDAPI.hpp"
#include "LEDConfig.hpp"
#include "EthernetDDAPI.hpp"
#include "PSUDDAPI.hpp"
#include "SASDDAPI.hpp"
#include "ControllerDDAPI.hpp"
// Add include header file for enclosure leds for your project

#define ETHERNET_MODULE_NAME    "eth"

/// @fn         LEDDAPI::LEDDAPI(STRING in_szClassName) : BaseClassDAPI(in_szClassName)
/// @brief      LEDDAPI constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
LEDDAPI::LEDDAPI(STRING in_szClassName) : BaseClassDAPI(in_szClassName)
{

}

/// @fn         LEDDAPI::~LEDDAPI(void)
/// @brief      LEDDAPI destructor
/// @exception  Nothing
/// @return     Nothing
LEDDAPI::~LEDDAPI(void)
{

}

/// @fn         LEDDAPI::getDeviceList(list<string>  *out_pList)
///
/// @brief      Get the list of LEDs
/// @param      out_pLEDList is a pointer to a list for LED devices
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
LEDDAPI::getDeviceList(list<string>  *out_pList)
{
    ErrorMsg*       pErrorMsg = NO_ERROR_MSG;
    BaseClassDDAPI  *pDDAPI = NULL;
    list<string>    tempList;
    list<string>::iterator itStr;
    string          strTemp;
    INT32           i, s32ModuleCount = ARRAY_SIZE(moduleList);

    for (i = 0; i < s32ModuleCount; i++)
    {
        pDDAPI = createModuleClassByID((LED_MODULE_INDEX)moduleList[i]);

        if (pDDAPI == NULL)
        {
            continue;
        }

        // If there is error, ignore current module
        if (((pErrorMsg = pDDAPI->getErrorMsg()) != NO_ERROR_MSG) ||
            ((pErrorMsg = pDDAPI->getLEDList(&tempList)) != NO_ERROR_MSG) ||
            tempList.empty())
        {
            deleteErrorMsg(pErrorMsg);
            delete pDDAPI;
            continue;
        }

        switch (moduleList[i])
        {
            case ETHERNET:
                strTemp = ETHERNET_MODULE_NAME;
                strTemp += "_";
                for (itStr = tempList.begin(); itStr != tempList.end(); itStr++)
                    itStr->insert(0, strTemp);
                break;
            default:
                break;
        }

        if (!tempList.empty())
        {
            out_pList->insert(out_pList->end(), tempList.begin(), tempList.end());
            tempList.clear();
        }

        delete pDDAPI;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_LED_DAPI_GET_DEVICE_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         LEDDAPI::getLEDStatus(string in_strLEDName,
///                                   string *out_strStatus)
/// @brief      Get status for the given LED
/// @param      in_strLEDName is a string for the LED name
/// @param      out_strStatus is a pointer to a string for the status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
LEDDAPI::getLEDStatus(string in_strLEDName,
                      string *out_strStatus)
{
    string          strModuleName = getModuleName(in_strLEDName);
    BaseClassDDAPI  *pDDAPI = createModuleClassByName(strModuleName);
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    list<string>    tempList;

    if (pDDAPI)
    {
        if (strModuleName == ETHERNET_MODULE_NAME)
        {
            in_strLEDName.erase(0, strModuleName.size() + 1);
        }

        if ((pErrorMsg = pDDAPI->getLEDStatus(in_strLEDName, out_strStatus)))
        {
            return returnErrorMsg(ERROR_LED_DAPI_GET_LED_STATUS, NULL, pErrorMsg);
        }

        delete pDDAPI;

        return NO_ERROR_MSG;
    }

    return returnErrorMsg((ERRORCODE)(ERROR_LED_DAPI_GET_LED_STATUS | ERROR_NO_DEVICE_FOUND),
                          in_strLEDName.c_str());
}

/// @fn         LEDDAPI::getLEDStatus(string in_strLEDName,
///                                   string in_strStatus)
/// @brief      Set status to the given LED
/// @param      in_strLEDName is a string for the LED name
/// @param      in_strStatus is a string for the status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
LEDDAPI::setLEDStatus(string in_strLEDName,
                      string in_strStatus)
{
    string          strModuleName = getModuleName(in_strLEDName);
    BaseClassDDAPI  *pDDAPI = createModuleClassByName(strModuleName);
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    list<string>    tempList;

    if (pDDAPI)
    {
        if (strModuleName == ETHERNET_MODULE_NAME)
        {
            in_strLEDName.erase(0, strModuleName.size() + 1);
        }

        if ((pErrorMsg = pDDAPI->setLEDStatus(in_strLEDName, in_strStatus)))
        {
            return returnErrorMsg(ERROR_LED_DAPI_SET_LED_STATUS, NULL, pErrorMsg);
        }

        delete pDDAPI;

        return NO_ERROR_MSG;
    }

    return returnErrorMsg((ERRORCODE)(ERROR_LED_DAPI_SET_LED_STATUS | ERROR_NO_DEVICE_FOUND),
                          in_strLEDName.c_str());
}

/// @fn         LEDDAPI::createModuleClassByName(string in_strModuleName)
///
/// @brief      Create module's ddapi base class based on the module name
/// @param      in_strModuleName is a string for the module name
/// @exception  Nothing
/// @return     A pointer to BaseClassDDAPI
BaseClassDDAPI*
LEDDAPI::createModuleClassByName(string in_strModuleName)
{
    std::transform(in_strModuleName.begin(), in_strModuleName.end(), in_strModuleName.begin(), ::tolower);

    if (in_strModuleName == CONTROLLER_MODULE_NAME)
        return new ControllerDDAPI();
    if (in_strModuleName == "encl")
        // Create object for your project
        return NULL;
    if (in_strModuleName == ETHERNET_MODULE_NAME)
        return new EthernetDDAPI("EthernetDDAPI");
    if (in_strModuleName == "psu")
        return new PSUDDAPI();
    if (in_strModuleName == "sas")
        return new SASDDAPI("SASDDAPI");

    return NULL;
}

/// @fn         LEDDAPI::createModuleClassByID(LED_MODULE_INDEX in_eModuleIndex)
///
/// @brief      Create module's ddapi base class based on the module index
/// @param      in_eModuleIndex is an enumeration for the module
/// @exception  Nothing
/// @return     A pointer to BaseClassDDAPI
BaseClassDDAPI*
LEDDAPI::createModuleClassByID(LED_MODULE_INDEX in_eModuleIndex)
{
    switch (in_eModuleIndex)
    {
        case CONTROLLER:
            return new ControllerDDAPI();
        case ENCLOSURE:
            // Create object for your project
            return NULL;
        case ETHERNET:
            return new EthernetDDAPI("EthernetDDAPI");
        case PSU:
            return new PSUDDAPI();
        case SAS:
            return new SASDDAPI("SASDDAPI");
        default:
            return NULL;
    }
}

/// @fn         LEDDAPI::getModuleName(string in_strLEDName)
///
/// @brief      Get module name from the given LED name
/// @param      in_strLEDName is a string for the LED name
/// @exception  Nothing
/// @return     A string for the module name
string
LEDDAPI::getModuleName(string in_strLEDName)
{
    INT32       s32End;
    std::locale loc;
    std::string::iterator strIt = in_strLEDName.begin();

    for (s32End = 0; strIt != in_strLEDName.end(); strIt++, s32End++)
    {
        if (!std::isalpha(*strIt, loc))
        {
            return in_strLEDName.substr(0, s32End);
        }
    }

    return "";
}
