/// @file       MemoryDAPI.hpp
/// @brief      MemoryDAPI class
/// @details    Provide diagnostic and utility APIs for memory module
/// @version    1.0.0
/// @date       2014.01.29
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef MEMORY_DAPI_HPP
#define MEMORY_DAPI_HPP

#include "BaseClassDAPI.hpp"
#include "MemoryDDAPI.hpp"
#include "DIMMInformation.hpp"
#include "ECCInformation.hpp"
#include "MemoryRange.hpp"

/// @emun       CHECKSUM_TYPES
///             List all of supported checksum types 
typedef enum {
    CHECKSUM_8BITS = 0,
} CHECKSUM_TYPES;

#define TOTAL_ALLOCATABLE_MEMORY_IN_MB    1280

/// @class      MemoryDAPI MemoryDAPI.hpp "MemoryDAPI.hpp"
/// @brief      Provide diagnostic and utility APIs for memory module
/// @param      m_pMemoryDDAPI is a pointer to MemoryDDAPI
/// @param      m_s32CPUAmount is a 32 bits value for the amount of CPU
/// @param      m_reservedMemoryList is a list of memory ranges
/// @param      m_DIMMInfoList is a list for DIMM information
class MemoryDAPI : public BaseClassDAPI
{
    public: 
                            MemoryDAPI(STRING in_szClassName);
        virtual             ~MemoryDAPI(void);


        ErrorMsg*           addressLineTest(PHYSICAL_ADDRESS_T in_startAddress,
                                            PHYSICAL_ADDRESS_T in_endAddress,
                                            INT32              in_s32ElementSize,
                                            UINT64             in_u64UserTestPattern);

        ErrorMsg*           addressLineTest(PHYSICAL_ADDRESS_T in_startAddress,
                                            UINT64             in_u64TestSize,
                                            INT32              in_s32Controller,
                                            INT32              in_s32Channel,
                                            INT32              in_s32Rank,
                                            INT32              in_s32ElementSize,
                                            UINT64             in_u64UserTestPattern);


        ErrorMsg*           checksumTest(PHYSICAL_ADDRESS_T in_startAddress,
                                         PHYSICAL_ADDRESS_T in_endAddress,
                                         string             in_strChecksum,
                                         CHECKSUM_TYPES     in_eChecksumType);

        ErrorMsg*           clearECCReport(void);

        ErrorMsg*           checkDeviceBySMBIOS(list<string>* inout_pDeviceList);

        ErrorMsg*           dataLineTest(PHYSICAL_ADDRESS_T in_startAddress,
                                         PHYSICAL_ADDRESS_T in_endAddress,
                                         INT32              in_s32DataLineAmount);

        ErrorMsg*           dataLineTest(PHYSICAL_ADDRESS_T in_startAddress,
                                         UINT64             in_u64TestSize,
                                         INT32              in_s32Controller,
                                         INT32              in_s32Channel,
                                         INT32              in_s32Rank,
                                         INT32              in_s32DataLineAmount);

        ErrorMsg*           getDeviceList(list<string>* out_pDeviceList,
                                          BOOL          in_bDetectedOnly = TRUE);

        ErrorMsg*           getECCReport(list<ECCInformation>* out_pECCInfoList);

        ErrorMsg*           getInformation(string           in_strDIMM,
                                           DIMMInformation* out_pDIMMInfo);

        ErrorMsg*           getMemoryBuffer(UINT32              in_u32RequestedSize,
                                            SYSTEM_ADDRESS_T*   out_pBufferAddress);

        ErrorMsg*           getTestableMemory(list<MemoryRange>* out_pTestableMemoryList,
                                              PHYSICAL_ADDRESS_T in_u64StartAddress = 0,
                                              PHYSICAL_ADDRESS_T in_u64EndAddress = 0);

        UINT32              getTestBlockSize(void);

        ErrorMsg*           eccErrorDetectionTest(PHYSICAL_ADDRESS_T in_startAddress,
                                                  PHYSICAL_ADDRESS_T in_endAddress);

        ErrorMsg*           eccErrorDetectionTest(PHYSICAL_ADDRESS_T in_startAddress,
                                                  UINT64             in_u64TestSize,
                                                  INT32              in_s32Controller,
                                                  INT32              in_s32Channel,
                                                  INT32              in_s32Rank);

        ErrorMsg*           marchCTest(PHYSICAL_ADDRESS_T in_startAddress,
                                       PHYSICAL_ADDRESS_T in_endAddress,
                                       INT32              in_s32ElementSize,
                                       UINT64             in_u64UserTestPattern);

        ErrorMsg*           patternTest(PHYSICAL_ADDRESS_T in_startAddress,
                                        PHYSICAL_ADDRESS_T in_endAddress,
                                        INT32              in_s32ElementSize,
                                        UINT64             in_u64UserTestPattern);

        ErrorMsg*           patternTest(PHYSICAL_ADDRESS_T in_startAddress,
                                        PHYSICAL_ADDRESS_T in_endAddress);

        ErrorMsg*           randomAddressTest(PHYSICAL_ADDRESS_T    in_startAddress,
                                              PHYSICAL_ADDRESS_T    in_endAddress,
                                              UINT32                in_u32TestTime);

        UINT8               readMemory8(PHYSICAL_ADDRESS_T  in_physicalAddress);

        UINT16              readMemory16(PHYSICAL_ADDRESS_T in_physicalAddress);

        UINT32              readMemory32(PHYSICAL_ADDRESS_T in_physicalAddress);

        UINT64              readMemory64(PHYSICAL_ADDRESS_T in_physicalAddress);

        ErrorMsg*           releaseMemoryBuffer(SYSTEM_ADDRESS_T in_bufferAddress);
        
        ErrorMsg*           writeMemory8(PHYSICAL_ADDRESS_T in_physicalAddress, 
                                         UINT8              in_u8Value);

        ErrorMsg*           writeMemory16(PHYSICAL_ADDRESS_T in_physicalAddress, 
                                          UINT16             in_u16Value);

        ErrorMsg*           writeMemory32(PHYSICAL_ADDRESS_T in_physicalAddress, 
                                          UINT32             in_u32Value); 

        ErrorMsg*           writeMemory64(PHYSICAL_ADDRESS_T in_physicalAddress, 
                                          UINT64             in_u64Value);         
    protected:

    private:
        MemoryDDAPI*        m_pMemoryDDAPI;

        PHYSICAL_ADDRESS_T  alignMemoryAddress(PHYSICAL_ADDRESS_T   in_physicalAddress,
                                               INT32                in_s32ElementSize);

        ErrorMsg*           executeMarchCTest(PHYSICAL_ADDRESS_T in_startAddress,
                                              PHYSICAL_ADDRESS_T in_endAddress,
                                              INT32              in_s32ElementSize,
                                              UINT64             in_u64UserTestPattern);

        MemoryRange         getOverlapMemoryRange(MemoryRange   in_memoryRangeA,
                                                  MemoryRange   in_memoryRangeB);

        ErrorMsg*           getRankRange(list<MemoryRange>* out_pRangeList);

        ErrorMsg*           initializeMemoryLayoutFile(void);

        ErrorMsg*           initializeTestableMemoryFile(void);

        ErrorMsg*           verifyMemoryBlock(PHYSICAL_ADDRESS_T    in_startAddress,
                                              PHYSICAL_ADDRESS_T    in_endAddress,
                                              UINT8                 in_u8UserTestPattern);
        
        ErrorMsg*           verifyMemoryBlock(PHYSICAL_ADDRESS_T    in_startAddress,
                                              PHYSICAL_ADDRESS_T    in_endAddress,
                                              UINT16                in_u16UserTestPattern);

        ErrorMsg*           verifyMemoryBlock(PHYSICAL_ADDRESS_T    in_startAddress,
                                              PHYSICAL_ADDRESS_T    in_endAddress,
                                              UINT32                in_u32UserTestPattern);  

        ErrorMsg*           verifyMemoryBlock(PHYSICAL_ADDRESS_T    in_startAddress,
                                              PHYSICAL_ADDRESS_T    in_endAddress,
                                              UINT64                in_u64UserTestPattern); 
        
        ErrorMsg*           verfiyMemoryBlockByMarchC(PHYSICAL_ADDRESS_T in_startAddress,
                                                      PHYSICAL_ADDRESS_T in_endAddress,
                                                      INT32              in_s32ElementSize,
                                                      UINT64             in_u64VerifyTestPattern,
                                                      UINT64             in_u64WriteTestPattern);

        INT32               m_s32CPUAmount;
        
        list<MemoryRange>   m_reservedMemoryList;

        list<DIMMInformation>   m_DIMMInfoList;
};

#endif

