/// @file       MemoryDAPI_GetECCReport.cpp
/// @brief      Provide ECC report retrieve function for MemoryDAPI
/// @details    Get ECC error status for each DIMM
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include <list>
#include <string>

#include "MemoryDAPI.hpp"
#include "CommonFunctions.hpp"
#include "I2CDAPI.hpp"

using::std::list;

/// @fn         MemoryDAPI::clearECCReport(void)
/// @brief      Clear ECC report for each DIMM
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
MemoryDAPI::clearECCReport(void)
{
    ErrorMsg*   pErrorMsg = NULL;

    if ((pErrorMsg = m_pMemoryDDAPI->clearECCReport()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_MEMORY_DAPI_CLEAR_ECC_REPORT, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}

/// @fn         MemoryDAPI::getECCReport(list<ECCInformation>* out_pECCInfoList)
/// @brief      Get ECC report for each DIMM
/// @details    ECC report will be: [CORRECTED] DIMMID0:ErrorCount0 DIMMID1:ErrorCount1 ...
/// @param      out_pECCInfoList is a pointer to a list of ECC information
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg* 
MemoryDAPI::getECCReport(list<ECCInformation>* out_pECCInfoList)
{
    ErrorMsg*       pErrorMsg = NULL;
    list<string>    deviceList;
    INT32           s32Channels = 0, s32Slots = 0;
    INT32           s32Index, s32Counter;
    list<string>::iterator          itDev;
    list<ECCInformation>::iterator  itECCInfo;
    
    if (out_pECCInfoList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_MEMORY_DAPI_GET_ECC_REPORT | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    out_pECCInfoList->clear();

    if ((pErrorMsg = m_pMemoryDDAPI->getECCReport(out_pECCInfoList)))
    {
        return returnErrorMsg(ERROR_MEMORY_DAPI_GET_ECC_REPORT, NULL, pErrorMsg);
    }
    
    if (out_pECCInfoList->empty())
    {
        return NO_ERROR_MSG;
    }
    
    // Get controller information
    if ((pErrorMsg = m_pMemoryDDAPI->getControllerChannelCount(&s32Channels)))
    {
        return returnErrorMsg(ERROR_MEMORY_DAPI_GET_ECC_REPORT, NULL, pErrorMsg);
    }

    if ((pErrorMsg = m_pMemoryDDAPI->getControllerSlotCount(&s32Slots)))
    {
        return returnErrorMsg(ERROR_MEMORY_DAPI_GET_ECC_REPORT, NULL, pErrorMsg);
    }
    
    if ((pErrorMsg = getDeviceList(&deviceList, FALSE)))
    {
        return returnErrorMsg(ERROR_MEMORY_DAPI_GET_ECC_REPORT, NULL, pErrorMsg);
    }

    // Find out corresponding DIMM for each ECC information
    itECCInfo = out_pECCInfoList->begin();
    while (itECCInfo != out_pECCInfoList->end())
    {
        s32Index = itECCInfo->m_s32ControllerID * s32Channels * s32Slots;
        s32Index += itECCInfo->m_s32ChannelID * s32Slots;
        s32Index += itECCInfo->m_s32SlotID;
        s32Counter = 0;

        itDev = deviceList.begin();
        while (itDev != deviceList.end())
        {
            if (s32Index == s32Counter)
            {
                itECCInfo->m_strDIMMName = *itDev;
                break;
            }
            s32Counter++;
            itDev++;
        }

        itECCInfo++;
    }

    return NO_ERROR_MSG;
}
