/// @file       DIMMInformation.hpp
/// @brief      DIMMInformation class
/// @details    Provide class of DIMM information for memory module
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef DIMM_INFORMATION_HPP
#define DIMM_INFORMATION_HPP

#include <list>

#include "MemoryRange.hpp"

using::std::list;

/// @class      DIMMInformation DIMMInformation.hpp "DIMMInformation.hpp"
/// @brief      Provide DIMM information for each DIMM
/// @param      m_s32ID is a 32 bits value for DIMM ID
/// @param      m_s32CPUID is a 32 bits value for CPU ID
/// @param      m_s32ChannelID is a 32 bits value for channel ID
/// @param      m_strLocator is a string for locator name
/// @param      m_bValid is boolean value to indicate whether DIMM information is valid or not
/// @param      m_strMemoryType is a string for memory type
/// @param      m_strModuleType is a string for module type
/// @param      m_strMaxSpeed is a string for supported maximum speed
/// @param      m_strConfiguredSpeed is a string for the configured speed
/// @param      m_strSize is a string for memory size
/// @param      m_strRank is a string for rank amount
/// @param      m_s32StartRank is a 32 bits value for start rank ID of the memory channel
/// @param      m_s32EndRank is a 32 bits value for end rank ID of the memory channel
/// @param      m_supportVoltageList is a list of strings for supported voltages
/// @param      m_strConfiguredVoltage is a string for the configured voltage
/// @param      m_strModuleManufacturer is a string for module's manufacturer
/// @param      m_strDRAMManufacturer is a string for DRAM's manufacturer
/// @param      m_strSerialNumber is a string for serial number
/// @param      m_strPartNumber is a string for part number
/// @param      m_logicalMemoryRange is a memory range information
/// @param      m_mapMemoryRangeList is a list of system mapped memory range information
class DIMMInformation
{
    public:
                            DIMMInformation() 
                            {
                                m_s32ID = 0;
                                m_bValid = FALSE;   
                            }
        virtual             ~DIMMInformation() {}

                            /// @fn         DIMMInformation::clear(void)                               
                            /// @brief      Clear all of DIMM information
                            /// @detailed   This API will not reset m_s32ID
                            /// @exception  Nothing
                            /// @return     Nothing
        void                clear(void)
                            {
                                m_s32CPUID = -1;
                                m_s32ChannelID = -1;
                                m_strLocator = "unknown";
                                m_bValid = FALSE;
                                m_strMemoryType.clear();
                                m_strModuleType.clear();
                                m_strMaxSpeed.clear();
                                m_strConfiguredSpeed = "unknown";
                                m_strSize.clear();
                                m_strRank.clear();
                                m_s32StartRank = -1;
                                m_s32EndRank = -1;
                                m_supportVoltageList.clear();
                                m_strConfiguredVoltage = "unknown";
                                m_strModuleManufacturer.clear();
                                m_strDRAMManufacturer.clear();
                                m_strPartNumber.clear();
                                m_logicalMemoryRange.m_startAddress = 0;
                                m_logicalMemoryRange.m_endAddress = 0;
                                m_mapMemoryRangeList.clear();
                            }
        
        INT32               m_s32ID;
        INT32               m_s32CPUID;
        INT32               m_s32ChannelID;
        string              m_strLocator;
        BOOL                m_bValid;
        string              m_strMemoryType;
        string              m_strModuleType;
        string              m_strMaxSpeed;
        string              m_strConfiguredSpeed;
        string              m_strSize;
        string              m_strRank;
        INT32               m_s32StartRank;
        INT32               m_s32EndRank;
        list<string>        m_supportVoltageList;
        string              m_strConfiguredVoltage;
        string              m_strModuleManufacturer;
        string              m_strDRAMManufacturer;
        string              m_strSerialNumber;
        string              m_strPartNumber;
        MemoryRange         m_logicalMemoryRange;
        list<MemoryRange>   m_mapMemoryRangeList;
    protected:
    private:
};

#endif
