/// @file       NVMeDDAPI.cpp
/// @brief      NVMeDDAPI class
/// @details    Provide low layer diagnostic and utility APIs for NVMe module
/// @version    1.0.0
/// @date       2014.02.21
/// @author     Jerry Lee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#include <string.h>
#include "NVMeDDAPI.hpp"
#include "MemoryDDAPI.hpp"

#define min(x, y) (x) > (y) ? (y) : (x)

#define MAX_NVME_BOOT_DEVICE 2
#define MAX_NVME_DRIVE 0
static NVMeDriveMap nvmeBootControllerID[] = {
    {"1c.4", 0},{"1c.6", 1}
};

vector<NVMe_DEVICE_INFO> NVMeDDAPI::nvmeList;
/// @fn         NVMeDDAPI::NVMeDDAPI(STRING className) : BaseClassDDAPI(className)
/// @brief      NVMeDDAPI constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
NVMeDDAPI::NVMeDDAPI(STRING className) : BaseClassDDAPI(className)
{
    ErrorMsg* m_errorMsg = NO_ERROR_MSG;

	if(nvmeList.empty())
	{
	    m_errorMsg = addNVMeController(); // search and add IOC information on the host
		if(m_errorMsg != NO_ERROR_MSG)
		{
	        addErrorMsg(ERROR_NVME_DDAPI_ERRORCODE, NULL, m_errorMsg);
		}

	    if(nvmeList.empty())
	    {
	        addErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_ERRORCODE | ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR), "Host has no NVME controller");
	    }

	    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
	    {
	        m_errorMsg = setDeviceNodeByController(iter - nvmeList.begin());
			if(m_errorMsg != NO_ERROR_MSG)
			{
				addErrorMsg(ERROR_NVME_DDAPI_ERRORCODE, NULL, m_errorMsg);
				break;

			}

	        m_errorMsg = doIOCTLCommand((iter - nvmeList.begin()), NVMeIOCTL::NAMESPACE_INFORMATION, &iter->idNs);
	        if(m_errorMsg != NO_ERROR_MSG)
	        {
	            addErrorMsg (ERROR_NVME_DDAPI_ERRORCODE, NULL, m_errorMsg);
				break;

	        }
		}
	}

}

/// @fn         NVMeDDAPI::~NVMeDDAPI(void)
/// @brief      NVMeDDAPI destructor
/// @exception  Nothing
/// @return     Nothing
NVMeDDAPI::~NVMeDDAPI(void)
{
}

/// @fn         NVMeDAPI::addNVMeController(void)
/// @brief      add all IOCs' information on the host
/// @param      None
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::addNVMeController(void)
{
    NVMe_DEVICE_INFO 		nvmeInfo;
    vector<PCIInformantion>	pciInfo;
    ErrorMsg 				*errorMsg = NULL;
	PCIDDAPI 				pciDDAPI("NVMePCIDriver");
	PFA               		bdf;
	
    if ((errorMsg = pciDDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_NVME_DDAPI_CONSTRUCTOR, NULL, errorMsg);
    }
	
    errorMsg = pciDDAPI.getPciInfoByClass(PCI_NVME, &pciInfo);
	
    if(errorMsg)
        return returnErrorMsg(ERROR_NVME_DDAPI_CONSTRUCTOR, NULL, errorMsg);

    for (vector<PCIInformantion>::iterator iter = pciInfo.begin(); iter != pciInfo.end(); iter++)
    {
        nvmeInfo.domindBusDeviceFunction = Int2HexStr(iter->u16Domain,4) + ":"
                + Int2HexStr(iter->u16Bus) + ":" + Int2HexStr(iter->u16Device) + "."
                + Int2HexStr(iter->u8Function, 1);

		//BDF
		bdf.u16Domain = iter->u16Domain;
		bdf.u16Bus = iter->u16Bus;
		bdf.u16Device = iter->u16Device;
		bdf.u8Function = iter->u8Function;
		//PCI link speed
		nvmeInfo.pciSpeed = pciDDAPI.GetFloatDataByID(bdf, PCI_DATA_TYPE_CURRENT_SPEED);
		//PCI width
		nvmeInfo.pciWidth = pciDDAPI.GetDataByBDF(bdf, PCI_DATA_TYPE_CURRENT_WIDTH);
		//Bar Info
		pciDDAPI.getBarByBDF(iter->u16Domain, iter->u16Bus, iter->u16Device, iter->u8Function, 0, &(nvmeInfo.barAddr));

		for(int i = 0; i < MAX_NVME_BOOT_DEVICE; i++)
        {
            if(getNVMeAttachedRootPort(nvmeInfo.domindBusDeviceFunction).find(nvmeBootControllerID[i].rootBDF) != string::npos)
            {
                nvmeInfo.wellKnowName = "NVME-BOOT" + Int2Str(nvmeBootControllerID[i].numOfHardDrive); // NVMe boot device
            }
        }
        nvmeList.push_back(nvmeInfo);
    }

    std::sort(nvmeList.begin(), nvmeList.end(), sortNVMEcontroller);

    if(errorMsg)
        return returnErrorMsg(ERROR_NVME_DDAPI_CONSTRUCTOR, NULL, errorMsg);

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::doIOCTLCommand
/// @brief      execute NVMe controller IOCTL command
/// @param      NVMe logical ID
/// @param      NVMe_IOCTL_REQUEST
/// @param      The returned value
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::doIOCTLCommand(UINT8 controllerID, NVMeIOCTL::IOCTL_OP_CODE ioctlCommand, void* returnedValue)
{
    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_EXECUTE_IOCTL | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
			NVMeIOCTL 	nvmeIOCTL((iter->blockdeviceNodeName).c_str());
			
            if((m_errorCode = nvmeIOCTL.allocateIOCTLArgument(ioctlCommand)) == NO_ERROR)
            {
                if ((m_errorCode = nvmeIOCTL.setIOCTLOperationArgument(ioctlCommand, returnedValue)) == NO_ERROR)
                // separate ioctl command here to distinguish the command has return data or customized argument
                switch(ioctlCommand)
                {
                	//!!
                    case NVMeIOCTL::EXECUTE_NVME_RESET:
                        nvmeIOCTL.setFileHandle((iter->chardeviceNodeName).c_str());
                        m_errorCode = nvmeIOCTL.executeIOCTL(ioctlCommand);
                        break;
                    case NVMeIOCTL::CONTROLLER_INFORMATION:
                    case NVMeIOCTL::NAMESPACE_INFORMATION:
                    case NVMeIOCTL::FIRMWARE_LOG:
                    case NVMeIOCTL::SMART_LOG:
                    case NVMeIOCTL::ERROR_LOG:
                    case NVMeIOCTL::FIRMWARE_ACTIVATE:
                        m_errorCode = nvmeIOCTL.executeIOCTL(ioctlCommand);

                        break;
                    case NVMeIOCTL::FORMAT:
                    case NVMeIOCTL::WRITE_ZERO:
                        if((m_errorCode = nvmeIOCTL.setIOCTLOperationArgument(ioctlCommand, returnedValue)) == NO_ERROR)
                        {
                            m_errorCode = nvmeIOCTL.executeIOCTL(ioctlCommand);
                        }

                        break;

                    case NVMeIOCTL::NAMESPACE_ID:
					case NVMeIOCTL::GET_FEATURES:
					case NVMeIOCTL::SET_FEATURES:
                        m_errorCode = nvmeIOCTL.executeIOCTL(ioctlCommand, returnedValue);

                        break;
                    case NVMeIOCTL::UnspportIOCTLCommand:
                        m_errorCode = ERROR_IOCTL_INVALID_COMMAND_CODE;

                        break;
                    case NVMeIOCTL::FIRMWARE_DOWNLOAD:
                        break;
                }
            }
            break;
        }
    }

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_EXECUTE_IOCTL | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::getControllerList
/// @brief      Get NVMe information list on the host
/// @param      NVMe logical ID
/// @param      A pointer to NVMe controller list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::getControllerList(vector<NVMe_DEVICE_INFO>* returnedList)
{
    if(nvmeList.empty())
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_GET_NVME_LIST | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);

    *returnedList = nvmeList;

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}

/// @fn         NVMeDDAPI::setDeviceNodeByController(UINT8 controllerID)
/// @brief      set specified NVME device node name
/// @param      NVMe logical ID
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::setDeviceNodeByController(UINT8 controllerID)
{
    string pciBDF;
    CHAR* charDirToRead = (CHAR *)"/sys/class/nvme/";
    CHAR* blockDirToRead = (CHAR *)"/sys/block/";
    DIR* chardir = opendir(charDirToRead);
    DIR* blockdir = opendir(blockDirToRead);
    struct dirent *dirent;
    struct stat fileAttribute;
    CHAR tempBuffer[256];
    CHAR fileName[256] = "";

    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }
    if(blockdir == NULL || chardir == NULL)
    {
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_OPEN_DIRECTORY), NULL, NULL);
    }
    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
            pciBDF = iter->domindBusDeviceFunction;
            while((dirent = readdir(chardir)) != NULL)
            {
                snprintf(fileName, sizeof(fileName), "%s%s", charDirToRead, dirent->d_name);
                lstat(fileName, &fileAttribute);

                if(S_ISDIR(fileAttribute.st_mode))
                {
                    // This directory dirent is another directory
                }
                else if(S_ISLNK(fileAttribute.st_mode))
                {
                    INT32 linkLength = readlink(fileName, tempBuffer, sizeof(tempBuffer) - 1);
                    if(linkLength != -1)
                    {
                        tempBuffer[linkLength] = '\0';
                    }
                    else
                    {
                        closedir(chardir);
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_READ_LINK_ERROR), NULL, NULL);
                    }

                    CHAR tempString[256] = "";
                    snprintf(tempString, sizeof(tempString), "%s", pciBDF.c_str());
                    CHAR *chpAddress = strstr(tempBuffer, tempString);

                    if(chpAddress == NULL)
                    {
                    continue;
                    }
                    iter->nativeName = (string)dirent->d_name;
                    iter->chardeviceNodeName = (string)"/dev/" + dirent->d_name;
                }
                else if(S_ISREG(fileAttribute.st_mode))
                {
                  // This dirent is a regular filename
                }
            }
            closedir(chardir);

            while((dirent = readdir(blockdir)) != NULL)
            {
                snprintf(fileName, sizeof(fileName), "%s%s", blockDirToRead, dirent->d_name);
                lstat(fileName, &fileAttribute);

                if(S_ISDIR(fileAttribute.st_mode))
                {
                    // This directory dirent is another directory
                }
                else if(S_ISLNK(fileAttribute.st_mode))
                {
                    INT32 linkLength = readlink(fileName, tempBuffer, sizeof(tempBuffer) - 1);
                    if(linkLength != -1)
                    {
                        tempBuffer[linkLength] = '\0';
                    }
                    else
                    {
                        closedir(blockdir);
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_READ_LINK_ERROR), NULL, NULL);
                    }

                    CHAR tempString[256] = "";
                    snprintf(tempString, sizeof(tempString), "%s", pciBDF.c_str());
                    CHAR *chpAddress = strstr(tempBuffer, tempString);

                    if(chpAddress == NULL)
                    {
                    continue;
                    }

                    iter->blockdeviceNodeName = (string)"/dev/" + dirent->d_name;
                }
                else if(S_ISREG(fileAttribute.st_mode))
                {
                  // This dirent is a regular filename
                }
            }
            closedir(blockdir);
            break;
        }
    }

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::downloadFirmware(UINT8 controllerID, void* buffer, UINT32 size)
/// @brief      download image to the requested NVMe device
/// @param      IOC ID
/// @param      image buffer
/// @param      image buffer size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::downloadFirmware(UINT8 controllerID, void* buffer, UINT32 size)
{
    UINT32 					tempSize;
    nvme_fw_download_info 	fwInfo;
	
    memset(&fwInfo, 0, sizeof(fwInfo));

    fwInfo.fw_buf = buffer;
    tempSize = size;
	
    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_DOWNLOAD_FW | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
			NVMeIOCTL 	nvmeIOCTL((iter->blockdeviceNodeName).c_str());
			
            if((m_errorCode = nvmeIOCTL.allocateIOCTLArgument(NVMeIOCTL::FIRMWARE_DOWNLOAD)) == NO_ERROR)
            {
                while (tempSize > 0) {
                    fwInfo.xfer = min(4096, tempSize);
                    nvmeIOCTL.setIOCTLOperationArgument(NVMeIOCTL::FIRMWARE_DOWNLOAD, &fwInfo);
                    if((m_errorCode = nvmeIOCTL.executeIOCTL(NVMeIOCTL::FIRMWARE_DOWNLOAD))!= NO_ERROR)
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_DOWNLOAD_FW | m_errorCode), NULL, NULL);

                    fwInfo.fw_buf   += fwInfo.xfer;
                    tempSize        -= fwInfo.xfer;
                    fwInfo.offset   += fwInfo.xfer;
                }
            }
            break;
        }
    }

    return NO_ERROR_MSG;
}

bool
NVMeDDAPI::sortNVMEcontroller(const NVMe_DEVICE_INFO &left, const NVMe_DEVICE_INFO &right)
{
    CHAR    *pchFirst, *pchSecond;
    UINT32  firstNum, secondNum;
    

    //Compare previous number
    pchFirst = (CHAR*)left.wellKnowName.c_str();
    pchSecond = (CHAR*)right.wellKnowName.c_str();
    while((*pchFirst != 0) && (*pchSecond != 0))
    {
        if (*pchFirst == *pchSecond)
        {
            pchFirst++;
            pchSecond++;
        }
        else
        {
            if ((*pchFirst < 0x30) || (*pchFirst > 0x39)) //character
            {
                return left.wellKnowName < right.wellKnowName;
            }
            else
            {
                firstNum = atoi(pchFirst);
                secondNum = atoi(pchSecond);
                return firstNum < secondNum;
            }
        }
    }

    return left.wellKnowName < right.wellKnowName;
}

/// @fn         NVMeDDAPI::resetSubSystem(UINT8 controllerID)
/// @brief      reset specific NVME subsystem
/// @param      NVME ID
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::resetSubSystem(UINT8 controllerID)
{
	ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
	UINT32			u32NSSROffset = 0x20;
    MemoryDDAPI		memoryDDAPI("NVMeMemDDAPI");

    if ((pErrorMsg = memoryDDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_NVME_DDAPI_RESET_SUBSYS, NULL, pErrorMsg);
    }

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
        	u32NSSROffset += iter->barAddr;

			memoryDDAPI.write(u32NSSROffset, (UINT64)NSSRC_VALUE);
			sleep(3);
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_NVME_DDAPI_RESET_SUBSYS, NULL, pErrorMsg);
    }
    
    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::getNVMeAttachedRootPort(string NVMeBDF)
/// @brief      get NVMe PCI bus topology
/// @param      NVME BDF
/// @exception  Nothing
/// @return     Root Port string
string
NVMeDDAPI::getNVMeAttachedRootPort(string NVMeBDF)
{
    CHAR* classDirToRead = (CHAR *)"/sys/class/nvme/";
    DIR* dir = opendir(classDirToRead);
    struct dirent *dirent;
    struct stat fileAttribute;
    CHAR linkString[256];
    CHAR fileName[256] = "";
    CHAR * pch;
    string tokenString[10];
    string rootPortString = "";
    UINT8 index = 0;

    // search NVME class to get bus topology information
    while((dirent = readdir(dir)) != NULL)
    {
        snprintf(fileName, sizeof(fileName), "%s%s", classDirToRead, dirent->d_name);
        lstat(fileName, &fileAttribute);

        if(S_ISDIR(fileAttribute.st_mode))
        {
            // This directory dirent is another directory
        }
        else if(S_ISLNK(fileAttribute.st_mode))
        {
            INT32 linkLength = readlink(fileName, linkString, sizeof(linkString) - 1);
            if(linkLength != -1)
            {
                linkString[linkLength] = '\0';
            }
            else
            {
                closedir(dir);
                return "";
            }

            if(strstr(linkString, NVMeBDF.c_str()))
            {
                index = 0;
                pch = strtok(linkString, "/");
                while (pch != NULL)
                {
                    tokenString[index].assign((const char*)pch);
                    pch = strtok (NULL, "/");
                    index++;
                }
            }
            else
                continue;
        }
        else if(S_ISREG(fileAttribute.st_mode))
        {
          // This dirent is a regular filename
        }
    }
    closedir(dir);

    for(index = 0; index < 10; index++)
    {
        if(tokenString[index] != "")
        {
            if(tokenString[index] == NVMeBDF)
            {
                rootPortString = tokenString[index - 1];
                break;
            }
        }
    }

    return rootPortString;
}
