/// @file       NVSRAMDDAPI.cpp
/// @brief      NVSRAMDDAPI class
/// @details    Provide low layer diagnostic and utility APIs for SAS module
/// @version    1.0.0
/// @date       2015.11.19
/// @author     Jerry Lee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include <string.h>
#include <time.h>
#include "NVSRAMDDAPI.hpp"

#define FPGA_PCI_MODE 1
#define NVSRAM_VENDOR_ID 0x1275
#define NVSRAM_DEVICE_ID 0x7110
#define NVSRAM_ELEMENT_SIZE 64

/// @fn         NVSRAMDDAPI::NVSRAMDDAPI(STRING className) : BaseClassDDAPI(className)
/// @brief      NVSRAMDDAPI constructor
/// @param      className is a char string for class name
/// @exception  Nothing
/// @return     Nothing
NVSRAMDDAPI::NVSRAMDDAPI(STRING ClassName) : BaseClassDDAPI(ClassName)
{
    // initialize here
    if((pciDDAPI = new PCIDDAPI("PCI_DRVR")) == NULL)
    {
        addErrorMsg((ERRORCODE)(ERROR_NVSRAM_DDAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR), "Create PCIDDAPI fail");
    }
    if((memDDAPI = new MemoryDDAPI("MEMORY_DRVR")) == NULL)
    {
        addErrorMsg((ERRORCODE)(ERROR_NVSRAM_DDAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR), "Create MEMORY_DDAPI fail");
    }
    nvsramBARSize = (UINT64)NULL;
    nvsramBaseAddress = (PHYSICAL_ADDRESS_T)NULL;

    addNVSRAMDevice(); // search and add NVSRAM information on the host

#ifndef FPGA_PCI_MODE
    nvsramBaseAddress = NVSRAM_BASE_ADDRESS;
#endif
}

/// @fn         NVSRAMDDAPI::~NVSRAMDDAPI(void)
/// @brief      NVSRAMDDAPI destructor
/// @exception  Nothing
/// @return     Nothing
NVSRAMDDAPI::~NVSRAMDDAPI(void)
{
    // delete object we created
    if(pciDDAPI != NULL)
        delete pciDDAPI;
    if(memDDAPI != NULL)
        delete memDDAPI;
}

/// @fn         SASDAPI::addIOCController(void)
/// @brief      add all IOCs' information on the host
/// @param      None
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::addNVSRAMDevice(void)
{
    vector<PCIInformantion> pciInfo;
    NVSRAM_RESOURCE_INFO nvsramInfo;
    PFA nvsramPFA;
    UINT8 nvsramNum = 0;
    ErrorMsg* errorMsg = NULL;

    errorMsg = pciDDAPI->getPciInfoByClass(OTHER_08, &pciInfo);

    if(errorMsg)
    {
        return returnErrorMsg(ERRORCODE(ERROR_NVSRAM_DDAPI_CONSTRUCTOR), NULL, errorMsg);
    }
    for(vector<PCIInformantion>::const_iterator iter = pciInfo.begin(); iter != pciInfo.end();
            iter++)
    {
        if((iter->u16VendorID == NVSRAM_VENDOR_ID) && (iter->u16DeviceID == NVSRAM_DEVICE_ID))
        {
            nvsramPFA.u16Domain = iter->u16Domain;
            nvsramPFA.u16Bus = iter->u16Bus;
            nvsramPFA.u16Device = iter->u16Device;
            nvsramPFA.u8Function = 0;

            errorMsg = pciDDAPI->getBarByBDF(nvsramPFA, 1, (UINT32*) &nvsramInfo.BaseAddress,
                    (UINT32*) &nvsramInfo.BARSize);

            if(errorMsg)
            {
                return returnErrorMsg(ERRORCODE(ERROR_NVSRAM_DDAPI_CONSTRUCTOR), NULL, errorMsg);
            }

            nvsramInfo.BusDeviceFunction = Int2HexStr((UINT32) iter->u16Bus) + ":"
                    + Int2HexStr((UINT32) iter->u16Device) + "."
                    + Int2HexStr((UINT32) iter->u8Function);
            nvsramInfo.logicalID = nvsramNum++;
            nvsramInfo.elementSize = NVSRAM_ELEMENT_SIZE;

            nvsramList.push_back(nvsramInfo);

            return NO_ERROR_MSG;
        }
    }
    return NO_ERROR_MSG;
}

/// @fn         NVSRAMDDAPI::getDeviceList
/// @brief      Get NVSRAM information list on the host
/// @param      IOC ID
/// @param      image buffer
/// @param      image buffer size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::getDeviceList(vector<NVSRAM_RESOURCE_INFO>* returnedList)
{
    if(nvsramList.empty())
    {
        return returnErrorMsg(ERRORCODE(ERROR_NVSRAM_DDAPI_GET_NVSRAM_LIST), NULL, NULL);
    }
    *returnedList = nvsramList;

    return NO_ERROR_MSG;
}

/// @fn         NVSRAMDDAPI::getElementSize(UINT8 logicalID, PHYSICAL_ADDRESS_T* returnBaseAddress)
/// @brief      Get NVSRAM base address
/// @param      logicalID for target NVSRAM device
/// @param      pointer to the return base address
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::getBaseAddress(UINT8 logicalID, PHYSICAL_ADDRESS_T* returnBaseAddress)
{
    for(vector<NvsramInfo>::const_iterator iter = nvsramList.begin(); iter != nvsramList.end();
            iter++)
    {
        if(logicalID == iter->logicalID)
            *returnBaseAddress = iter->BaseAddress;
    }
    return NO_ERROR_MSG;
}

/// @fn         NVSRAMDDAPI::getResourceSize(UINT8 logicalID, UINT32* returnResourceSize)
/// @brief      Get NVSRAM resource size
/// @param      logicalID for target NVSRAM device
/// @param      pointer to the return NVSRAM resource size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::getResourceSize(UINT8 logicalID, UINT32* returnResourceSize)
{
    for(vector<NvsramInfo>::const_iterator iter = nvsramList.begin(); iter != nvsramList.end();
            iter++)
    {
        if(logicalID == iter->logicalID)
            *returnResourceSize = iter->BARSize;
    }
    return NO_ERROR_MSG;
}

/// @fn         NVSRAMDDAPI::getElementSize(UINT8 logicalID, UINT8* returnElementSize)
/// @brief      Get NVSRAM element size
/// @param      logicalID for target NVSRAM device
/// @param      pointer to the return element size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::getElementSize(UINT8 logicalID, UINT8* returnElementSize)
{
    for(vector<NvsramInfo>::const_iterator iter = nvsramList.begin(); iter != nvsramList.end();
            iter++)
    {
        if(logicalID == iter->logicalID)
            *returnElementSize = iter->elementSize;
    }
    return NO_ERROR_MSG;
}

/// @fn         NVSRAMDDAPI::recallNVSRAM(void)
/// @brief      Initiate the RECALL cycle to perform data transferred from non-volatile
//              memory to the SRAM
/// @param      None
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::recallNVSRAM(void)
{
    struct timespec req;
    // RECALL
    //1. Read address 0x4E38 Valid READ
    //2. Read address 0xB1C7 Valid READ
    //3. Read address 0x83E0 Valid READ
    //4. Read address 0x7C1F Valid READ
    //5. Read address 0x703F Valid READ
    //6. Read address 0x4C63 Initiate RECALL cycle
    memDDAPI->read8(nvsramBaseAddress + 0x4E38);
    memDDAPI->read8(nvsramBaseAddress + 0xB1C7);
    memDDAPI->read8(nvsramBaseAddress + 0x83E0);
    memDDAPI->read8(nvsramBaseAddress + 0x7C1F);
    memDDAPI->read8(nvsramBaseAddress + 0x703F);
    memDDAPI->read8(nvsramBaseAddress + 0x4C63);

    // wait for recall operation time
    req.tv_sec = 0;
    req.tv_nsec = 500000;
    nanosleep(&req, NULL);

    return NO_ERROR_MSG;
}


/// @fn         NVSRAMDDAPI::storeNVSRAM(void)
/// @brief      Initiate the STORE cycle to perform data transferred from SRAM
///             to the non-volatile memory
/// @param      None
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVSRAMDDAPI::storeNVSRAM(void)
{
    struct timespec req;
    // STORE
    //1. Read address 0x4E38 Valid READ
    //2. Read address 0xB1C7 Valid READ
    //3. Read address 0x83E0 Valid READ
    //4. Read address 0x7C1F Valid READ
    //5. Read address 0x703F Valid READ
    //6. Read address 0x8FC0 Initiate STORE cycle
    memDDAPI->read8(nvsramBaseAddress + 0x4E38);
    memDDAPI->read8(nvsramBaseAddress + 0xB1C7);
    memDDAPI->read8(nvsramBaseAddress + 0x83E0);
    memDDAPI->read8(nvsramBaseAddress + 0x7C1F);
    memDDAPI->read8(nvsramBaseAddress + 0x703F);
    memDDAPI->read8(nvsramBaseAddress + 0x8FC0);

    // wait for store operation time
    req.tv_sec = 0;
    req.tv_nsec = 10000000;
    nanosleep(&req, NULL);

    return NO_ERROR_MSG;
}
