/// @file       PSUDDAPI.hpp
/// @brief      PSUDDAPI class
/// @details    Provide low layer diagnostic and utility APIs for PSU module
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include "PSUDDAPI.hpp"
#include "I2CDAPI.hpp"

/// @fn         PSUDDAPI::PSUDDAPI(STRING in_szClassName) : BaseClassDDAPI(in_szClassName)
/// @brief      PSUDDAPI constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
PSUDDAPI::PSUDDAPI() : BaseClassDDAPI("PSUDDAPI")
{
    string      strFile, strTemp;
    FILE        *pFile = NULL;
    CHAR        szLine[128];
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    BaseClassPSUDDAPI   *pPSUDDAPI = NULL;
    BOOL        bIsPresent = TRUE;

    m_psuList.clear();

    // FIXME: This project does not support PSU
    addErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_CONSTRUCTOR | ERROR_FUNCTION_NOT_SUPPORTED),
                "This project does not support PSU");
    return;

    strFile = TEMP_CONFIG_DIR;
    strFile += TEMP_FILE_PSU_INFO;

    if ((pFile = fopen(strFile.c_str(), "r")) == NULL)
    {
        if ((pErrorMsg = initializePSUInfoFile()) != NO_ERROR_MSG)
        {
            addErrorMsg(ERROR_PSU_DDAPI_CONSTRUCTOR, NULL, pErrorMsg);
            return;
        }

        if ((pFile = fopen(strFile.c_str(), "r")) == NULL)
        {
            addErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_CONSTRUCTOR | ERROR_OPEN_FILE),
                        strFile.c_str());
            return;
        }
    }

    while (fgets(szLine, sizeof(szLine), pFile) != NULL)
    {
        // FIXME: This project does not support PSU
        if ((pPSUDDAPI /*= new XXX(szLine)*/) == NULL)
        {
            addErrorMsg(ERROR_PSU_DDAPI_CONSTRUCTOR, "fail to create XXX PSU");
            break;
        }

        if ((pErrorMsg = pPSUDDAPI->getErrorMsg()) != NO_ERROR_MSG)
        {
            addErrorMsg(ERROR_PSU_DDAPI_CONSTRUCTOR, NULL, pErrorMsg);
            break;
        }

        if ((pErrorMsg = pPSUDDAPI->isPresent(&bIsPresent)) != NO_ERROR_MSG)
        {
            delete pPSUDDAPI;
            addErrorMsg(ERROR_PSU_DDAPI_CONSTRUCTOR, NULL, pErrorMsg);
            break;
        }

        // Ignore non-existent device
        if (bIsPresent)
        {
            m_psuList.push_back(pPSUDDAPI);
        }
        else
        {
            delete pPSUDDAPI;
            pPSUDDAPI = NULL;
        }
    }

    fclose(pFile);
}

/// @fn         PSUDDAPI::~PSUDDAPI(void)
/// @brief      PSUDDAPI destructor
/// @exception  Nothing
/// @return     Nothing
PSUDDAPI::~PSUDDAPI(void)
{
    while (!m_psuList.empty())
    {
        delete m_psuList.front();
        m_psuList.pop_front();
    }
}

/// @fn         PSUDDAPI::getDeviceList(list<string> *out_pList)
///
/// @brief      Get device name list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getDeviceList(list<string> *out_pList)
{
    list<BaseClassPSUDDAPI *>::iterator itPSU = m_psuList.begin();

    if (out_pList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_DEVICE_LIST | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    while (itPSU != m_psuList.end())
    {
        out_pList->push_back((*itPSU)->getName());
        itPSU++;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_DEVICE_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         PSUDDAPI::getFanList(list<string> *out_pList)
///
/// @brief      Get FAN list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getFanList(list<string> *out_pList)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU = m_psuList.begin();
    list<string>::iterator itStr;
    INT32       s32OldListSize = 0;

    if (out_pList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_LIST | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    while (itPSU != m_psuList.end())
    {
        if ((pErrorMsg = (*itPSU)->getFanList(out_pList)) != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PSU_DDAPI_GET_FAN_LIST, NULL, pErrorMsg);
        }

        // Device name will start by PSU name
        itStr = out_pList->begin();

        // Find the first un-handled item
        for (;s32OldListSize > 0; s32OldListSize--, itStr++) ;

        while (itStr != out_pList->end())
        {
            itStr->insert(0, "_");
            itStr->insert(0, (*itPSU)->getName());
            itStr++;
        }

        s32OldListSize = out_pList->size();

        itPSU++;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         PSUDDAPI::getFanSpeed(string  in_strDeviceName,
///                                   INT32   *out_s32pValue)
///
/// @brief      Get current speed for the given FAN
/// @param      in_strDeviceName is a string for FAN name
/// @param      out_s32pValue is a pointer to a INT32 for the current speed
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getFanSpeed(string  in_strDeviceName,
                      INT32   *out_s32pValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_SPEED | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getFanSpeed(in_strDeviceName, out_s32pValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_FAN_SPEED, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_SPEED | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getFWVersion(string   in_strDevice,
///                                    string   *out_strpValue)
///
/// @brief      Get FW version of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the FW version (Format: XX.YY)
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getFWVersion(string    in_strDevice,
                       string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getFWVersion(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_FW_VERSION, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FW_VERSION | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::getLEDList(list<string> *out_pList)
///
/// @brief      Get LED name list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getLEDList(list<string> *out_pList)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU = m_psuList.begin();
    list<string>::iterator itStr;
    INT32       s32OldListSize = 0;

    if (out_pList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_LED_LIST | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    while (itPSU != m_psuList.end())
    {
        if ((pErrorMsg = (*itPSU)->getLEDList(out_pList)) != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PSU_DDAPI_GET_LED_LIST, NULL, pErrorMsg);
        }

        // Device name will start by PSU name
        itStr = out_pList->begin();

        // Find the first un-handled item
        for (;s32OldListSize > 0; s32OldListSize--, itStr++) ;

        while (itStr != out_pList->end())
        {
            itStr->insert(0, "_");
            itStr->insert(0, (*itPSU)->getName());
            itStr++;
        }

        s32OldListSize = out_pList->size();

        itPSU++;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_LED_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         PSUDDAPI::getLEDStatus(string in_strDeviceName,
///                                    string *out_strpStatus)
///
/// @brief      Get status of the given LED
/// @param      in_strDeviceName is a string for the LED name
/// @param      out_strpStatus is a pointer to a string for the status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getLEDStatus(string in_strDeviceName,
                       string *out_strpStatus)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_LED_STATUS | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getLEDStatus(in_strDeviceName, out_strpStatus)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_LED_STATUS, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_LED_STATUS | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getMaxPower(string   in_strDevice,
///                                   string   *out_strpValue)
///
/// @brief      Get maximum output power of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the maximum output power
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getMaxPower(string   in_strDevice,
                      string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getMaxPower(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_MAX_POWER, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_MAX_POWER | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::getPartNumber(string   in_strDevice,
///                                     string   *out_strpValue)
///
/// @brief      Get part number of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the part number
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getPartNumber(string   in_strDevice,
                        string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getPartNumber(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_PART_NUMBER, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_PART_NUMBER | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::getSerialNumber(string   in_strDevice,
///                                       string   *out_strpValue)
///
/// @brief      Get serial number of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the serial number
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getSerialNumber(string   in_strDevice,
                          string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getSerialNumber(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_SERIAL_NUMBER, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_SERIAL_NUMBER | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::getSupportFanSpeed(string  in_strDeviceName,
///                                          list<string>   *out_pList)
///
/// @brief      Get supported speeds for the given FAN
/// @param      in_strDeviceName is a string for FAN name
/// @param      out_pList is a pointer to a list of strings for supported speeds
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getSupportFanSpeed(string         in_strDeviceName,
                             list<string>   *out_pList)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getSupportFanSpeed(in_strDeviceName, out_pList)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getTemperature(string in_strDeviceName,
///                                      string *out_strpValue)
///
/// @brief      Get current temperature of the given temperature sensor
/// @param      in_strDeviceName is a string for the LED
/// @param      out_strpValue is a pointer to a string for the temperature
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getTemperature(string in_strDeviceName,
                         string *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;
    FLOAT       fValue;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getTemperature(in_strDeviceName, &fValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_TEMPERATURE, NULL, pErrorMsg);
            }

            *out_strpValue = std::to_string(fValue);

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getTemperatureList(list<string> *out_pList)
///
/// @brief      Get temperature name list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getTemperatureList(list<string> *out_pList)
{
    ErrorMsg *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU = m_psuList.begin();
    list<string>::iterator itStr;
    INT32       s32OldListSize = 0;

    if (out_pList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    while (itPSU != m_psuList.end())
    {
        if ((pErrorMsg = (*itPSU)->getTemperatureList(out_pList)) != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST, NULL, pErrorMsg);
        }

        // Device name will start by PSU name
        itStr = out_pList->begin();

        // Find the first un-handled item
        for (;s32OldListSize > 0; s32OldListSize--, itStr++) ;

        while (itStr != out_pList->end())
        {
            itStr->insert(0, "_");
            itStr->insert(0, (*itPSU)->getName());
            itStr++;
        }

        s32OldListSize = out_pList->size();

        itPSU++;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         PSUDDAPI::getTemperatureThreshold(string in_strDeviceName,
///                                               string *out_strpValue)
///
/// @brief      Get temperature threshold
/// @param      in_strDeviceName is a string for the threshold name
/// @param      out_strpValue is a pointer to a string for the threshold
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getTemperatureThreshold(string in_strDeviceName,
                                  string *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;
    FLOAT       fValue;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getTemperatureThreshold(in_strDeviceName, &fValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD, NULL, pErrorMsg);
            }

            *out_strpValue = std::to_string(fValue);

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getTemperatureThresholdList(list<string> *out_pList)
///
/// @brief      Get temperature threshold list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getTemperatureThresholdList(list<string> *out_pList)
{
    ErrorMsg *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU = m_psuList.begin();
    list<string>::iterator itStr;
    INT32       s32OldListSize = 0;

    if (out_pList == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST | ERROR_BAD_PARAMETER),
                              "NULL pointer");
    }

    while (itPSU != m_psuList.end())
    {
        if ((pErrorMsg = (*itPSU)->getTemperatureThresholdList(out_pList)) != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST, NULL, pErrorMsg);
        }

        // Device name will start by PSU name
        itStr = out_pList->begin();

        // Find the first un-handled item
        for (;s32OldListSize > 0; s32OldListSize--, itStr++) ;

        while (itStr != out_pList->end())
        {
            itStr->insert(0, "_");
            itStr->insert(0, (*itPSU)->getName());
            itStr++;
        }

        s32OldListSize = out_pList->size();

        itPSU++;
    }

    if (out_pList->empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST | ERROR_NO_DEVICE_FOUND));
    }

    return NO_ERROR_MSG;
}

/// @fn         PSUDDAPI::getType(string   in_strDevice,
///                               string   *out_strpValue)
///
/// @brief      Get protocol type of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the protocol type
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getType(string   in_strDevice,
                  string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getType(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_TYPE, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TYPE | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::getVendor(string   in_strDevice,
///                                 string   *out_strpValue)
///
/// @brief      Get vendor name of the given device
/// @param      in_strDevice is a string for the PSU device
/// @param      out_strpValue is a pointer to a string for the vendor name
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::getVendor(string   in_strDevice,
                    string   *out_strpValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (in_strDevice == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->getVendor(out_strpValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_GET_VENDOR_NAME, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_VENDOR_NAME | ERROR_NO_DEVICE_FOUND),
                          in_strDevice.c_str());
}

/// @fn         PSUDDAPI::program(string    in_strDevice,
///                               UINT8     in_u8Region,
///                               CHAR      *in_chpFwBuffer,
///                               INT32     in_s32FwSize)
/// @brief      Program FW to the given PSU device
/// @param      in_strDevice is a string for the PSU
/// @param      in_u8Region is a UINT8 for the FW region which will be updated
/// @param      in_chpFwBuffer is a pointer to a memory buffer for the FW
/// @param      in_s32FwSize is INT32 for the FW size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::program(string    in_strDevice,
                  UINT8     in_u8Region,
                  CHAR      *in_chpFwBuffer,
                  INT32     in_s32FwSize)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (in_chpFwBuffer == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_PROGRAM | ERROR_BAD_PARAMETER_THREE),
                              "NULL pointer");
    }

    if (in_s32FwSize <= 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_PROGRAM | ERROR_BAD_PARAMETER_FOUR),
                              "invalid FW size");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
        {
            if (in_strDevice == (*itPSU)->getName())
            {
                m_resourceLock.lock(ERROR_PSU_DDAPI_PROGRAM);

                pErrorMsg = (*itPSU)->program(in_u8Region, in_chpFwBuffer, in_s32FwSize);

                m_resourceLock.release(ERROR_PSU_DDAPI_PROGRAM);

                if (pErrorMsg != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERROR_PSU_DDAPI_PROGRAM, NULL, pErrorMsg);
                }

                return NO_ERROR_MSG;
            }
        }

        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_PROGRAM | ERROR_NO_DEVICE_FOUND),
                              in_strDevice.c_str());
}

/// @fn         PSUDDAPI::setFanSpeed(string   in_strDeviceName,
///                                    INT32    in_s32Value)
/// @brief      Set FAN speed to the given FAN
/// @param      in_strDeviceName is a string for the FAN
/// @param      in_s32Value is a INT32 for the FAN speed
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::setFanSpeed(string   in_strDeviceName,
                      INT32    in_s32Value)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_FAN_SPEED | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->setFanSpeed(in_strDeviceName, in_s32Value)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_SET_FAN_SPEED, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_FAN_SPEED | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::setLEDStatus(string in_strDeviceName,
///                                    string in_strStatus)
///
/// @brief      Set status to the given LED
/// @param      in_strDeviceName is a string for the LED
/// @param      in_strStatus is a string for the status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::setLEDStatus(string in_strDeviceName,
                       string in_strStatus)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_LED_STATUS | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            if ((pErrorMsg = (*itPSU)->setLEDStatus(in_strDeviceName, in_strStatus)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_SET_LED_STATUS, NULL, pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_LED_STATUS | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::setTemperatureThreshold(string in_strDeviceName,
///                                               string  in_strValue)
///
/// @brief      Set temperature threshold for temperature sensors
/// @param      in_strDeviceName is a string for the temperature threshold name
/// @param      in_strValue is a string for the threshold value
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::setTemperatureThreshold(string    in_strDeviceName,
                                  string    in_strValue)
{
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    string      strPSUName = getOutPSUName(&in_strDeviceName);
    list<BaseClassPSUDDAPI *>::iterator itPSU;
    FLOAT       fValue;

    if (strPSUName.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD | ERROR_BAD_PARAMETER_ONE),
                              "unexpected format");
    }

    for (itPSU = m_psuList.begin(); itPSU != m_psuList.end(); itPSU++)
    {
        if (strPSUName == (*itPSU)->getName())
        {
            fValue = std::stof(in_strValue, NULL);
            if ((pErrorMsg = (*itPSU)->setTemperatureThreshold(in_strDeviceName, fValue)) != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD,
                                      NULL,
                                      pErrorMsg);
            }

            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD | ERROR_NO_DEVICE_FOUND),
                          strPSUName.c_str());
}

/// @fn         PSUDDAPI::getOutPSUName(string *inout_strpString)
///
/// @brief      Get PSU name out of the given string
/// @param      inout_strpString is a pointer to the given string
/// @exception  Nothing
/// @return     A string for the PSU name
string
PSUDDAPI::getOutPSUName(string *inout_strpString)
{
    string strTemp;
    std::size_t pos = inout_strpString->find("_");

    if (pos != std::string::npos)
    {
        // PSU name is at the beginning
        strTemp = inout_strpString->substr(0, pos);
        inout_strpString->erase(0, pos + 1);
    }

    return strTemp;
}

/// @fn         PSUDDAPI::initializePSUInfoFile(void)
///
/// @brief      Initialize the file for PSU information
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
PSUDDAPI::initializePSUInfoFile(void)
{
    string      strTemp;
    FILE        *pFile = NULL;
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    I2CDAPI     i2cDAPI("I2CDAPI");
    list<string> infoList;
    list<string>::iterator itStr;

    // Create directory
    strTemp = "mkdir -p ";
    strTemp += TEMP_CONFIG_DIR;
    strTemp += "psu";
    system(strTemp.c_str());

    if ((pErrorMsg = i2cDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE, NULL, pErrorMsg);
    }

    // Derive PSU information from I2C config file
    if ((pErrorMsg = i2cDAPI.DisplayI2cDevicesList(&infoList)) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE, NULL, pErrorMsg);
    }

    itStr = infoList.begin();
    while (itStr != infoList.end())
    {
        if (itStr->find("psu") == std::string::npos)
        {
            itStr = infoList.erase(itStr);
        }
        else
        {
            itStr++;
        }
    }

    if (infoList.size() == 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE | ERROR_NO_DEVICE_FOUND));
    }

    strTemp = TEMP_CONFIG_DIR;
    strTemp += TEMP_FILE_PSU_INFO;
    if ((pFile = fopen(strTemp.c_str(), "w")) == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE | ERROR_OPEN_FILE), strTemp.c_str());
    }

    itStr = infoList.begin();
    while (itStr != infoList.end())
    {
        if (itStr != infoList.begin())
        {
            itStr->insert(0, "\n");
        }

        fwrite(itStr->c_str(), sizeof(char), itStr->length(), pFile);
        itStr++;
    }

    fclose(pFile);

    return NO_ERROR_MSG;
}
