/// @file       SCSIFactory.cpp
/// @brief      SCSIFactory class
/// @details    Provide low layer diagnostic and utility APIs for Storage module
/// @version    1.0.0
/// @date       2015.12.01
/// @author     Ben Yang
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#include "SCSIFactory.hpp"
static char spare_buff[128];

/// @fn         SCSIFactory::SCSIFactory(STRING in_strClassName)
/// @brief      SCSIFactory constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
SCSIFactory::SCSIFactory(STRING in_szClassName) : BaseClassDDAPI(in_szClassName)
{
    // initialize here
    if((senseAnylyzer = new SCSISenseAnalyzer("Sense Analyzer")) == NULL)
    {
        addErrorMsg((ERRORCODE)(ERROR_SCSI_FACTORY_ERRORCODE | ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR), "Create Sense Analyzer fail");
    }

    memset(m_senseBuff, 0, sizeof(m_senseBuff));
}

/// @fn         SCSISense::~SCSISense(void)
/// @brief      SCSISense destructor
/// @exception  Nothing
/// @return     Nothing
SCSIFactory::~SCSIFactory(void)
{
    if(senseAnylyzer != NULL)
        delete senseAnylyzer;
}

ErrorMsg *SCSIFactory::executeSCSIRead(INT32 fileDescription, UINT64 in_u64StartLogicalSector,
                                UINT32 numberOfBlocksToRead, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    SCSI_BLOCK_CDB scsiCDB;
    
    // clear the SCSI CDB
    memset(&iop, 0, sizeof(SCSI_COMMAND_CDB));
    memset(&scsiCDB, 0, sizeof(SCSI_BLOCK_CDB));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    // select the SCSI command based upon the amount of continuous data to read
    if (in_u64StartLogicalSector & ~0xffffffffULL)
    {
        scsiCDB.cdbType = SCSI_CDB_16_BYTE;
        scsiCDB.cdb.cdb16.opCode = SCSI_OPCODE_READ_16;
        scsiCDB.cdb.cdb16.allocationLengthByte10 = (UINT8) ((numberOfBlocksToRead >> 24) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte11 = (UINT8) ((numberOfBlocksToRead >> 16) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte12 = (UINT8) ((numberOfBlocksToRead >> 8) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte13 = (UINT8) (numberOfBlocksToRead & 0xff);

        scsiCDB.cdb.cdb16.logicalBlockByte2 = (UINT8) ((in_u64StartLogicalSector >> 56) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte3 = (UINT8) ((in_u64StartLogicalSector >> 48) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte4 = (UINT8) ((in_u64StartLogicalSector >> 40) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte5 = (UINT8) ((in_u64StartLogicalSector >> 32) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte6 = (UINT8) ((in_u64StartLogicalSector >> 24) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte7 = (UINT8) ((in_u64StartLogicalSector >> 16) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte8 = (UINT8) ((in_u64StartLogicalSector >> 8) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte9 = (UINT8) (in_u64StartLogicalSector & 0xff);
    }
    else
    {
        scsiCDB.cdbType = SCSI_CDB_10_BYTE;
        scsiCDB.cdb.cdb10.opCode = SCSI_OPCODE_READ_10;
        scsiCDB.cdb.cdb10.allocationLengthByte7 = (UINT8) ((numberOfBlocksToRead >> 8) & 0xff);
        scsiCDB.cdb.cdb10.allocationLengthByte8 = (UINT8) (numberOfBlocksToRead & 0xff);

        scsiCDB.cdb.cdb10.logicalBlockByte2 = (UINT8) ((in_u64StartLogicalSector >> 24) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte3 = (UINT8) ((in_u64StartLogicalSector >> 16) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte4 = (UINT8) ((in_u64StartLogicalSector >> 8) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte5 = (UINT8) (in_u64StartLogicalSector & 0xff);
    }

    iop.cmnd = (UINT8*)&(scsiCDB.cdb);
    iop.cmnd_len = sizeof(scsiCDB.cdb);
    iop.dxferp = buffer;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);

    if ((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_SCSI | m_errorCode), NULL,
                NULL);
    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::executeSCSIWrite(INT32 fileDescription, UINT64 in_u64StartLogicalSector,
                                  UINT32 numberOfBlocksToWrite, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    SCSI_BLOCK_CDB scsiCDB;
    // clear the SCSI CDB
    memset(&iop, 0, sizeof(SCSI_COMMAND_CDB));
    memset(&scsiCDB, 0, sizeof(SCSI_BLOCK_CDB));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    // select the SCSI command based upon the amount of continuous data to read
    if (in_u64StartLogicalSector & ~0xffffffffULL)
    {
        scsiCDB.cdbType = SCSI_CDB_16_BYTE;
        scsiCDB.cdb.cdb16.opCode = SCSI_OPCODE_WRITE_16;
        scsiCDB.cdb.cdb16.allocationLengthByte10 = (UINT8) ((numberOfBlocksToWrite >> 24) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte11 = (UINT8) ((numberOfBlocksToWrite >> 16) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte12 = (UINT8) ((numberOfBlocksToWrite >> 8) & 0xff);
        scsiCDB.cdb.cdb16.allocationLengthByte13 = (UINT8) (numberOfBlocksToWrite & 0xff);

        scsiCDB.cdb.cdb16.logicalBlockByte2 = (UINT8) ((in_u64StartLogicalSector >> 56) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte3 = (UINT8) ((in_u64StartLogicalSector >> 48) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte4 = (UINT8) ((in_u64StartLogicalSector >> 40) & 0xff);
        scsiCDB.cdb.cdb16.logicalBlockByte5 = (UINT8) ((in_u64StartLogicalSector >> 32) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte6 = (UINT8) ((in_u64StartLogicalSector >> 24) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte7 = (UINT8) ((in_u64StartLogicalSector >> 16) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte8 = (UINT8) ((in_u64StartLogicalSector >> 8) & 0xff);
        scsiCDB.cdb.cdb16.additionalCBPByte9 = (UINT8) (in_u64StartLogicalSector & 0xff);
    }
    else
    {
        scsiCDB.cdbType = SCSI_CDB_10_BYTE;
        scsiCDB.cdb.cdb10.opCode = SCSI_OPCODE_WRITE_10;
        scsiCDB.cdb.cdb10.allocationLengthByte7 = (UINT8) ((numberOfBlocksToWrite >> 8) & 0xff);
        scsiCDB.cdb.cdb10.allocationLengthByte8 = (UINT8) (numberOfBlocksToWrite & 0xff);

        scsiCDB.cdb.cdb10.logicalBlockByte2 = (UINT8) ((in_u64StartLogicalSector >> 24) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte3 = (UINT8) ((in_u64StartLogicalSector >> 16) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte4 = (UINT8) ((in_u64StartLogicalSector >> 8) & 0xff);
        scsiCDB.cdb.cdb10.logicalBlockByte5 = (UINT8) (in_u64StartLogicalSector & 0xff);
    }

    iop.cmnd = (UINT8*)&(scsiCDB.cdb);
    iop.cmnd_len = sizeof(scsiCDB.cdb);
    iop.dxferp = buffer;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxfer_dir = DXFER_TO_DEVICE;
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);

    if ((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_WRITE_SCSI | m_errorCode), NULL,
                NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiTestUnitReady(INT32 fileDescription)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];
    bool tryAgain = FALSE;

try_again:
    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_NONE;
    iop.dxfer_len = 0;
    iop.dxferp = NULL;
    cdb[0] = SCSI_OPCODE_TEST_UNIT_READY;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_TEST_UNIT_READY | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if((m_errorCode == ERROR_SCSI_FACTORY_ERR_TRY_AGAIN) && !tryAgain)
    {
        tryAgain = TRUE;
        goto try_again;
    }

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_TEST_UNIT_READY | m_errorCode), NULL, NULL);

	return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiStdInquiry(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];

    if((in_u32BufferSize < 0) || (in_u32BufferSize > 1023))
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_STD_INQUIRY | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_INQUIRY;
    cdb[3] = (in_u32BufferSize >> 8) & 0xff;
    cdb[4] = (in_u32BufferSize & 0xff);
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_STD_INQUIRY | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_STD_INQUIRY | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiInquiryVpd(INT32 fileDescription, UINT8 vpdPageNumber, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];
    int res;

    if ((in_u32BufferSize < 0) || (in_u32BufferSize > 1023))
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_INQUIRY_VPD | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

try_again:
    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    if (in_u32BufferSize > 1)
        buffer[1] = 0x0;
    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_INQUIRY;
    cdb[1] = 0x1;       /* set EVPD bit (enable Vital Product Data) */
    cdb[2] = vpdPageNumber;
    cdb[3] = (in_u32BufferSize >> 8) & 0xff;
    cdb[4] = (in_u32BufferSize & 0xff);
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_INQUIRY_VPD | m_errorCode), NULL, NULL);

    if ((SCSI_STATUS_CHECK_CONDITION == iop.scsi_status) && (cdb[3] > 0)) // doesn't check sense key and asc
    {
        in_u32BufferSize &= 0xff; /* make sure cdb[3] is 0 next time around */
        goto try_again;
    }

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_INQUIRY_VPD | m_errorCode), NULL, NULL);

    /* Guard against devices that ignore EVPD bit and do standard INQUIRY */
    if(in_u32BufferSize > 1)
    {
        if(vpdPageNumber == buffer[1])
        {
            if((0x80 == vpdPageNumber) && (in_u32BufferSize > 2) && (0x0 != buffer[2]))
                return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_INQUIRY_VPD | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
        }
        else
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_INQUIRY_VPD | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiLogSense(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
                                    UINT8 *buffer, UINT32 in_u32BufferSize, UINT8 knownRespLength)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[10];
    int pageLen;
    int status, res;

    if (knownRespLength > in_u32BufferSize)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    if (knownRespLength > 0)
        pageLen = knownRespLength;
    else
    {
        /* Starting twin fetch strategy: first fetch to find respone length */
        pageLen = 4;
        if (pageLen > in_u32BufferSize)
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);
        else
            memset(buffer, 0, pageLen);

        memset(&iop, 0, sizeof(iop));
        memset(cdb, 0, sizeof(cdb));
        memset(m_senseBuff, 0, sizeof(m_senseBuff));

        iop.dxfer_dir = DXFER_FROM_DEVICE;
        iop.dxfer_len = pageLen;
        iop.dxferp = buffer;
        cdb[0] = SCSI_OPCODE_LOG_SENSE;
        cdb[2] = 0x40 | (pageNumber & 0x3f);  /* Page control (PC)==1 */
        cdb[3] = subPageNumber;
        cdb[7] = (pageLen >> 8) & 0xff;
        cdb[8] = pageLen & 0xff;
        iop.cmnd = cdb;
        iop.cmnd_len = sizeof(cdb);
        iop.sensep = m_senseBuff;
        iop.max_sense_len = sizeof(m_senseBuff);
        iop.timeout = SCSI_TIMEOUT_DEFAULT;

        if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | m_errorCode), NULL, NULL);

        m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

        if(m_errorCode != NO_ERROR)
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | m_errorCode), NULL, NULL);

        /* sanity check on response */
        if ((SUPPORTED_LPAGES != pageNumber) && ((buffer[0] & 0x3f) != pageNumber))
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
        if (0 == ((buffer[2] << 8) + buffer[3]))
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
        pageLen = (buffer[2] << 8) + buffer[3] + 4;
        if (4 == pageLen)  /* why define a lpage with no payload? */
            pageLen = 252; /* some IBM tape drives don't like double fetch */
        /* some SCSI HBA don't like "odd" length transfers */
        if (pageLen % 2)
            pageLen += 1;
        if (pageLen > in_u32BufferSize)
            pageLen = in_u32BufferSize;
    }
    memset(buffer, 0, 4);
    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = pageLen;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_LOG_SENSE;
    cdb[2] = 0x40 | (pageNumber & 0x3f);  /* Page control (PC)==1 */
    cdb[7] = (pageLen >> 8) & 0xff;
    cdb[8] = pageLen & 0xff;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | m_errorCode), NULL, NULL);

    /* sanity check on response */
    if ((SUPPORTED_LPAGES != pageNumber) && ((buffer[0] & 0x3f) != pageNumber))
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
    if (0 == ((buffer[2] << 8) + buffer[3]))
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiLogSelect(INT32 fileDescription, UINT8 pcr, UINT8 sp, UINT8 pageControl, UINT8 pageNumber,
                                     UINT8 subpageNumber, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[10];

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_TO_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_LOG_SELECT;
    cdb[1] = (pcr ? 2 : 0) | (sp ? 1 : 0);
    cdb[2] = ((pageControl << 6) & 0xc0) | (pageNumber & 0x3f);
    cdb[3] = (subpageNumber & 0xff);
    cdb[7] = ((in_u32BufferSize >> 8) & 0xff);
    cdb[8] = (in_u32BufferSize & 0xff);
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SELECT | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_LOG_SELECT | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiModeSense(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
                                     UINT8 pageControl, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];

    if ((in_u32BufferSize < 0) || (in_u32BufferSize > 255))
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_MODE_SENSE;
    cdb[2] = (pageControl << 6) | (pageNumber & 0x3f);
    cdb[3] = subPageNumber;
    cdb[4] = in_u32BufferSize;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode == ERROR_SCSI_FACTORY_ERR_TRY_AGAIN)
    {
        if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
                return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE | m_errorCode), NULL, NULL);

        m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));
    }

    if((m_errorCode == NO_ERROR) && (ALL_MODE_PAGES != pageNumber))
    {
        int offset;

        offset = getSCSIModePageOffset(buffer, in_u32BufferSize, 0);
        if(offset < 0)
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
        else if(pageNumber != (buffer[offset] & 0x3f))
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiModeSelect(INT32 fileDescription, UINT8 sp, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];
    int pg_offset, pg_len, hdr_plus_1_pg;

    pg_offset = 4 + buffer[3];
    if (pg_offset + 2 >= in_u32BufferSize)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    pg_len = buffer[pg_offset + 1] + 2;
    hdr_plus_1_pg = pg_offset + pg_len;
    if (hdr_plus_1_pg > in_u32BufferSize)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    buffer[0] = 0;    /* Length of returned mode sense data reserved for SELECT */
    buffer[pg_offset] &= 0x7f;    /* Mask out PS bit from byte 0 of page data */
    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_TO_DEVICE;
    iop.dxfer_len = hdr_plus_1_pg;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_MODE_SELECT;
    cdb[1] = 0x10 | (sp & 1);      /* set PF (page format) bit always */
    cdb[4] = hdr_plus_1_pg; /* make sure only one page sent */
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiModeSense10(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
							           UINT8 pageControl, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[10];
    int status;

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_MODE_SENSE_10;
    cdb[2] = (pageControl << 6) | (pageNumber & 0x3f);
    cdb[3] = subPageNumber;
    cdb[7] = (in_u32BufferSize >> 8) & 0xff;
    cdb[8] = in_u32BufferSize & 0xff;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE10 | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode == ERROR_SCSI_FACTORY_ERR_TRY_AGAIN)
    {
        if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
                return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE10 | m_errorCode), NULL, NULL);

        m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));
    }

    if((m_errorCode == NO_ERROR) && (ALL_MODE_PAGES != pageNumber))
    {
        int offset;

        offset = getSCSIModePageOffset(buffer, in_u32BufferSize, 1);
        if(offset < 0)
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE10 | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
        else if(pageNumber != (buffer[offset] & 0x3f))
            return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SENSE10 | ERROR_SCSI_FACTORY_ERR_BAD_RESP), NULL, NULL);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiModeSelect10(INT32 fileDescription, UINT8 sp, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[10];
    int pg_offset, pg_len, hdr_plus_1_pg;

    pg_offset = 8 + (buffer[6] << 8) + buffer[7];
    if (pg_offset + 2 >= in_u32BufferSize)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT10 | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    pg_len = buffer[pg_offset + 1] + 2;
    hdr_plus_1_pg = pg_offset + pg_len;
    if (hdr_plus_1_pg > in_u32BufferSize)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT10 | ERROR_ILLEGAL_BUFFER_SIZE), NULL, NULL);

    buffer[0] = 0;
    buffer[1] = 0; /* Length of returned mode sense data reserved for SELECT */
    buffer[pg_offset] &= 0x7f;    /* Mask out PS bit from byte 0 of page data */
    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_TO_DEVICE;
    iop.dxfer_len = hdr_plus_1_pg;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_MODE_SELECT_10;
    cdb[1] = 0x10 | (sp & 1);      /* set PF (page format) bit always */
    cdb[8] = hdr_plus_1_pg; /* make sure only one page sent */
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT10 | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_MODE_SELECT10 | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiRequestSense(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = sizeof(buffer);
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_REQUEST_SENSE;
    cdb[4] = in_u32BufferSize;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_REQUEST_SENSE | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_REQUEST_SENSE | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiSendDiagnostic(INT32 fileDescription, UINT8 functioncode, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[6];

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = in_u32BufferSize ? DXFER_TO_DEVICE: DXFER_NONE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_SEND_DIAGNOSTIC;
    if (SCSI_DIAG_DEF_SELF_TEST == functioncode)
        cdb[1] = 0x4;  /* SelfTest bit */
    else if (SCSI_DIAG_NO_SELF_TEST != functioncode)
        cdb[1] = (functioncode & 0x7) << 5; /* SelfTest _code_ */
    else   /* SCSI_DIAG_NO_SELF_TEST == functioncode */
        cdb[1] = 0x10;  /* PF bit */
    cdb[3] = (in_u32BufferSize >> 8) & 0xff;
    cdb[4] = in_u32BufferSize & 0xff;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    /* worst case is an extended foreground self test on a big disk */
    iop.timeout = SCSI_TIMEOUT_SELF_TEST;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_SEND_DIAGNOSTIC | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_SEND_DIAGNOSTIC | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiReadCapacity10(INT32 fileDescription, UINT32 *lastLogicalBlock, UINT32 *logicalBlockSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[10];
    UINT8 resp[8];

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(resp, 0, sizeof(resp));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = sizeof(resp);
    iop.dxferp = resp;
    cdb[0] = SCSI_OPCODE_READ_CAPACITY_10;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_CAPACITY10 | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_CAPACITY10 | m_errorCode), NULL, NULL);

    if (lastLogicalBlock)
        *lastLogicalBlock = (resp[0] << 24) | (resp[1] << 16) | (resp[2] << 8) |
                     resp[3];
    if (logicalBlockSize)
        *logicalBlockSize = (resp[4] << 24) | (resp[5] << 16) | (resp[6] << 8) |
                    resp[7];

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiReadCapacity16(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize)
{
    SCSI_COMMAND_CDB iop;
    UINT8 cdb[16];

    memset(&iop, 0, sizeof(iop));
    memset(cdb, 0, sizeof(cdb));
    memset(m_senseBuff, 0, sizeof(m_senseBuff));

    iop.dxfer_dir = DXFER_FROM_DEVICE;
    iop.dxfer_len = in_u32BufferSize;
    iop.dxferp = buffer;
    cdb[0] = SCSI_OPCODE_READ_CAPACITY_16;
    cdb[1] = SAI_READ_CAPACITY_16;
    cdb[10] = (in_u32BufferSize >> 24) & 0xff;
    cdb[11] = (in_u32BufferSize >> 16) & 0xff;
    cdb[12] = (in_u32BufferSize >> 8) & 0xff;
    cdb[13] = in_u32BufferSize & 0xff;
    iop.cmnd = cdb;
    iop.cmnd_len = sizeof(cdb);
    iop.sensep = m_senseBuff;
    iop.max_sense_len = sizeof(m_senseBuff);
    iop.timeout = SCSI_TIMEOUT_DEFAULT;

    if((m_errorCode = sg_io_cmnd_io(fileDescription, &iop)) != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_CAPACITY16 | m_errorCode), NULL, NULL);

    m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff));

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_CAPACITY16 | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::scsiGetProtectionPhysicalInfo(INT32 fileDescription, UINT8 *rc16_12_31p)
{
    ErrorMsg* errorMsg = NULL;
    UINT8 rc16resp[32];

    errorMsg = scsiReadCapacity16(fileDescription, rc16resp, sizeof(rc16resp));
    if(errorMsg != (ErrorMsg *) NO_ERROR)
        return errorMsg;

    if(rc16_12_31p)
        memcpy(rc16_12_31p, rc16resp + 12, 20);

    return NO_ERROR_MSG;
}

UINT8 *SCSIFactory::getCurrentSenseBuffer(void)
{
     return m_senseBuff;
}

/* Returns a negative value on error, 0 if unknown and 1 if SSD,
 * otherwise the positive returned value is the speed in rpm. First checks
 * the Block Device Characteristics VPD page and if that fails it tries the
 * RIGID_DISK_DRIVE_GEOMETRY_PAGE mode page. */
ErrorMsg *SCSIFactory::getSCSIDriveRPM(INT32 fileDescription, UINT8 modeSenseLength, int *rpm,
                                       int *formFactor, int *zonedBlockCapable)
{
    ErrorMsg* errorMsg = NULL;
    int offset;
    UINT8 buff[64];
    int pc = MPAGE_CONTROL_DEFAULT;

    memset(buff, 0, sizeof(buff));
    if(((ErrorMsg *)NO_ERROR == scsiInquiryVpd(fileDescription, SCSI_VPD_BLOCK_DEVICE_CHARACTERISTICS,
                                 buff, sizeof(buff))) && (((buff[2] << 8) + buff[3]) > 2))
    {
        *rpm = (buff[4] << 8) + buff[5];
        if (formFactor)
            *formFactor = buff[7] & 0xf;
        if (zonedBlockCapable)
            *zonedBlockCapable = !!(0x10 & buff[8]);

        return NO_ERROR_MSG;
    }
    if(formFactor)
        *formFactor = 0;
    if(zonedBlockCapable)
        *zonedBlockCapable = 0;
    if(modeSenseLength <= 6)
    {
        errorMsg = scsiModeSense(fileDescription, RIGID_DISK_DRIVE_GEOMETRY_PAGE, 0, pc, buff, sizeof(buff));
        if(errorMsg != (ErrorMsg *) NO_ERROR)
        {
            if (ERROR_SCSI_FACTORY_ERR_BAD_OPCODE == m_errorCode)
                modeSenseLength = 10;
            else
                return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_RPM), NULL, errorMsg);
        }
        else if(0 == modeSenseLength)
            modeSenseLength = 6;
    }
    if(10 == modeSenseLength)
    {
        errorMsg = scsiModeSense10(fileDescription, RIGID_DISK_DRIVE_GEOMETRY_PAGE, 0, pc, buff, sizeof(buff));
        if(errorMsg != (ErrorMsg *) NO_ERROR)
            return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_RPM), NULL, errorMsg);
    }

    offset = getSCSIModePageOffset(buff, sizeof(buff), modeSenseLength);
    *rpm = (buff[offset + 20] << 8) | buff[offset + 21];

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::getSCSIDriveTemperature(INT32 fileDescription, UINT8 *currenttemp, UINT8 *triptemp)
{
    ErrorMsg* errorMsg = NULL;
    UINT8 tBuf[252];
    int err;

    memset(tBuf, 0, sizeof(tBuf));
    errorMsg = scsiLogSense(fileDescription, TEMPERATURE_LPAGE, 0, tBuf, sizeof(tBuf), 0);
    if(errorMsg != (ErrorMsg *) NO_ERROR)
    {
        *currenttemp = 0;
        *triptemp = 0;
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_DRIVE_TEMPERATURE), NULL, errorMsg);
    }

    *currenttemp = tBuf[9];
    *triptemp = tBuf[15];
    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::getSCSITransportProtocol(INT32 fileDescription, UINT8 modeSenseLength, int *protocolType)
{
    ErrorMsg* errorMsg = NULL;
    int offset;
    UINT8 buff[64];

    memset(buff, 0, sizeof(buff));
    if(modeSenseLength <= 6)
    {
        if((errorMsg = scsiModeSense(fileDescription, PROTOCOL_SPECIFIC_PORT_PAGE, 0,
                                 MPAGE_CONTROL_CURRENT, buff, sizeof(buff))) != (ErrorMsg *) NO_ERROR)
        {
            if(ERROR_SCSI_FACTORY_ERR_BAD_OPCODE == m_errorCode)
                modeSenseLength = 10;
            else
                return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_TRANSPORT_PROTOCAL), NULL, errorMsg);
        }
        else if(0 == modeSenseLength)
            modeSenseLength = 6;
    }
    if(10 == modeSenseLength)
    {
        if((errorMsg = scsiModeSense10(fileDescription, PROTOCOL_SPECIFIC_PORT_PAGE, 0,
                              MPAGE_CONTROL_CURRENT, buff, sizeof(buff))) != (ErrorMsg *) NO_ERROR)
            return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_TRANSPORT_PROTOCAL), NULL, errorMsg);
    }
    offset = getSCSIModePageOffset(buff, sizeof(buff), modeSenseLength);
    if((offset >= 0) && (buff[offset + 1] > 1))
    {
        if((0 == (buff[offset] & 0x40)) && (PROTOCOL_SPECIFIC_PORT_PAGE == (buff[offset] & 0x3f))) /* SPF==0 */
            *protocolType = (buff[offset + 2] & 0xf);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::getSCSIErrorLog(INT32 fileDescription, SCSI_ERROR_LOG* errorlog)
{
    ErrorMsg* errorMsg = NULL;
    struct scsiErrorCounter ecp;
    struct scsiNonMediumError nme;
    UINT8 buff[252];
    CHAR tempResultString[64];

    memset(buff, 0, sizeof(buff));

    if((errorMsg = scsiLogSense(fileDescription, READ_ERROR_COUNTER_LPAGE, 0, buff, LOG_RESP_LEN, 0)) != (ErrorMsg *)NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_ERROR_LOG), NULL, errorMsg);
    scsiDecodeErrCounterPage(buff, &ecp);
    errorlog->read.fastECC = ecp.counter[0];
    errorlog->read.delayedECC = ecp.counter[1];
    errorlog->read.reReadWrite = ecp.counter[2];
    errorlog->read.totalCorrected = ecp.counter[3];
    errorlog->read.algoInvocation = ecp.counter[4];
    sprintf(tempResultString, "%.3f", (ecp.counter[5] / 1000000000.0));
    errorlog->read.processedGigaBytes.assign(tempResultString);
    errorlog->read.totalUnCorrected = ecp.counter[6];

    if((errorMsg = scsiLogSense(fileDescription, WRITE_ERROR_COUNTER_LPAGE, 0, buff, LOG_RESP_LEN, 0)) != (ErrorMsg *)NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_ERROR_LOG), NULL, errorMsg);
    scsiDecodeErrCounterPage(buff, &ecp);
    errorlog->write.fastECC = ecp.counter[0];
    errorlog->write.delayedECC = ecp.counter[1];
    errorlog->write.reReadWrite = ecp.counter[2];
    errorlog->write.totalCorrected = ecp.counter[3];
    errorlog->write.algoInvocation = ecp.counter[4];
    sprintf(tempResultString, "%.3f", (ecp.counter[5] / 1000000000.0));
    errorlog->write.processedGigaBytes.assign(tempResultString);
    errorlog->write.totalUnCorrected = ecp.counter[6];

    if((errorMsg = scsiLogSense(fileDescription, VERIFY_ERROR_COUNTER_LPAGE, 0, buff, LOG_RESP_LEN, 0)) != (ErrorMsg *)NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_ERROR_LOG), NULL, errorMsg);
    scsiDecodeErrCounterPage(buff, &ecp);
    for (int k = 0; k < 7; ++k)
    {
        if (ecp.gotPC[k] && ecp.counter[k])
        {
            errorlog->verify.fastECC = ecp.counter[0];
            errorlog->verify.delayedECC = ecp.counter[1];
            errorlog->verify.reReadWrite = ecp.counter[2];
            errorlog->verify.totalCorrected = ecp.counter[3];
            errorlog->verify.algoInvocation = ecp.counter[4];
            sprintf(tempResultString, "%.3f", (ecp.counter[5] / 1000000000.0));
            errorlog->verify.processedGigaBytes.assign(tempResultString);
            errorlog->verify.totalUnCorrected = ecp.counter[6];
            break;
        }
    }

    if((errorMsg = scsiLogSense(fileDescription, NON_MEDIUM_ERROR_LPAGE, 0, buff, LOG_RESP_LEN, 0)) != (ErrorMsg *)NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_ERROR_LOG), NULL, errorMsg);
    scsiDecodeNonMediumErrPage(buff, &nme);
    if (nme.gotPC0)
        errorlog->nmErrorCount = nme.counterPC0;

    return NO_ERROR_MSG;
}

ErrorMsg *SCSIFactory::getSmartHealthStatus(INT32 fileDescription, string* statusString)
{
    ErrorMsg* errorMsg = NULL;
    UINT8 buff[252];
    UINT8 pagesize;
    STRING cp;
    string tempString;
    memset(buff, 0, sizeof(buff));

    if((errorMsg = scsiLogSense(fileDescription, IE_LPAGE, 0, buff, sizeof(buff), 0)) != (ErrorMsg *)NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_SMART_HEALTH_STATUS), NULL, errorMsg);
    // pull out page size from response, don't forget to add 4
    pagesize = ((buff[2] << 8) | buff[3]) + 4;
    if ((pagesize < 4) || buff[4] || buff[5])
        return returnErrorMsg(ERRORCODE(ERROR_SCSI_FACTORY_GET_SMART_HEALTH_STATUS), "bad parameter code or length", NULL);

    cp = scsiGetInformationExceptionString(buff[8], buff[9]);
    if(cp)
    {
        tempString.assign(cp);
        *statusString = tempString;
    }
    else
        *statusString = "OK";

    return NO_ERROR_MSG;
}

// Convenience function for formatting strings from SCSI identify
void SCSIFactory::scsiFormatIDtoString(STRING idBuffer, UINT8 idBufferLength, UINT8 *stringBuffer)
{
    char tmp[65];
    idBufferLength = idBufferLength > 64 ? 64 : idBufferLength;
    strncpy(tmp, (const char *)idBuffer, idBufferLength);
    tmp[idBufferLength] = '\0';

    // Find the first non-space character (maybe none).
    int first = -1;
    int i;
    for(i = 0; tmp[i]; i++)
    {
        if(!isspace((int)tmp[i]))
        {
            first = i;
            break;
        }
    }

    if(first == -1)
    {
        // There are no non-space characters.
        stringBuffer[0] = '\0';
        return;
    }

    // Find the last non-space character.
    for (i = strlen(tmp)-1; i >= first && isspace((int)tmp[i]); i--);
    int last = i;

    strncpy((char*)stringBuffer, tmp+first, last-first+1);
    stringBuffer[last-first+1] = '\0';
}

/* Decode VPD page 0x83 logical unit designator into a string. If both
 * numeric address and SCSI name string present, prefer the former.
 * Returns 0 on success, -1 on error with error string in s. */
int SCSIFactory::scsiDecodeLUDevID(STRING b, int blen, UINT8 *s, int slen, int *transport)
{
    int m, c_set, assoc, desig_type, i_len, naa, off, u, have_scsi_ns;
    STRING ucp;
    STRING ip;
    int si = 0;

    if (transport)
        *transport = -1;
    if (slen < 32) {
        if (slen > 0)
            s[0] = '\0';
        return -1;
    }
    have_scsi_ns = 0;
    s[0] = '\0';
    off = -1;
    while ((u = scsiVPDDevIDIter(b, blen, &off, -1, -1, -1)) == 0) {
        ucp = b + off;
        i_len = ucp[3];
        if ((off + i_len + 4) > blen) {
            snprintf((char *)s+si, slen-si, "error: designator length");
            return -1;
        }
        assoc = ((ucp[1] >> 4) & 0x3);
        if (transport && assoc && (ucp[1] & 0x80) && (*transport < 0))
            *transport = (ucp[0] >> 4) & 0xf;
        if (0 != assoc)
            continue;
        ip = ucp + 4;
        c_set = (ucp[0] & 0xf);
        desig_type = (ucp[1] & 0xf);

        switch (desig_type) {
        case 0: /* vendor specific */
        case 1: /* T10 vendor identification */
            break;
        case 2: /* EUI-64 based */
            if ((8 != i_len) && (12 != i_len) && (16 != i_len)) {
                snprintf((char *)s+si, slen-si, "error: EUI-64 length");
                return -1;
            }
            if (have_scsi_ns)
                si = 0;
            si += snprintf((char *)s+si, slen-si, "0x");
            for (m = 0; m < i_len; ++m)
                si += snprintf((char *)s+si, slen-si, "%02x", (unsigned int)ip[m]);
            break;
        case 3: /* NAA */
            if (1 != c_set) {
                snprintf((char *)s+si, slen-si, "error: NAA bad code_set");
                return -1;
            }
            naa = (ip[0] >> 4) & 0xff;
            if ((naa < 2) || (naa > 6) || (4 == naa)) {
                snprintf((char *)s+si, slen-si, "error: unexpected NAA");
                return -1;
            }
            if (have_scsi_ns)
                si = 0;
            if (2 == naa) {             /* NAA IEEE Extended */
                if (8 != i_len) {
                    snprintf((char *)s+si, slen-si, "error: NAA 2 length");
                    return -1;
                }
                si += snprintf((char *)s+si, slen-si, "0x");
                for (m = 0; m < 8; ++m)
                    si += snprintf((char *)s+si, slen-si, "%02x", (unsigned int)ip[m]);
            } else if ((3 == naa ) || (5 == naa)) {
                /* NAA=3 Locally assigned; NAA=5 IEEE Registered */
                if (8 != i_len) {
                    snprintf((char *)s+si, slen-si, "error: NAA 3 or 5 length");
                    return -1;
                }
                si += snprintf((char *)s+si, slen-si, "0x");
                for (m = 0; m < 8; ++m)
                    si += snprintf((char *)s+si, slen-si, "%02x", (unsigned int)ip[m]);
            } else if (6 == naa) {      /* NAA IEEE Registered extended */
                if (16 != i_len) {
                    snprintf((char *)s+si, slen-si, "error: NAA 6 length");
                    return -1;
                }
                si += snprintf((char *)s+si, slen-si, "0x");
                for (m = 0; m < 16; ++m)
                    si += snprintf((char *)s+si, slen-si, "%02x", (unsigned int)ip[m]);
            }
            break;
        case 4: /* Relative target port */
        case 5: /* (primary) Target port group */
        case 6: /* Logical unit group */
        case 7: /* MD5 logical unit identifier */
            break;
        case 8: /* SCSI name string */
            if (3 != c_set) {
                snprintf((char *)s+si, slen-si, "error: SCSI name string");
                return -1;
            }
            /* does %s print out UTF-8 ok?? */
            if (si == 0) {
                si += snprintf((char *)s+si, slen-si, "%s", (const char *)ip);
                ++have_scsi_ns;
            }
            break;
        default: /* reserved */
            break;
        }
    }
    if (-2 == u) {
        snprintf((char *)s+si, slen-si, "error: bad structure");
        return -1;
    }
    return 0;
}

/* Iterates to next designation descriptor in the device identification
 * VPD page. The 'initial_desig_desc' should point to start of first
 * descriptor with 'page_len' being the number of valid bytes in that
 * and following descriptors. To start, 'off' should point to a negative
 * value, thereafter it should point to the value yielded by the previous
 * call. If 0 returned then 'initial_desig_desc + *off' should be a valid
 * descriptor; returns -1 if normal end condition and -2 for an abnormal
 * termination. Matches association, designator_type and/or code_set when
 * any of those values are greater than or equal to zero. */
int SCSIFactory::scsiVPDDevIDIter(STRING initial_desig_desc, int page_len,
                                  int* off, int m_assoc, int m_desig_type, int m_code_set)
{
    STRING ucp;
    int k, c_set, assoc, desig_type;

    for (k = *off, ucp = initial_desig_desc ; (k + 3) < page_len; ) {
        k = (k < 0) ? 0 : (k + ucp[k + 3] + 4);
        if ((k + 4) > page_len)
            break;
        c_set = (ucp[k] & 0xf);
        if ((m_code_set >= 0) && (m_code_set != c_set))
            continue;
        assoc = ((ucp[k + 1] >> 4) & 0x3);
        if ((m_assoc >= 0) && (m_assoc != assoc))
            continue;
        desig_type = (ucp[k + 1] & 0xf);
        if ((m_desig_type >= 0) && (m_desig_type != desig_type))
            continue;
        *off = k;
        return 0;
    }
    return (k == page_len) ? -1 : -2;
}

ERRORCODE SCSIFactory::sg_io_cmnd_io(INT32 fileDescription, struct scsi_cmnd_io *iop)
{
     struct sg_io_hdr io_hdr;
     int res;
     m_errorCode = NO_ERROR;

     memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
     io_hdr.interface_id = 'S';
     io_hdr.cmd_len = iop->cmnd_len;
     io_hdr.mx_sb_len = iop->max_sense_len;
     io_hdr.dxfer_len = iop->dxfer_len;
     io_hdr.dxferp = iop->dxferp;
     io_hdr.cmdp = iop->cmnd;
     io_hdr.sbp = iop->sensep;
     /* sg_io_hdr interface timeout has millisecond units. Timeout of 0
        defaults to 60 seconds. */
     io_hdr.timeout = DEF_TIMEOUT;
     io_hdr.flags |= SG_FLAG_DIRECT_IO; /* direct I/O */
     switch(iop->dxfer_dir)
     {
         case DXFER_NONE:
             io_hdr.dxfer_direction = SG_DXFER_NONE;
             break;
         case DXFER_FROM_DEVICE:
             io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
             break;
         case DXFER_TO_DEVICE:
             io_hdr.dxfer_direction = SG_DXFER_TO_DEV;
             break;
         default:
             return ERROR_SCSI_BAD_DIRECTION;
     }
     iop->resp_sense_len = 0;
     iop->scsi_status = 0;
     iop->resid = 0;
     if(iop->dxfer_dir == DXFER_TO_DEVICE)
     {
         while(((res = ioctl(fileDescription, SG_IO, &io_hdr)) < 0) && (EINTR == errno))
             ;

         if (res < 0)
             m_errorCode = ERROR_SCSI_IOCTL_ACCESS_FAIL;
     }
     else
     {
         if(ioctl(fileDescription, SG_IO, &io_hdr) < 0)
         {
             m_errorCode = ERROR_SCSI_IOCTL_ACCESS_FAIL;
         }
     }
     return m_errorCode;
}

int SCSIFactory::getSCSIModePageOffset(UINT8 *buffer, UINT32 in_u32BufferSize, UINT8 modeLength)
{
    int resp_len, bd_len;
    int offset = -1;

    if(buffer)
    {
        if(10 == modeLength)
        {
            resp_len = (buffer[0] << 8) + buffer[1] + 2;
            bd_len = (buffer[6] << 8) + buffer[7];
            offset = bd_len + 8;
        }
        else
        {
            resp_len = buffer[0] + 1;
            bd_len = buffer[3];
            offset = bd_len + 4;
        }

        if((offset + 2) > in_u32BufferSize)
            offset = -1;
        else if ((offset + 2) > resp_len)
            offset = -1;
    }

    return offset;
}

void
SCSIFactory::scsiDecodeErrCounterPage(unsigned char * resp, struct scsiErrorCounter *ecp)
{
    int k, j, num, pl, pc;
    unsigned char * ucp;
    unsigned char * xp;
    uint64_t * ullp;

    memset(ecp, 0, sizeof(*ecp));
    num = (resp[2] << 8) | resp[3];
    ucp = &resp[0] + 4;
    while (num > 3) {
        pc = (ucp[0] << 8) | ucp[1];
        pl = ucp[3] + 4;
        switch (pc) {
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
                ecp->gotPC[pc] = 1;
                ullp = &ecp->counter[pc];
                break;
        default:
                ecp->gotExtraPC = 1;
                ullp = &ecp->counter[7];
                break;
        }
        k = pl - 4;
        xp = ucp + 4;
        if (k > (int)sizeof(*ullp)) {
            xp += (k - sizeof(*ullp));
            k = sizeof(*ullp);
        }
        *ullp = 0;
        for (j = 0; j < k; ++j) {
            if (j > 0)
                *ullp <<= 8;
            *ullp |= xp[j];
        }
        num -= pl;
        ucp += pl;
    }
}

void
SCSIFactory::scsiDecodeNonMediumErrPage(unsigned char *resp, struct scsiNonMediumError *nmep)
{
    int k, j, num, pl, pc, szof;
    unsigned char * ucp;
    unsigned char * xp;

    memset(nmep, 0, sizeof(*nmep));
    num = (resp[2] << 8) | resp[3];
    ucp = &resp[0] + 4;
    szof = sizeof(nmep->counterPC0);
    while (num > 3) {
        pc = (ucp[0] << 8) | ucp[1];
        pl = ucp[3] + 4;
        switch (pc) {
            case 0:
                nmep->gotPC0 = 1;
                k = pl - 4;
                xp = ucp + 4;
                if (k > szof) {
                    xp += (k - szof);
                    k = szof;
                }
                nmep->counterPC0 = 0;
                for (j = 0; j < k; ++j) {
                    if (j > 0)
                        nmep->counterPC0 <<= 8;
                    nmep->counterPC0 |= xp[j];
                }
                break;
            case 0x8009:
                nmep->gotTFE_H = 1;
                k = pl - 4;
                xp = ucp + 4;
                if (k > szof) {
                    xp += (k - szof);
                    k = szof;
                }
                nmep->counterTFE_H = 0;
                for (j = 0; j < k; ++j) {
                    if (j > 0)
                        nmep->counterTFE_H <<= 8;
                    nmep->counterTFE_H |= xp[j];
                }
                break;
            case 0x8015:
                nmep->gotPE_H = 1;
                k = pl - 4;
                xp = ucp + 4;
                if (k > szof) {
                    xp += (k - szof);
                    k = szof;
                }
                nmep->counterPE_H = 0;
                for (j = 0; j < k; ++j) {
                    if (j > 0)
                        nmep->counterPE_H <<= 8;
                    nmep->counterPE_H |= xp[j];
                }
                break;
        default:
                nmep->gotExtraPC = 1;
                break;
        }
        num -= pl;
        ucp += pl;
    }
}

STRING SCSIFactory::scsiGetInformationExceptionString(UINT8 asc, UINT8 ascq)
{
    const char * rp;

    if (SCSI_ASC_IMPENDING_FAILURE == asc) {
        if (ascq == 0xff)
            return "FAILURE PREDICTION THRESHOLD EXCEEDED (FALSE)";
        else if (ascq <
                 (sizeof(strs_for_asc_5d) / sizeof(strs_for_asc_5d[0]))) {
            rp = strs_for_asc_5d[ascq];
            if (strlen(rp) > 0)
                return rp;
        }
        snprintf(spare_buff, sizeof(spare_buff),
                 "FAILURE PREDICTION THRESHOLD EXCEEDED: ascq=0x%x", ascq);
        return spare_buff;
    } else if (SCSI_ASC_WARNING == asc) {
        if (ascq < (sizeof(strs_for_asc_b) / sizeof(strs_for_asc_b[0]))) {
            rp = strs_for_asc_b[ascq];
            if (strlen(rp) > 0)
                return rp;
        }
        snprintf(spare_buff, sizeof(spare_buff), "WARNING: ascq=0x%x", ascq);
        return spare_buff;
    }
    return NULL;        /* not a IE additional sense code */
}

/* READ DEFECT (12) command. 
Returns 0 if ok,
1 if NOT READY,
2 if * command not supported,
3 if field in command not supported or Returns * negated errno. 
SBC-3 section 5.18 (rev 35; vale Mark Evans) */
ErrorMsg *SCSIFactory::scsiReadDefect12(INT32 in_s32FileDescription, UINT32 in_u32RequestPList, UINT32 in_u32RequestGList, UINT32 in_u32DLFormat, UINT32 in_u32AddrDescIndex, UINT8 *out_u8pBuffer, UINT32 in_u32BufferLength)
{
     SCSI_COMMAND_CDB iop;
     UINT8 u8aCDB[12];
     memset(&iop, 0, sizeof(iop));
     memset(u8aCDB, 0, sizeof(u8aCDB));
     memset(m_senseBuff, 0, sizeof(m_senseBuff));

     iop.dxfer_dir = DXFER_FROM_DEVICE;
     iop.dxfer_len = in_u32BufferLength;
     iop.dxferp = out_u8pBuffer;
     u8aCDB[0] = SCSI_OPCODE_READ_DEFECT_DATA_12;//READ_DEFECT_12;
     u8aCDB[1] = (unsigned char)(((in_u32RequestPList << 4) & 0x10) |
				 ((in_u32RequestGList << 3) & 0x8) | (in_u32DLFormat & 0x7));
     u8aCDB[2] = (in_u32AddrDescIndex >> 24) & 0xff;
     u8aCDB[3] = (in_u32AddrDescIndex >> 16) & 0xff;
     u8aCDB[4] = (in_u32AddrDescIndex >> 8) & 0xff;
     u8aCDB[5] = in_u32AddrDescIndex & 0xff;
     u8aCDB[6] = (in_u32BufferLength >> 24) & 0xff;
     u8aCDB[7] = (in_u32BufferLength >> 16) & 0xff;
     u8aCDB[8] = (in_u32BufferLength >> 8) & 0xff;
     u8aCDB[9] = in_u32BufferLength & 0xff;
     iop.cmnd = u8aCDB;
     iop.cmnd_len = sizeof(u8aCDB);
     iop.sensep = m_senseBuff;
     iop.max_sense_len = sizeof(m_senseBuff);
     iop.timeout = SCSI_TIMEOUT_DEFAULT;

     if((m_errorCode = sg_io_cmnd_io(in_s32FileDescription, &iop)) != NO_ERROR)
     {
	  return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_DEFECT12 | m_errorCode), NULL, NULL);
     }

     if((m_errorCode = senseAnylyzer->translateSenseDataToErrorCode(m_senseBuff, sizeof(m_senseBuff))) != NO_ERROR)
     {
	  return returnErrorMsg((ERRORCODE) (ERROR_SCSI_FACTORY_READ_DEFECT12 | m_errorCode), NULL, NULL);
     }

     return NO_ERROR_MSG;

//     if (!device->scsi_pass_through(&io_hdr))
//	  return -device->get_errno();
//     scsi_do_m_senseBuff_disect(&io_hdr, &sinfo);
//     return scsiSimpleM_SenseBuffFilter(&sinfo);
}

