#ifndef SCSI_FACTORY_HPP
#define SCSI_FACTORY_HPP

#include <iostream>
#include <sstream>
#include <string>
#include <list>
#include <scsi/sg.h>
#include <sys/ioctl.h>
#include "BaseClassDDAPI.hpp"
#include "SCSISenseAnalyzer.hpp"

#define LOG_RESP_LEN 252
#define LOG_RESP_LONG_LEN ((62 * 256) + 252)
#define LOG_RESP_TAPE_ALERT_LEN 0x144

#define SCSI_OPCODE_TEST_UNIT_READY            0x00
#define SCSI_OPCODE_INQUIRY                    0x12
#define SCSI_OPCODE_FORMAT                     0x04
#define SCSI_OPCODE_SEND_DIAGNOSTIC            0x1d
#define SCSI_OPCODE_RECEIVE_DIAGNOSTIC_RESULTS 0x1c
#define SCSI_OPCODE_REPORT_LUNS                0xa0
#define SCSI_OPCODE_REQUEST_SENSE              0x03
#define SCSI_OPCODE_READ_CAPACITY_10           0x25
#define SCSI_OPCODE_READ_CAPACITY_16           0x9e
#define SCSI_OPCODE_LOG_SELECT                 0x4c
#define SCSI_OPCODE_LOG_SENSE                  0x4d
#define SCSI_OPCODE_MODE_SELECT                0x15
#define SCSI_OPCODE_MODE_SELECT_10             0x55
#define SCSI_OPCODE_RESERVE_10                 0x56
#define SCSI_OPCODE_RELEASE_10                 0x57
#define SCSI_OPCODE_MODE_SENSE                 0x1a
#define SCSI_OPCODE_MODE_SENSE_10              0x5a


#ifndef SAI_READ_CAPACITY_16    /* service action for READ_CAPACITY_16 */
#define SAI_READ_CAPACITY_16  0x10
#endif

#define SCSI_OPCODE_READ_6      0x08
#define SCSI_OPCODE_READ_10     0x28
#define SCSI_OPCODE_READ_16     0x88
#define SCSI_OPCODE_WRITE_6     0x0a
#define SCSI_OPCODE_WRITE_10    0x2a
#define SCSI_OPCODE_WRITE_16    0x8a

#define SCSI_OPCODE_READ_BUFFER  0x3c
#define SCSI_OPCODE_WRITE_BUFFER 0x3b

#define SCSI_OPCODE_READ_DEFECT_DATA_12 0xb7

#define MAX_SCSI_CDBSZ 16
#define SENSE_BUFF_LEN 32
#define DEF_BLOCK_SIZE 512
#define DEF_TIMEOUT 40000       /* 40,000 millisecs == 40 seconds */
#define SCSI_SENSE_BUFFERSIZE   96
#define MAX_SCSI_XSFER_BLKS     (512)

#define SCSI_GOOD    0

#define DXFER_NONE        0
#define DXFER_FROM_DEVICE 1
#define DXFER_TO_DEVICE   2

/* ANSI SCSI-3 Mode Pages */
#define VENDOR_UNIQUE_PAGE                       0x00
#define READ_WRITE_ERROR_RECOVERY_PAGE           0x01
#define DISCONNECT_RECONNECT_PAGE                0x02
#define FORMAT_DEVICE_PAGE                       0x03
#define RIGID_DISK_DRIVE_GEOMETRY_PAGE           0x04
#define FLEXIBLE_DISK_PAGE                       0x05
#define VERIFY_ERROR_RECOVERY_PAGE               0x07
#define CACHING_PAGE                             0x08
#define PERIPHERAL_DEVICE_PAGE                   0x09
#define XOR_CONTROL_MODE_PAGE                    0x10
#define CONTROL_MODE_PAGE                        0x0a
#define MEDIUM_TYPES_SUPPORTED_PAGE              0x0b
#define NOTCH_PAGE                               0x0c
#define CD_DEVICE_PAGE                           0x0d
#define CD_AUDIO_CONTROL_PAGE                    0x0e
#define DATA_COMPRESSION_PAGE                    0x0f
#define ENCLOSURE_SERVICES_MANAGEMENT_PAGE       0x14
#define PROTOCOL_SPECIFIC_LUN_PAGE               0x18
#define PROTOCOL_SPECIFIC_PORT_PAGE              0x19
#define POWER_CONDITION_PAGE                     0x1a
#define INFORMATIONAL_EXCEPTIONS_CONTROL_PAGE    0x1c
#define FAULT_FAILURE_REPORTING_PAGE             0x1c

/* ANSI SCSI-3 Log Pages retrieved by LOG SENSE. */
#define SUPPORTED_LPAGES                        0x00
#define BUFFER_OVERRUN_LPAGE                    0x01
#define WRITE_ERROR_COUNTER_LPAGE               0x02
#define READ_ERROR_COUNTER_LPAGE                0x03
#define READ_REVERSE_ERROR_COUNTER_LPAGE        0x04
#define VERIFY_ERROR_COUNTER_LPAGE              0x05
#define NON_MEDIUM_ERROR_LPAGE                  0x06
#define LAST_N_ERROR_LPAGE                      0x07
#define FORMAT_STATUS_LPAGE                     0x08
#define LB_PROV_LPAGE                           0x0c   /* SBC-3 */
#define TEMPERATURE_LPAGE                       0x0d
#define STARTSTOP_CYCLE_COUNTER_LPAGE           0x0e
#define APPLICATION_CLIENT_LPAGE                0x0f
#define SELFTEST_RESULTS_LPAGE                  0x10
#define SS_MEDIA_LPAGE                          0x11   /* SBC-3 */
#define BACKGROUND_RESULTS_LPAGE                0x15   /* SBC-3 */
#define NONVOL_CACHE_LPAGE                      0x17   /* SBC-3 */
#define PROTOCOL_SPECIFIC_LPAGE                 0x18
#define IE_LPAGE                                0x2f

/* Background control mode subpage is [0x1c,0x1] */
#define BACKGROUND_CONTROL_M_SUBPAGE             0x1   /* SBC-2 */

#define ALL_MODE_PAGES                           0x3f

/* Mode page control field */
#define MPAGE_CONTROL_CURRENT               0
#define MPAGE_CONTROL_CHANGEABLE            1
#define MPAGE_CONTROL_DEFAULT               2
#define MPAGE_CONTROL_SAVED                 3

/* SCSI Vital Product Data (VPD) pages */
#define SCSI_VPD_SUPPORTED_VPD_PAGES    0x0
#define SCSI_VPD_UNIT_SERIAL_NUMBER     0x80
#define SCSI_VPD_DEVICE_IDENTIFICATION  0x83
#define SCSI_VPD_EXTENDED_INQUIRY_DATA  0x86
#define SCSI_VPD_ATA_INFORMATION        0x89
#define SCSI_VPD_POWER_CONDITION        0x8a
#define SCSI_VPD_POWER_CONSUMPTION      0x8d
#define SCSI_VPD_BLOCK_LIMITS           0xb0
#define SCSI_VPD_BLOCK_DEVICE_CHARACTERISTICS   0xb1
#define SCSI_VPD_LOGICAL_BLOCK_PROVISIONING     0xb2

#define SCSI_ASCQ_ATA_PASS_THROUGH      0x1d

/* defines for function code parameter in SENDDIAGNOSTIC function */
#define SCSI_DIAG_NO_SELF_TEST          0x00
#define SCSI_DIAG_DEF_SELF_TEST         0xff
#define SCSI_DIAG_BG_SHORT_SELF_TEST    0x01
#define SCSI_DIAG_BG_EXTENDED_SELF_TEST 0x02
#define SCSI_DIAG_FG_SHORT_SELF_TEST    0x05
#define SCSI_DIAG_FG_EXTENDED_SELF_TEST 0x06
#define SCSI_DIAG_ABORT_SELF_TEST       0x04

/* SCSI command timeout values (units are seconds) */
#define SCSI_TIMEOUT_DEFAULT    20  // should be longer than the spin up time
                                    // of a disk in standby mode.
#define SCSI_TIMEOUT_SELF_TEST  (5 * 60 * 60)   // allow max 5 hours for
                                                // extended foreground self test

// SCSI_CDB_TYPE definitions
typedef enum
{
    SCSI_CDB_6_BYTE = 0,
    SCSI_CDB_10_BYTE,
    SCSI_CDB_12_BYTE,
    SCSI_CDB_16_BYTE,
} SCSI_CDB_TYPE;

// SCSI_CDB definition
typedef struct
{
    SCSI_CDB_TYPE cdbType;
    union scsiCDB
    {
        struct {
                UINT8 opCode;
                UINT8 lun;
                UINT8 reservedField1;
                UINT8 reservedField2;
                UINT8 allocationLength;
                UINT8 control;
        } cdb6;

        struct {
                UINT8 opCode;
                UINT8 lun;
                UINT8 logicalBlockByte2;
                UINT8 logicalBlockByte3;
                UINT8 logicalBlockByte4;
                UINT8 logicalBlockByte5;
                UINT8 miscData;
                UINT8 allocationLengthByte7;
                UINT8 allocationLengthByte8;
                UINT8 control;
        } cdb10;

        struct {
                UINT8 opCode;
                UINT8 lun;
                UINT8 logicalBlockByte2;
                UINT8 logicalBlockByte3;
                UINT8 logicalBlockByte4;
                UINT8 logicalBlockByte5;
                UINT8 allocationLengthByte6;
                UINT8 allocationLengthByte7;
                UINT8 allocationLengthByte8;
                UINT8 allocationLengthByte9;
                UINT8 miscData;
                UINT8 control;
        } cdb12;

        struct {
                UINT8 opCode;
                UINT8 lun;
                UINT8 logicalBlockByte2;
                UINT8 logicalBlockByte3;
                UINT8 logicalBlockByte4;
                UINT8 logicalBlockByte5;
                UINT8 additionalCBPByte6;
                UINT8 additionalCBPByte7;
                UINT8 additionalCBPByte8;
                UINT8 additionalCBPByte9;
                UINT8 allocationLengthByte10;
                UINT8 allocationLengthByte11;
                UINT8 allocationLengthByte12;
                UINT8 allocationLengthByte13;
                UINT8 miscData;
                UINT8 control;
        } cdb16;
    } cdb;
} SCSI_BLOCK_CDB;

static const char * transport_proto_arr[] = {
        "Fibre channel (FCP-2)",
        "Parallel SCSI (SPI-4)",
        "SSA",
        "IEEE 1394 (SBP-2)",
        "RDMA (SRP)",
        "iSCSI",
        "SAS (SPL-3)",
        "ADT",
        "ATA (ACS-2)",
        "UAS",
        "SOP",
        "0xb",
        "0xc",
        "0xd",
        "0xe",
        "0xf"
};

// refer to smartctl utility scsicmds.h
struct scsi_cmnd_io
{
    UINT8 * cmnd;       /* [in]: ptr to SCSI command block (cdb) */
    size_t  cmnd_len;   /* [in]: number of bytes in SCSI command */
    int dxfer_dir;      /* [in]: DXFER_NONE, DXFER_FROM_DEVICE, or
                                 DXFER_TO_DEVICE */
    UINT8 * dxferp;     /* [in]: ptr to outgoing or incoming data buffer */
    size_t dxfer_len;   /* [in]: bytes to be transferred to/from dxferp */
    UINT8 * sensep;     /* [in]: ptr to sense buffer, filled when
                                 CHECK CONDITION status occurs */
    size_t max_sense_len; /* [in]: max number of bytes to write to sensep */
    unsigned timeout;   /* [in]: seconds, 0-> default timeout (60 seconds?) */
    size_t resp_sense_len;  /* [out]: sense buffer length written */
    UINT8 scsi_status;  /* [out]: 0->ok, 2->CHECK CONDITION, etc ... */
    int resid;          /* [out]: Number of bytes requested to be transferred
                                  less actual number transferred (0 if not
                                   supported) */
};
typedef scsi_cmnd_io SCSI_COMMAND_CDB;

/* Carrier for Error counter log pages (e.g. read, write, verify ...) */
struct scsiErrorCounter {
    UINT8 gotPC[7];
    UINT8 gotExtraPC;
    uint64_t counter[8];
};

/* Carrier for Non-medium error log page */
struct scsiNonMediumError {
    UINT8 gotPC0;
    UINT8 gotExtraPC;
    uint64_t counterPC0;
    UINT8 gotTFE_H;
    uint64_t counterTFE_H; /* Track following errors [Hitachi] */
    UINT8 gotPE_H;
    uint64_t counterPE_H;  /* Positioning errors [Hitachi] */
};

/* this is a subset of the SCSI additional sense code strings indexed
 * by "ascq" for the case when asc==SCSI_ASC_IMPENDING_FAILURE (0x5d)
 */
static const char * strs_for_asc_5d[] = {
   /* 0x00 */   "FAILURE PREDICTION THRESHOLD EXCEEDED",
        "MEDIA FAILURE PREDICTION THRESHOLD EXCEEDED",
        "LOGICAL UNIT FAILURE PREDICTION THRESHOLD EXCEEDED",
        "SPARE AREA EXHAUSTION PREDICTION THRESHOLD EXCEEDED",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
   /* 0x10 */   "HARDWARE IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "HARDWARE IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "HARDWARE IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "HARDWARE IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "HARDWARE IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "HARDWARE IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "HARDWARE IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "HARDWARE IMPENDING FAILURE CHANNEL PARAMETRICS",
        "HARDWARE IMPENDING FAILURE CONTROLLER DETECTED",
        "HARDWARE IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "HARDWARE IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "HARDWARE IMPENDING FAILURE SPIN-UP RETRY COUNT",
        "HARDWARE IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT",
        "",
        "",
        "",
   /* 0x20 */   "CONTROLLER IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "CONTROLLER IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "CONTROLLER IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "CONTROLLER IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "CONTROLLER IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "CONTROLLER IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "CONTROLLER IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "CONTROLLER IMPENDING FAILURE CHANNEL PARAMETRICS",
        "CONTROLLER IMPENDING FAILURE CONTROLLER DETECTED",
        "CONTROLLER IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "CONTROLLER IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "CONTROLLER IMPENDING FAILURE SPIN-UP RETRY COUNT",
        "CONTROLLER IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT",
        "",
        "",
        "",
   /* 0x30 */   "DATA CHANNEL IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "DATA CHANNEL IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "DATA CHANNEL IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "DATA CHANNEL IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "DATA CHANNEL IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "DATA CHANNEL IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "DATA CHANNEL IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "DATA CHANNEL IMPENDING FAILURE CHANNEL PARAMETRICS",
        "DATA CHANNEL IMPENDING FAILURE CONTROLLER DETECTED",
        "DATA CHANNEL IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "DATA CHANNEL IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "DATA CHANNEL IMPENDING FAILURE SPIN-UP RETRY COUNT",
        "DATA CHANNEL IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT",
        "",
        "",
        "",
   /* 0x40 */   "SERVO IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "SERVO IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "SERVO IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "SERVO IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "SERVO IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "SERVO IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "SERVO IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "SERVO IMPENDING FAILURE CHANNEL PARAMETRICS",
        "SERVO IMPENDING FAILURE CONTROLLER DETECTED",
        "SERVO IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "SERVO IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "SERVO IMPENDING FAILURE SPIN-UP RETRY COUNT",
        "SERVO IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT",
        "",
        "",
        "",
   /* 0x50 */   "SPINDLE IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "SPINDLE IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "SPINDLE IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "SPINDLE IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "SPINDLE IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "SPINDLE IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "SPINDLE IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "SPINDLE IMPENDING FAILURE CHANNEL PARAMETRICS",
        "SPINDLE IMPENDING FAILURE CONTROLLER DETECTED",
        "SPINDLE IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "SPINDLE IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "SPINDLE IMPENDING FAILURE SPIN-UP RETRY COUNT",
        "SPINDLE IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT",
        "",
        "",
        "",
   /* 0x60 */   "FIRMWARE IMPENDING FAILURE GENERAL HARD DRIVE FAILURE",
        "FIRMWARE IMPENDING FAILURE DRIVE ERROR RATE TOO HIGH",
        "FIRMWARE IMPENDING FAILURE DATA ERROR RATE TOO HIGH",
        "FIRMWARE IMPENDING FAILURE SEEK ERROR RATE TOO HIGH",
        "FIRMWARE IMPENDING FAILURE TOO MANY BLOCK REASSIGNS",
        "FIRMWARE IMPENDING FAILURE ACCESS TIMES TOO HIGH",
        "FIRMWARE IMPENDING FAILURE START UNIT TIMES TOO HIGH",
        "FIRMWARE IMPENDING FAILURE CHANNEL PARAMETRICS",
        "FIRMWARE IMPENDING FAILURE CONTROLLER DETECTED",
        "FIRMWARE IMPENDING FAILURE THROUGHPUT PERFORMANCE",
        "FIRMWARE IMPENDING FAILURE SEEK TIME PERFORMANCE",
        "FIRMWARE IMPENDING FAILURE SPIN-UP RETRY COUNT",
   /* 0x6c */   "FIRMWARE IMPENDING FAILURE DRIVE CALIBRATION RETRY COUNT"};


/* this is a subset of the SCSI additional sense code strings indexed
 *  * by "ascq" for the case when asc==SCSI_ASC_WARNING (0xb)
 *   */
static const char * strs_for_asc_b[] = {
       /* 0x00 */   "WARNING",
               "WARNING - SPECIFIED TEMPERATURE EXCEEDED",
               "WARNING - ENCLOSURE DEGRADED"};


using namespace std;

/// @class      SCSIFactory SCSIFactory.hpp "SCSIFactory.hpp""
/// @brief      Analyze SCSI sense buffer
class SCSIFactory : public BaseClassDDAPI
{
public:
     SCSIFactory(STRING className);
     ~SCSIFactory(void);

     ErrorMsg *executeSCSIRead(INT32 fileDescription, UINT64 in_u64StartLogicalSector,
                        UINT32 numberOfBlocksToRead, UINT8* buffer, UINT32 in_u32BufferSize);
     ErrorMsg *executeSCSIWrite(INT32 fileDescription, UINT64 in_u64StartLogicalSector,
                         UINT32 numberOfBlocksToWrite, UINT8 *buffer, UINT32 in_u32BufferSize);
     /* refer to smartctl utility */
     /* STANDARD SCSI Commands  */
     ErrorMsg *scsiTestUnitReady(INT32 fileDescription);
     ErrorMsg *scsiStdInquiry(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiInquiryVpd(INT32 fileDescription, UINT8 vpdPageNumber, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiLogSense(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
                            UINT8 *buffer, UINT32 in_u32BufferSize, UINT8 knownRespLength);
     ErrorMsg *scsiLogSelect(INT32 fileDescription, UINT8 pcr, UINT8 sp, UINT8 pageControl, UINT8 pageNumber,
    		                 UINT8 subpageNumber, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiModeSense(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
                             UINT8 pageControl, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiModeSelect(INT32 fileDescription, UINT8 sp, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiModeSense10(INT32 fileDescription, UINT8 pageNumber, UINT8 subPageNumber,
                               UINT8 pageControl, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiModeSelect10(INT32 fileDescription, UINT8 sp, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiRequestSense(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiSendDiagnostic(INT32 fileDescription, UINT8 functioncode, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiReadCapacity10(INT32 fileDescription, UINT32 *lastLogicalBlock, UINT32 *logicalBlockSize);
     ErrorMsg *scsiReadCapacity16(INT32 fileDescription, UINT8 *buffer, UINT32 in_u32BufferSize);
     ErrorMsg *scsiGetProtectionPhysicalInfo(INT32 fileDescription, UINT8 *rc16_12_31p);
     ErrorMsg *getSCSIDriveRPM(INT32 fileDescription, UINT8 modeSenseLength, int *rpm, int *formFactor, int *zonedBlockCapable);
     ErrorMsg *getSCSIDriveTemperature(INT32 fileDescription, UINT8 *currenttemp, UINT8 *triptemp);
     ErrorMsg *getSCSITransportProtocol(INT32 fileDescription, UINT8 modeSenseLength, int *protocolType);
     ErrorMsg *getSCSIErrorLog(INT32 fileDescription, SCSI_ERROR_LOG* errorlog);
     ErrorMsg *getSmartHealthStatus(INT32 fileDescription, string* statusString);
     ErrorMsg *scsiReadDefect12(INT32 in_s32FileDescritpion, UINT32 in_u32RequestPList, UINT32 in_u32RequestGList, UINT32 in_u32DLFormat, UINT32 in_u32AddrDescIndex, UINT8 *out_u8pBuffer, UINT32 in_u32BufferLength);

     UINT8 *getCurrentSenseBuffer(void);
     void scsiFormatIDtoString(STRING idBuffer, UINT8 idBufferLength, UINT8 *stringBuffer);
     int scsiDecodeLUDevID(STRING b, int blen, UINT8 *s, int slen, int *transport);
     int scsiVPDDevIDIter(STRING initial_desig_desc, int page_len, int *off, int m_assoc,
                              int m_desig_type, int m_code_set);
protected:

private:
     ERRORCODE sg_io_cmnd_io(INT32 fileDescription, struct scsi_cmnd_io *iop);
     int getSCSIModePageOffset(UINT8 *buffer, UINT32 in_u32BufferSize, UINT8 modeLength);
     void scsiDecodeErrCounterPage(UINT8* resp, struct scsiErrorCounter *ecp);
     void scsiDecodeNonMediumErrPage(UINT8* resp, struct scsiNonMediumError *nmep);
     STRING scsiGetInformationExceptionString(UINT8 asc, UINT8 ascq);

     SCSISenseAnalyzer *senseAnylyzer;
     UINT8 m_senseBuff[SCSI_SENSE_BUFFERSIZE];
};
#endif
