/// @file       StorageDDAPIOperation.cpp
/// @brief      StorageDDAPIOperation class
/// @details    Provide low layer diagnostic and utility APIs for STORAGE module
/// @version    1.0.0
/// @date       2015.12.01
/// @author     Ben Yang
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#include "StorageDDAPIOperation.hpp"

/// @fn         StorageDDAPIOperation::StorageDDAPIOperation(STRING in_strClassName):BaseClassDDAPI(in_szClassName)
/// @brief      StorageOperation constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
StorageDDAPIOperation::StorageDDAPIOperation(STRING in_szClassName) : BaseClassDDAPI(in_szClassName)
{
     m_pSASDDAPI = new SASDDAPI("SAS");
     if(m_pSASDDAPI == NULL)
     {
	  addErrorMsg(ERROR_STORAGE_DDAPI_CONSTRUCTOR, NULL);
     }
     m_pNVMeDDAPI = new NVMeDDAPI("NVME");
     if(m_pNVMeDDAPI == NULL)
     {
	  addErrorMsg(ERROR_STORAGE_DDAPI_CONSTRUCTOR, NULL);
     }
     m_pUSBDDAPI = new USBDDAPI("USB");
     if(m_pUSBDDAPI == NULL)
     {
	  addErrorMsg(ERROR_STORAGE_DDAPI_CONSTRUCTOR, NULL);
     }

     if((m_pSCSIFactory = new SCSIFactory("SCSIFactory")) == NULL)
     {
	  addErrorMsg(ERROR_STORAGE_DDAPI_CONSTRUCTOR, "Create SCSI factory fail");
     }

     m_storageDriveList.clear();

     ErrorMsg * pErrorMsg = NO_ERROR_MSG;
     if((pErrorMsg = generateStorageList()) != NO_ERROR_MSG)
     {
	  addErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Generate storage list fail");
     }
//    generateStorageList();
}

/// @fn         StorageOperation::~StorageOperation(void)
/// @brief      StorageOperation destructor
/// @exception  Nothing
/// @return     Nothing
StorageDDAPIOperation::~StorageDDAPIOperation(void)
{
     if(m_pSASDDAPI != NULL)
     {
	  delete m_pSASDDAPI;
     }
     if(m_pNVMeDDAPI != NULL)
     {
	  delete m_pNVMeDDAPI;
     }
     if(m_pUSBDDAPI != NULL)
     {
	  delete m_pUSBDDAPI;
     }
     if (m_pSCSIFactory != NULL)
     {
        delete m_pSCSIFactory;
     }
}

/// @fn         ErrorMsg *StorageDDAPI::IOCTL(char *in_szDeviceNodeName, int in_eOperation, STORAGEInformationXMLData *inout_pXMLData)
/// @brief      Function for abstraction of DDAPI IOCTL operation
/// @param      in_u32DefinedIndex is a index for STORAGE device node name (eg. 1)
/// @param      in_eOperation desired driver opertaion
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg *StorageDDAPIOperation::IOCTL(UINT32 in_u32DefinedIndex, StorageIOCTLOperation in_eOperation)
{
     ERRORCODE errorCode = NO_ERROR;
     ErrorMsg *pErrorMsg = NULL;

     switch(in_eOperation)
     {
     case eStorageGenerateList:
          pErrorMsg = generateStorageList();
	  if(pErrorMsg != NO_ERROR_MSG)
	  {
	       return returnErrorMsg(ERROR_STORAGE_DDAPI_IOCTL, NULL, pErrorMsg);
	  }
	  break;

     case eStorageListDrive:
	  pErrorMsg = listDrive();
	  if(pErrorMsg != NO_ERROR_MSG)
	  {
	       return returnErrorMsg(ERROR_STORAGE_DDAPI_IOCTL, NULL, pErrorMsg);
	  }
	  break;

     default:
	  errorCode = ERROR_STORAGE_DDAPI_NO_SUCH_IOCTL_OPERATION;
	  return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_IOCTL | errorCode), NULL, NULL);
     }

     return pErrorMsg;
}

ErrorMsg *StorageDDAPIOperation::generateStorageList()
{
     ERRORCODE errorCode = NO_ERROR;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;
     vector<SAS_CONTROLLER_INFO> sasList;
     vector<SAS_DRIVE_INFO> sasDriveList;
     vector<NVMe_DEVICE_INFO> nvmeList;
     list<string> usbList;

     // generate SAS list
     if((pErrorMsg = m_pSASDDAPI->getControllerList(&sasList)) != NO_ERROR_MSG)
     {
	  return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST), NULL, pErrorMsg);
     }

     // generate NVME list
     if((pErrorMsg = m_pNVMeDDAPI->getControllerList(&nvmeList)) != NO_ERROR_MSG)
     {
	  return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST), NULL, pErrorMsg);
     }

     // generate USB list
     if((pErrorMsg = m_pUSBDDAPI->getUSBDevicesList(&usbList)) != NO_ERROR_MSG)
     {
	  return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST), NULL, pErrorMsg);
     }

     UINT32 u32DefinedIndex = 0;
     // copy list data
     list <StorageDriveStruct> *pStorageDriveList = &m_storageDriveList;

     // get SAS list data
     for(vector<SAS_CONTROLLER_INFO>::iterator sasIter = sasList.begin(); sasIter != sasList.end(); sasIter++)
     {
	  pErrorMsg = m_pSASDDAPI->getDriveListByController((sasIter - sasList.begin()), &sasDriveList);
	  if(pErrorMsg != NO_ERROR_MSG)
	       return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST), NULL, pErrorMsg);

	  for(vector<SAS_DRIVE_INFO>::iterator driveIter = sasDriveList.begin(); driveIter != sasDriveList.end(); driveIter++)
	  {
	       StorageDriveStruct tempStorageDriveStruct;

	       pErrorMsg = m_pSASDDAPI->getDriveInfoByName(driveIter->wellKnownName, &(*driveIter));
	       if(pErrorMsg != NO_ERROR_MSG)
		    continue;

	       tempStorageDriveStruct.u32ControllerType = eStorageControllerTypeSAS;//0; //0:SAS
	       tempStorageDriveStruct.u32DefinedIndex = u32DefinedIndex;
	       tempStorageDriveStruct.strDefinedName = driveIter->wellKnownName;
	       tempStorageDriveStruct.strDeviceNodeName = driveIter->deviceNodeName;
	       tempStorageDriveStruct.strSGDeviceNodeName = driveIter->sgDeviceNodeName;
	       tempStorageDriveStruct.strSerialNumber = driveIter->serialNumber;
	       tempStorageDriveStruct.strVendorName = driveIter->vendorName;
	       tempStorageDriveStruct.strRotationRate = driveIter->rotationRate;
	       tempStorageDriveStruct.strFormFactor = driveIter->formFactor;
	       tempStorageDriveStruct.strModelName = driveIter->productName;
	       tempStorageDriveStruct.u64CapacityMB = driveIter->capacity/1048576;
	       tempStorageDriveStruct.strLinkSpeed = driveIter->sasPhyInfo[0].linkRate;
	       tempStorageDriveStruct.strFirmwareVersion = driveIter->revision;
	       tempStorageDriveStruct.strTemperature = driveIter->temperature;
	       tempStorageDriveStruct.strProtocalType = driveIter->protocalType;
	       tempStorageDriveStruct.u32LogicalBlockSize = driveIter->logicalBlockSize;
	       tempStorageDriveStruct.u32PhysicalBlockSize = driveIter->physicalBlockSize;
	       tempStorageDriveStruct.u64SASAddress = driveIter->sasAddress;
	       tempStorageDriveStruct.SASSMARTData = driveIter->smartData;

	       pStorageDriveList->push_back(tempStorageDriveStruct);

	       u32DefinedIndex++;
	  }
     }

     // get NVME list data
     for(vector<NVMe_DEVICE_INFO>::iterator nvmeIter = nvmeList.begin(); nvmeIter != nvmeList.end(); nvmeIter++)
     {
	  StorageDriveStruct tempStorageDriveStruct;
	  long long int lba;
	  double nsze;

	  lba = nvmeIter->idNs.lbaf[(nvmeIter->idNs.flbas & 0x0f)].ds;
	  lba  = (1 << lba);
	  nsze = nvmeIter->idNs.nsze;
	  nsze *= lba;
	  tempStorageDriveStruct.u32ControllerType = eStorageControllerTypeNVME;//3; //3:NVME
	  tempStorageDriveStruct.u32DefinedIndex = u32DefinedIndex;
	  tempStorageDriveStruct.strDefinedName = nvmeIter->wellKnowName;
	  tempStorageDriveStruct.strDeviceNodeName = nvmeIter->blockdeviceNodeName;
	  tempStorageDriveStruct.strSGDeviceNodeName = nvmeIter->blockdeviceNodeName;
	  tempStorageDriveStruct.u64CapacityMB = nsze/1048576;
	  tempStorageDriveStruct.u32PhysicalBlockSize = lba;
	  tempStorageDriveStruct.u32LogicalBlockSize = lba;

	  pStorageDriveList->push_back(tempStorageDriveStruct);

	  u32DefinedIndex++;
     }

     // get USB list data
     for(list<string>::iterator USBIter = usbList.begin(); USBIter != usbList.end(); USBIter++)
     {
	  UINT32 u32DefinedIndex = -1;
	  list<string>  Infolist;
	  //list<string>::iterator tempInfo;
	  pErrorMsg = m_pUSBDDAPI->getIndexByName(USBIter->c_str(), &u32DefinedIndex);
	  if (pErrorMsg != NO_ERROR_MSG)
	  {
	       //getErrorMessage(pErrorMsg, &strErrorMessage);
	       //cout << strErrorMessage.c_str() << endl;
	       //return pErrorMsg;
	       continue;
	  }

	  StorageDriveStruct tempStorageDriveStruct;
	  long long int lba;
	  double nsze;

	  tempStorageDriveStruct.u32ControllerType = eStorageControllerTypeUSB;//2; //2:USB
	  tempStorageDriveStruct.u32DefinedIndex = u32DefinedIndex;
	  tempStorageDriveStruct.strDefinedName = USBIter->c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_NODE_INFO, &Infolist);
	  tempStorageDriveStruct.strDeviceNodeName = Infolist.front().c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_SN_INFO, &Infolist);
	  tempStorageDriveStruct.strSerialNumber = Infolist.front().c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_VENDOR_INFO, &Infolist);
	  tempStorageDriveStruct.strVendorName = Infolist.front().c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_CAPACITY_INFO, &Infolist);
	  tempStorageDriveStruct.u64CapacityMB = atoi(Infolist.front().c_str());
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_SPEED_INFO, &Infolist);
	  tempStorageDriveStruct.strLinkSpeed = Infolist.front().c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_LINK_INFO, &Infolist);
	  tempStorageDriveStruct.strUSBVersion = Infolist.front().c_str();
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_PHYSICAL_BLOCK_SIZE_INFO, &Infolist);
	  tempStorageDriveStruct.u32PhysicalBlockSize = atoi(Infolist.front().c_str());
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_LOGICAL_BLOCK_SIZE_INFO, &Infolist);
	  tempStorageDriveStruct.u32LogicalBlockSize = atoi(Infolist.front().c_str());
	  pErrorMsg = m_pUSBDDAPI->getElementByIndex(u32DefinedIndex, USB_DATA_SG_NODE_INFO, &Infolist);
	  tempStorageDriveStruct.strSGDeviceNodeName = Infolist.front().c_str();

	  pStorageDriveList->push_back(tempStorageDriveStruct);

	  u32DefinedIndex++;
     }

     return returnErrorMsg(errorCode, NULL, NULL);
}

ErrorMsg *StorageDDAPIOperation::listDrive(void)
{
     ERRORCODE errorCode = NO_ERROR;

     list <StorageDriveStruct> *pStorageDriveList = &m_storageDriveList;
     for(list <StorageDriveStruct>::iterator listIterator = pStorageDriveList->begin(); listIterator != pStorageDriveList->end() ; listIterator++)
     {
	  if(listIterator->strDeviceNodeName != "")
	  {
	       cout << listIterator->strDefinedName << endl;
	  }
     }

     return returnErrorMsg(errorCode, NULL, NULL);
}

/// @fn         ErrorMsg *StorageDDAPI::getIndexByName(string in_strDefinedName, UINT32 *out_u32pDefinedIndex)
/// @brief      Get defined index by defined name
/// @param      in_strDefinedName is defined name
/// @param      *out_u32pDefinedIndex is a pointer for returned defined index
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg *StorageDDAPIOperation::getIndexByName(string in_strDefinedName, UINT32 *out_u32pDefinedIndex)
{
     ERRORCODE errorCode = ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND;

     for(list <StorageDriveStruct>::iterator storageDriveListIterator = m_storageDriveList.begin(); storageDriveListIterator != m_storageDriveList.end(); storageDriveListIterator++)
     {
	  if(convertStringToUpperCase(storageDriveListIterator->strDefinedName) == convertStringToUpperCase(in_strDefinedName))
	  {
	       errorCode = NO_ERROR;
	       *out_u32pDefinedIndex = storageDriveListIterator->u32DefinedIndex;
	       return returnErrorMsg(errorCode, NULL, NULL);
	  }
     }

     return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_DEFINED_INDEX_BY_NAME | errorCode), NULL, NULL);
}

ErrorMsg *StorageDDAPIOperation::getNameByIndex(UINT32 in_u32DefinedIndex, string *out_strDefinedName)
{
     ERRORCODE errorCode = ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND;

     for(list <StorageDriveStruct>::iterator storageDriveListIterator = m_storageDriveList.begin(); storageDriveListIterator != m_storageDriveList.end(); storageDriveListIterator++)
     {
	  if(storageDriveListIterator->u32DefinedIndex == in_u32DefinedIndex)
	  {
	       errorCode = NO_ERROR;
	       *out_strDefinedName = storageDriveListIterator->strDefinedName;
	       return returnErrorMsg(errorCode, NULL, NULL);
	  }
     }

     return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_DEFINED_INDEX_BY_NAME | errorCode), NULL, NULL);
}

/// @fn         ErrorMsg *StorageDDAPI::getDataByID(UINT32 in_u32DefinedIndex, StorageDataElement in_eStorageDataElement, string *out_strpReturnData)
/// @brief      Get data by definedIndex
/// @param      in_u32DefinedIndex is defined index for Storage device
/// @param      in_eStorageDataElement is which data should be returned
/// @param      *out_strpReturnData is a string pointer for returned data
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg *StorageDDAPIOperation::getElementDataByIndex(UINT32 in_u32DefinedIndex, StorageDataElement in_eStorageDataElement, string *out_strpReturnData)
{
     ERRORCODE errorCode = ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;

     for(list <StorageDriveStruct>::iterator storageDriveListIterator = m_storageDriveList.begin(); storageDriveListIterator != m_storageDriveList.end(); storageDriveListIterator++)
     {
	  if(storageDriveListIterator->u32DefinedIndex == in_u32DefinedIndex)
	  {
	       errorCode = NO_ERROR;
	       stringstream ssTemp;
	       ssTemp.str("");
	       ssTemp.clear();
	       string strTempData = "";

	       switch(storageDriveListIterator->u32ControllerType)
	       {
	       case eStorageControllerTypeSAS://0: //SAS
		    switch(in_eStorageDataElement)
		    {
		    case eStorageDriveDataAll:
			 ssTemp << "Device Node Name    : " << storageDriveListIterator->strDeviceNodeName << endl;
			 ssTemp << "SG Device Node Name : " << storageDriveListIterator->strSGDeviceNodeName << endl;
			 ssTemp << "Vendor              : " << storageDriveListIterator->strVendorName << endl;
			 ssTemp << "Model Name          : " << storageDriveListIterator->strModelName << endl;
			 ssTemp << "Serial Number       : " << storageDriveListIterator->strSerialNumber << endl;
			 ssTemp << "Capacity            : " << storageDriveListIterator->u64CapacityMB << "MB" << endl;
			 ssTemp << "SAS Address         : " << UINT64ToHexStr(storageDriveListIterator->u64SASAddress) << endl;
			 ssTemp << "Logical Block Size  : " << storageDriveListIterator->u32LogicalBlockSize << "Bytes" << endl;
			 ssTemp << "Physical Block Size : " << storageDriveListIterator->u32PhysicalBlockSize << "Bytes" << endl;
			 ssTemp << "Firmware Version    : " << storageDriveListIterator->strFirmwareVersion << endl;
			 ssTemp << "Link Speed          : " << storageDriveListIterator->strLinkSpeed << endl;
			 ssTemp << "Rotation Rate       : " << storageDriveListIterator->strRotationRate << endl;
			 ssTemp << "Form Factor         : " << storageDriveListIterator->strFormFactor << endl;
			 ssTemp << "Transport Protocol  : " << storageDriveListIterator->strProtocalType << endl;
			 ssTemp << "Temperature         : " << storageDriveListIterator->strTemperature;

			 *out_strpReturnData = "";
			 *out_strpReturnData = ssTemp.str();
			 break;
		    case eStorageDriveDeviceNodeName:
			 *out_strpReturnData = storageDriveListIterator->strDeviceNodeName;
			 break;
		    case eStorageDriveSGDeviceNodeName:
			 *out_strpReturnData = storageDriveListIterator->strSGDeviceNodeName;
			 break;
		    case eStorageDriveVendorName:
			 *out_strpReturnData = storageDriveListIterator->strVendorName;
			 break;
		    case eStorageDriveSerialNumber:
			 *out_strpReturnData = storageDriveListIterator->strSerialNumber;
			 break;
		    case eStorageDriveModelName:
			 *out_strpReturnData = storageDriveListIterator->strModelName;
			 break;
		    case eStorageDriveFirmwareVersion:
			 *out_strpReturnData = storageDriveListIterator->strFirmwareVersion;
			 break;
		    case eStorageDriveCapacity:
			 ssTemp.clear();
			 ssTemp << storageDriveListIterator->u64CapacityMB;
			 *out_strpReturnData = ssTemp.str();
//                            cout << "get capacity: " << out_strpReturnData << endl;
			 break;
		    case eStorageDriveLinkSpeed:
			 *out_strpReturnData = storageDriveListIterator->strLinkSpeed;
			 break;
		    case eStorageDriveTemperature:
			 *out_strpReturnData = storageDriveListIterator->strTemperature;
			 break;
		    case eStorageDriveRotationRate:
			 *out_strpReturnData = storageDriveListIterator->strRotationRate;
			 break;
		    case eStorageDriveFormFactor:
			 *out_strpReturnData = storageDriveListIterator->strFormFactor;
			 break;
		    case eStorageDriveProtocolType:
			 *out_strpReturnData = storageDriveListIterator->strProtocalType;
			 break;
		    case eStorageDriveLogicalBlockSize:
			 *out_strpReturnData = Int2Str(storageDriveListIterator->u32LogicalBlockSize);
			 break;
		    case eStorageDrivePhysicalBlockSize:
			 *out_strpReturnData = Int2Str(storageDriveListIterator->u32PhysicalBlockSize);
			 break;
		    case eStorageDriveSASAddress:
			 *out_strpReturnData = UINT64ToHexStr(storageDriveListIterator->u64SASAddress);
			 break;
		    default:
			 errorCode = ERROR_STORAGE_DDAPI_NO_SUCH_DATA_ELEMENT;
			 return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX | errorCode), NULL, NULL);
			 break;
		    }
		    break;

	       case eStorageControllerTypeNVME://3: //NVME
		    switch(in_eStorageDataElement)
		    {
		    case eStorageDriveDataAll:
			 ssTemp << "Device Node Name    : " << storageDriveListIterator->strDeviceNodeName << endl;
			 ssTemp << "SG Device Node Name : " << storageDriveListIterator->strSGDeviceNodeName << endl;
			 ssTemp << "Capacity            : " << storageDriveListIterator->u64CapacityMB << "MB" << endl;
			 ssTemp << "Logical Block Size  : " << storageDriveListIterator->u32LogicalBlockSize << "Bytes" << endl;
			 ssTemp << "Physical Block Size : " << storageDriveListIterator->u32PhysicalBlockSize << "Bytes";

			 *out_strpReturnData = "";
			 *out_strpReturnData = ssTemp.str();
			 break;
		    case eStorageDriveDeviceNodeName:
			 *out_strpReturnData = storageDriveListIterator->strDeviceNodeName;
			 break;
		    case eStorageDriveSGDeviceNodeName:
			 *out_strpReturnData = storageDriveListIterator->strSGDeviceNodeName;
			 break;
		    case eStorageDriveCapacity:
			 ssTemp.clear();
			 ssTemp << storageDriveListIterator->u64CapacityMB;
			 *out_strpReturnData = ssTemp.str();
			 break;
		    case eStorageDriveLogicalBlockSize:
			 *out_strpReturnData = Int2Str(storageDriveListIterator->u32LogicalBlockSize);
			 break;
		    case eStorageDrivePhysicalBlockSize:
			 *out_strpReturnData = Int2Str(storageDriveListIterator->u32PhysicalBlockSize);
			 break;
		    default:
			 errorCode = ERROR_STORAGE_DDAPI_NO_SUCH_DATA_ELEMENT;
			 return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX | errorCode), NULL, NULL);
			 break;
		    }
		    break;

	       case eStorageControllerTypeUSB://2: //USB
		    switch(in_eStorageDataElement)
		    {
		    case eStorageDriveDataAll:
			 ssTemp << "Device Node Name    : " << storageDriveListIterator->strDeviceNodeName << endl;
			 ssTemp << "SG Device Node Name : " << storageDriveListIterator->strSGDeviceNodeName << endl;
			 ssTemp << "Vendor              : " << storageDriveListIterator->strVendorName << endl;
			 ssTemp << "Serial Number       : " << storageDriveListIterator->strSerialNumber	<< endl;
			 ssTemp << "Capacity            : " << storageDriveListIterator->u64CapacityMB << "MB"<< endl;
			 ssTemp << "Link Speed          : " << storageDriveListIterator->strLinkSpeed << "Mb/s"<< endl;
			 ssTemp << "USB Version         : " << storageDriveListIterator->strUSBVersion << endl;
			 ssTemp << "Physical Block Size : " << storageDriveListIterator->u32PhysicalBlockSize << "Bytes"<< endl;
			 ssTemp << "Logical Block Size  : " << storageDriveListIterator->u32LogicalBlockSize << "Bytes";


			 *out_strpReturnData = ssTemp.str();
			 break;
		    case eStorageDriveVendorName:
			 *out_strpReturnData = storageDriveListIterator->strVendorName;
		      break;
		 case eStorageDriveDeviceNodeName:
		      *out_strpReturnData = storageDriveListIterator->strDeviceNodeName;
		      break;
		 case eStorageDriveSerialNumber:
		      *out_strpReturnData = storageDriveListIterator->strSerialNumber;
		      break;
		 case eStorageDriveCapacity:
		      ssTemp.clear();
		      ssTemp << storageDriveListIterator->u64CapacityMB << "MB";
		      *out_strpReturnData = ssTemp.str();
		      break;
		 case eStorageDriveLinkSpeed:
		      ssTemp.clear();
		      ssTemp << storageDriveListIterator->strLinkSpeed << "Mb/s";
		      *out_strpReturnData = ssTemp.str();
		      break;
		 case eStorageDriveUSBVersion:
		      *out_strpReturnData = storageDriveListIterator->strUSBVersion;
		      break;
		 case eStorageDrivePhysicalBlockSize:
		      ssTemp.clear();
		      ssTemp << storageDriveListIterator->u32PhysicalBlockSize << "Bytes";
		      *out_strpReturnData = ssTemp.str();
		      break;
		 case eStorageDriveLogicalBlockSize:
		      ssTemp.clear();
		      ssTemp << storageDriveListIterator->u32LogicalBlockSize << "Bytes";
		      *out_strpReturnData = ssTemp.str();
		      break;
		 case eStorageDriveSGDeviceNodeName:
		      *out_strpReturnData = storageDriveListIterator->strSGDeviceNodeName;
		      break;
		 default:
		      errorCode = ERROR_STORAGE_DDAPI_NO_SUCH_DATA_ELEMENT;
		      return returnErrorMsg(
			   (ERRORCODE) (ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX
					| errorCode), NULL, NULL);
		      break;
		 }
		 break;

	    default:
		 errorCode = ERROR_STORAGE_DDAPI_NO_SUCH_CONTROLLER_TYPE;
		 return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX | errorCode), NULL, NULL);
		 break;
            }
            return returnErrorMsg(errorCode, NULL, pErrorMsg);
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX | errorCode), NULL, NULL);
}

/// @fn         ErrorMsg *StorageDDAPI::getDataByName(string in_strDefinedName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData)
/// @brief      Get data by defined name
/// @param      in_strDefinedName is defined name for Storage device
/// @param      in_eStorageDataElement is which data should be returned
/// @param      *out_strpReturnData is a string pointer for returned data
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg *StorageDDAPIOperation::getElementDataByName(string in_strDefinedName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData)
{
    ERRORCODE errorCode = NO_ERROR;
    ErrorMsg *pErrorMsg = NO_ERROR_MSG;
    UINT32 u32DefinedIndex = 0;

    if((pErrorMsg = getIndexByName(in_strDefinedName, &u32DefinedIndex)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_NAME | errorCode), NULL, pErrorMsg);
    }

    if((pErrorMsg = getElementDataByIndex(u32DefinedIndex, in_eStorageDataElement, out_strpReturnData)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_NAME | errorCode), NULL, pErrorMsg);
    }

    return returnErrorMsg(errorCode, NULL, NULL);
}

ErrorMsg *StorageDDAPIOperation::getSMARTByIndex(UINT32 in_u32DefinedIndex, string *out_strpReturnData)
{
     ERRORCODE errorCode = ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;

     for(list <StorageDriveStruct>::iterator storageDriveListIterator = m_storageDriveList.begin(); storageDriveListIterator != m_storageDriveList.end(); storageDriveListIterator++)
     {
	  if(storageDriveListIterator->u32DefinedIndex == in_u32DefinedIndex)
	  {
	       errorCode = NO_ERROR;
	       
	       UINT32 u32ControllerType = 0;
	       if((pErrorMsg = getControllerTypeByIndex(in_u32DefinedIndex, &u32ControllerType)) != NO_ERROR_MSG)
	       {
		    return returnErrorMsg(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX, NULL, pErrorMsg);		    
	       }

	       string strSGDeviceNodeName = "";
	       INT32 s32FileDescriptor = 0;
	       string strSMARTHealthStatus = "";
	       SCSI_ERROR_LOG SCSIErrorLog;
	       stringstream ssTemp;
	       ssTemp.clear();
	       ssTemp.str("");

	       switch(u32ControllerType)
	       {
	       case eStorageControllerTypeSAS:
		    if((pErrorMsg = getElementDataByIndex(in_u32DefinedIndex, eStorageDriveSGDeviceNodeName, &strSGDeviceNodeName)) != NO_ERROR_MSG)
		    {
			 return returnErrorMsg(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX, NULL, pErrorMsg);
		    }

		    s32FileDescriptor = open(strSGDeviceNodeName.c_str(), O_RDWR | O_SYNC | O_NONBLOCK);
		    if(s32FileDescriptor < 0)
		    {
			 errorCode = ERROR_STORAGE_DDAPI_OPEN_DEVICE_FAIL;
			 return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX | errorCode), NULL, NULL);
		    }

		    if((pErrorMsg = m_pSCSIFactory->getSmartHealthStatus(s32FileDescriptor, &strSMARTHealthStatus)) != NO_ERROR_MSG)
		    {
			 return returnErrorMsg(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX, NULL, pErrorMsg);
		    }

		    if((pErrorMsg = m_pSCSIFactory->getSCSIErrorLog(s32FileDescriptor, &SCSIErrorLog)) != NO_ERROR_MSG)
		    {
			 return returnErrorMsg(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX, NULL, pErrorMsg);
		    }

		    if(close(s32FileDescriptor) < 0)
		    {
			 errorCode = ERROR_STORAGE_DDAPI_CLOSE_DEVICE_FAIL;
			 return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX | errorCode), NULL, NULL);
		    }

		    storageDriveListIterator->SASSMARTData.strHealthStatus = strSMARTHealthStatus;
		    storageDriveListIterator->SASSMARTData.errorLog = SCSIErrorLog;

		    ssTemp.clear();
		    ssTemp.str("");

		    ssTemp << "SMART Health Status   : " << strSMARTHealthStatus << "\n";
		    ssTemp << "Non-medium error count: " << SCSIErrorLog.nmErrorCount << "\n";
		    ssTemp << "=========================================\n";
		    ssTemp << "Read error count :" << SCSIErrorLog.read.fastECC << "\n";
		    ssTemp << "Fast ECC              : " << SCSIErrorLog.read.fastECC << "\n";
		    ssTemp << "Delayed ECC           : " << SCSIErrorLog.read.delayedECC << "\n";
		    ssTemp << "ReReadWrites          : " << SCSIErrorLog.read.reReadWrite << "\n";
		    ssTemp << "Corrected Error       : " << SCSIErrorLog.read.totalCorrected << "\n";
		    ssTemp << "Uncorrected Error     : " << SCSIErrorLog.read.totalUnCorrected << "\n";
		    ssTemp << "Algorithm Invocation  : " << SCSIErrorLog.read.algoInvocation << "\n";
		    ssTemp << "=========================================\n";
		    ssTemp << "Write error count :" << "\n";
		    ssTemp << "fast ECC              : " << SCSIErrorLog.write.fastECC << "\n";
		    ssTemp << "delayed ECC           : " << SCSIErrorLog.write.delayedECC << "\n";
		    ssTemp << "reReadWrites          : " << SCSIErrorLog.write.reReadWrite << "\n";
		    ssTemp << "corrected Error       : " << SCSIErrorLog.write.totalCorrected << "\n";
		    ssTemp << "uncorrected Error     : " << SCSIErrorLog.write.totalUnCorrected << "\n";
		    ssTemp << "algorithm Invocation  : " << SCSIErrorLog.write.algoInvocation << "\n";
		    ssTemp << "=========================================" << "\n";
		    ssTemp << "Verify error count :" << "\n";
		    ssTemp << "fast ECC              : " << SCSIErrorLog.verify.fastECC << "\n";
		    ssTemp << "delayed ECC           : " << SCSIErrorLog.verify.delayedECC << "\n";
		    ssTemp << "reReadWrites          : " << SCSIErrorLog.verify.reReadWrite << "\n";
		    ssTemp << "corrected Error       : " << SCSIErrorLog.verify.totalCorrected << "\n";
		    ssTemp << "uncorrected Error     : " << SCSIErrorLog.verify.totalUnCorrected << "\n";
		    ssTemp << "algorithm Invocation  : " << SCSIErrorLog.verify.algoInvocation;

		    *out_strpReturnData = ssTemp.str();
		    break;
	       default:
		    *out_strpReturnData = "";
		    break;
	       }
	       return returnErrorMsg(errorCode, NULL, NULL);
	  }
     }
     return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX | errorCode), NULL, NULL);
}

ErrorMsg *StorageDDAPIOperation::getControllerTypeByIndex(UINT32 in_u32DefinedIndex, UINT32 *out_u32pControllerType)
{
     ERRORCODE errorCode = ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND;

     for(list <StorageDriveStruct>::iterator storageDriveListIterator = m_storageDriveList.begin(); storageDriveListIterator != m_storageDriveList.end(); storageDriveListIterator++)
     {
	  if(storageDriveListIterator->u32DefinedIndex == in_u32DefinedIndex)
	  {
	       errorCode = NO_ERROR;
	       *out_u32pControllerType = storageDriveListIterator->u32ControllerType;
	       return returnErrorMsg(errorCode, NULL, NULL);
	  }
     }

     return returnErrorMsg((ERRORCODE)(ERROR_STORAGE_DDAPI_GET_DEFINED_INDEX_BY_NAME | errorCode), NULL, NULL);
}

/// @fn         StorageDDAPIOperationI::convertStringToUpperCase(string in_strInputString, string *out_strpOutputString)
/// @brief      convert string to upper case
/// @param      in_strInputString is input string
/// @exception  Nothing
/// @return     returned output string
string StorageDDAPIOperation::convertStringToUpperCase(string in_strInputString)
{
    string strOutputString = "";
    for(string::iterator stringIterator = in_strInputString.begin(); stringIterator != in_strInputString.end(); stringIterator++)
    {
        strOutputString += toupper(*stringIterator);
    }
    return strOutputString;
}
